"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$visualequal = exports.$unorderedEqual = exports.$setequal = exports.$mapequal = exports.$arrayequal = exports.$objectsequal = exports.$equal = exports.$bytesequal = exports.$visualorder = exports.$visualcompare = exports.$order = exports.$compare = exports.$bytesorder = exports.$bytescompare = exports.$dateorder = exports.$arraycompare = exports.$datecompare = exports.$numorder = exports.$numcompare = void 0;
const commons_1 = require("./commons");
const tsdate_1 = require("./tsdate");
const types_1 = require("./types");
const strings_1 = require("./strings");
const tserrors_1 = require("./tserrors");
function $numcompare(a, b) {
    if (isNaN(a) || isNaN(b)) {
        return undefined;
    }
    return a === b ? types_1.Same : (a < b ? types_1.Ascending : types_1.Descending);
}
exports.$numcompare = $numcompare;
function $numorder(a, b) {
    if (a === b) {
        return types_1.Same;
    }
    return isNaN(a) ? (isNaN(b) ? types_1.Same : types_1.Ascending) : (isNaN(b) ? types_1.Descending : $numcompare(a, b));
}
exports.$numorder = $numorder;
function $datecompare(a, b) {
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return undefined;
    }
    if (a === b) {
        return types_1.Same;
    }
    if (typeof a === 'number' && typeof b === 'number') {
        return $numcompare(a, b);
    }
    if (a instanceof Date) {
        return _heterogenDateCompare(a, b);
    }
    if (b instanceof Date) {
        return _inverseComparison(_heterogenDateCompare(b, a));
    }
    if ((0, commons_1.$isstring)(a)) {
        a = new tsdate_1.TSDate(a);
    }
    if (a instanceof tsdate_1.TSDate) {
        return _heterogenTimestampCompare(a.timestamp, 0, b);
    }
    if ((0, commons_1.$isstring)(b)) {
        b = new tsdate_1.TSDate(b);
    }
    if (b instanceof tsdate_1.TSDate) {
        return _inverseComparison(_heterogenTimestampCompare(b.timestamp, 0, a));
    }
    return undefined;
}
exports.$datecompare = $datecompare;
function $arraycompare(a, b) {
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return undefined;
    }
    if (a === b) {
        return types_1.Same;
    }
    const na = a.length, nb = b.length;
    let i = 0;
    while (i < na && i < nb) {
        const c = $compare(a[i], b[i]);
        if (c !== types_1.Same) {
            return c;
        }
        i++;
    }
    return na === nb ? types_1.Same : (i < na ? types_1.Descending : types_1.Ascending);
}
exports.$arraycompare = $arraycompare;
function $dateorder(a, b) { return (0, commons_1.$ok)(a) ? ((0, commons_1.$ok)(b) ? $datecompare(a, b) : types_1.Descending) : ((0, commons_1.$ok)(b) ? types_1.Ascending : types_1.Same); }
exports.$dateorder = $dateorder;
function $bytescompare(a, b) {
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return undefined;
    }
    if (a === b) {
        return types_1.Same;
    }
    const na = a.length, nb = b.length;
    let i = 0;
    while (i < na && i < nb) {
        const c = $numcompare(a[i], b[i]);
        if (c !== types_1.Same) {
            return c;
        }
        i++;
    }
    return na === nb ? types_1.Same : (i < na ? types_1.Descending : types_1.Ascending);
}
exports.$bytescompare = $bytescompare;
function $bytesorder(a, b) { return (0, commons_1.$ok)(a) ? ((0, commons_1.$ok)(b) ? $bytescompare(a, b) : types_1.Descending) : ((0, commons_1.$ok)(b) ? types_1.Ascending : types_1.Same); }
exports.$bytesorder = $bytesorder;
function $compare(a, b) {
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return undefined;
    }
    if (a === b) {
        return types_1.Same;
    }
    if (typeof a === 'number' && typeof b === 'number') {
        return $numcompare(a, b);
    }
    if (a instanceof URL && b instanceof URL) {
        a = a.href;
        b = b.href;
    }
    if ((0, commons_1.$isstring)(a) && (0, commons_1.$isstring)(b)) {
        return a > b ? types_1.Descending : (a < b ? types_1.Ascending : types_1.Same);
    }
    if ((a instanceof Date || a instanceof tsdate_1.TSDate) && (b instanceof Date || b instanceof tsdate_1.TSDate)) {
        return $datecompare(a, b);
    }
    if (a instanceof Buffer && b instanceof Buffer) {
        return Buffer.compare(a, b);
    }
    if (a instanceof Uint8Array && b instanceof Uint8Array) {
        return $bytescompare(a, b);
    }
    if ((a instanceof ArrayBuffer || ArrayBuffer.isView(a)) && (b instanceof ArrayBuffer || ArrayBuffer.isView(b))) {
        return $bytescompare(new Uint8Array(a), new Uint8Array(b));
    }
    return (0, commons_1.$ismethod)(a, 'compare') ? a.compare(b) : undefined;
}
exports.$compare = $compare;
function $order(a, b) {
    if (a === b) {
        return types_1.Same;
    }
    if (typeof a === 'number' && typeof b === 'number') {
        return $numorder(a, b);
    }
    return _order(a, b, $compare);
}
exports.$order = $order;
function $visualcompare(a, b) {
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return undefined;
    }
    if (a === b) {
        return types_1.Same;
    }
    if (a instanceof URL && b instanceof URL) {
        a = a.href;
        b = b.href;
    }
    if (!(0, commons_1.$isstring)(a)) {
        a = (0, commons_1.$string)(a);
    }
    if (!(0, commons_1.$isstring)(b)) {
        b = (0, commons_1.$string)(b);
    }
    if (a === b) {
        return types_1.Same;
    }
    a = (0, strings_1.$ascii)((0, strings_1.$normspaces)(a)).toUpperCase();
    b = (0, strings_1.$ascii)((0, strings_1.$normspaces)(b)).toUpperCase();
    return a > b ? types_1.Descending : (a < b ? types_1.Ascending : types_1.Same);
}
exports.$visualcompare = $visualcompare;
function $visualorder(a, b) { return a === b ? types_1.Same : _order(a, b, $visualcompare); }
exports.$visualorder = $visualorder;
function $bytesequal(a, b) {
    if (a === b) {
        return true;
    }
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b))
        return false;
    const n = a.length;
    if (n !== b.length)
        return false;
    for (let i = 0; i < n; i++) {
        if (a[i] !== b[i])
            return false;
    }
    return true;
}
exports.$bytesequal = $bytesequal;
function $equal(a, b) {
    if (a === b) {
        return true;
    }
    if (typeof a === 'number' && typeof b === 'number') {
        return a === b;
    }
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b))
        return false;
    if (a instanceof URL && b instanceof URL) {
        return a.href === b.href;
    }
    if ((0, commons_1.$ismethod)(a, 'isEqual')) {
        return a.isEqual(b);
    }
    if ((0, commons_1.$ismethod)(b, 'isEqual')) {
        return b.isEqual(a);
    }
    if (a instanceof Buffer && b instanceof Buffer) {
        return Buffer.compare(a, b) === 0;
    }
    if (a instanceof Uint8Array && b instanceof Uint8Array) {
        return $bytesequal(a, b);
    }
    if ((a instanceof ArrayBuffer || ArrayBuffer.isView(a)) && (b instanceof ArrayBuffer || ArrayBuffer.isView(b))) {
        return $bytesequal(new Uint8Array(a), new Uint8Array(b));
    }
    return $objectsequal(a, b);
}
exports.$equal = $equal;
function $objectsequal(a, b) {
    if (a === b) {
        return true;
    }
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b))
        return false;
    if (Object.getPrototypeOf(a) === Object.prototype && Object.getPrototypeOf(b) === Object.prototype) {
        const ak = Object.getOwnPropertyNames(a);
        const bk = Object.getOwnPropertyNames(b);
        const keys = ak.length >= bk.length ? ak : bk;
        for (let k of keys) {
            if (!$equal(a[k], b[k]))
                return false;
        }
        return true;
    }
    return false;
}
exports.$objectsequal = $objectsequal;
function $arrayequal(a, b) {
    if (a === b) {
        return true;
    }
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return false;
    }
    const n = a.length;
    if (n !== b.length)
        return false;
    for (let i = 0; i < n; i++) {
        if (!$equal(a[i], b[i]))
            return false;
    }
    return true;
}
exports.$arrayequal = $arrayequal;
function $mapequal(a, b) {
    if (a === b) {
        return true;
    }
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return false;
    }
    const ak = a.keys();
    const bk = b.keys();
    const keys = a.size >= b.size ? ak : bk;
    for (let k of keys) {
        if (!$equal(a.get(k), b.get(k)))
            return false;
    }
    return true;
}
exports.$mapequal = $mapequal;
function $setequal(sa, sb) {
    if (sa === sb) {
        return true;
    }
    return (0, commons_1.$ok)(sa) && (0, commons_1.$ok)(sb) && sa.size === sb.size && sa.toArray().every(e => sb.has(e));
}
exports.$setequal = $setequal;
function $unorderedEqual(sa, sb) {
    if (sa === sb) {
        return true;
    }
    if (!(0, commons_1.$ok)(sa) || !(0, commons_1.$ok)(sb)) {
        return false;
    }
    let a = undefined;
    let b = undefined;
    let na = -1;
    let nb = -2;
    if ((0, commons_1.$isarray)(sa)) {
        na = (0, commons_1.$count)(sa);
        a = new Set(sa);
    }
    else if (sa instanceof Set) {
        na = sa.size;
        a = sa;
    }
    if ((0, commons_1.$isarray)(sb)) {
        nb = (0, commons_1.$count)(sb);
        b = new Set(sb);
    }
    else if (sb instanceof Set) {
        nb = sb.size;
        b = sb;
    }
    return na === nb && $setequal(a, b);
}
exports.$unorderedEqual = $unorderedEqual;
function $visualequal(a, b) {
    if (a === b) {
        return true;
    }
    if (!(0, commons_1.$ok)(a) || !(0, commons_1.$ok)(b)) {
        return false;
    }
    if (!(0, commons_1.$isstring)(a)) {
        a = (0, commons_1.$string)(a);
    }
    if (!(0, commons_1.$isstring)(b)) {
        b = (0, commons_1.$string)(b);
    }
    if (a === b) {
        return true;
    }
    return (0, strings_1.$ascii)((0, strings_1.$normspaces)(a)).toUpperCase() === (0, strings_1.$ascii)((0, strings_1.$normspaces)(b)).toUpperCase();
}
exports.$visualequal = $visualequal;
function _order(a, b, comparisonFn) {
    if (!(0, commons_1.$ok)(a)) {
        return (0, commons_1.$ok)(b) ? types_1.Ascending : types_1.Same;
    }
    else if (!(0, commons_1.$ok)(b)) {
        return (0, commons_1.$ok)(a) ? types_1.Descending : types_1.Same;
    }
    else {
        const comp = comparisonFn(a, b);
        if (!(0, commons_1.$defined)(comp)) {
            tserrors_1.TSError.throw("Impossible to order elements A and B", { A: a, B: b });
        }
        return comp;
    }
}
function _heterogenDateCompare(a, b) {
    if (b instanceof Date) {
        return $numcompare(a.getTime(), b.getTime());
    }
    return _heterogenTimestampCompare(a.timeStamp, a.getMilliseconds(), b);
}
function _heterogenTimestampCompare(ats, mts, b) {
    let ts = NaN;
    let ms = 0;
    if (b instanceof tsdate_1.TSDate) {
        ts = b.timestamp;
    }
    else {
        const t = typeof b;
        if (t === 'number') {
            ts = Math.trunc(b);
            ms = b - ts;
        }
        else if (t === 'string') {
            ts = (new tsdate_1.TSDate(b)).timestamp;
        }
    }
    const c = $numcompare(ats, ts);
    return c === types_1.Same ? $numcompare(mts, ms) : c;
}
function _inverseComparison(c) { return c === types_1.Same || c === undefined ? c : (c === types_1.Ascending ? types_1.Descending : types_1.Ascending); }
//# sourceMappingURL=compare.js.map