"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports._insideHash512 = exports.$sha512 = exports.$sha384 = exports.$sha256partial = exports.$sha256 = exports.$sha224 = exports.$sha1 = exports.$md5 = exports.$password = exports.TS_MAX_PASSWORD_LENGTH = exports.$shuffle = exports.$random = exports.$uuidhashfile = exports.$uuidhash = exports.$hashfile = exports.$slowhash = exports.$hash = exports.$crc32 = exports.$crc16 = exports.$decrypt = exports.$encrypt = exports.$commonInitializationVectorCopy = exports.$setCommonItializationVector = exports.$uuidVersion = exports.$slowuuid = exports.$uuid = exports.AES256 = exports.AES128 = exports.SHA1 = exports.MD5 = exports.SHA512 = exports.SHA384 = exports.SHA256 = exports.SHA224 = void 0;
const fs_1 = require("fs");
const crypto_1 = require("crypto");
const types_1 = require("./types");
const commons_1 = require("./commons");
const data_1 = require("./data");
const tscharset_1 = require("./tscharset");
const tsdata_1 = require("./tsdata");
const utils_1 = require("./utils");
const strings_1 = require("./strings");
const crypto_tables_1 = require("./crypto_tables");
const tserrors_1 = require("./tserrors");
const crypto_hashBuffer_1 = require("./crypto_hashBuffer");
const object_1 = require("./object");
exports.SHA224 = 'SHA224';
exports.SHA256 = 'SHA256';
exports.SHA384 = 'SHA384';
exports.SHA512 = 'SHA512';
exports.MD5 = 'MD5';
exports.SHA1 = 'SHA1';
exports.AES128 = 'AES128';
exports.AES256 = 'AES256';
function $uuid(internalImplementation = false) {
    if (!internalImplementation && typeof crypto_1.randomUUID === 'function') {
        try {
            return crypto_1.randomUUID();
        }
        catch (_a) {
            (0, utils_1.$logterm)('Warning:crypto.randomUUID() is not available');
        }
    }
    return $slowuuid(true);
}
exports.$uuid = $uuid;
function $slowuuid(convertToLowerCase = false) {
    let uuid = "";
    const rbytes = _randomBytes(32);
    for (let i = 0; i < 32; i++) {
        if (i === 12) {
            uuid += '-4';
        }
        else {
            if (i === 8 || i === 16 || i === 20) {
                uuid += '-';
            }
            const rand = (rbytes[i] >> (i % 24)) & 0x0F;
            uuid += (i == 16 ? (rand & 3 | 8) : rand).toHex1(convertToLowerCase);
        }
    }
    return uuid;
}
exports.$slowuuid = $slowuuid;
function $uuidVersion(str) {
    const s = (0, strings_1.$ftrim)(str);
    if (!s.length) {
        return undefined;
    }
    if (commons_1.__uuidV4Regex.test(s)) {
        return types_1.UUIDv4;
    }
    if (commons_1.__uuidV1Regex.test(s)) {
        return types_1.UUIDv1;
    }
    return undefined;
}
exports.$uuidVersion = $uuidVersion;
const __TSNoSpecificIV = Buffer.from([3, 67, 0, 14, 2, 95, 191, 0, 217, 255, 7, 6, 1, 67, 13, 89]);
let __CommonInitializationVector = __TSNoSpecificIV;
function $setCommonItializationVector(d) { __CommonInitializationVector = (0, commons_1.$length)(d) === 16 ? (0, data_1.$bufferFromDataLike)(d) : __TSNoSpecificIV; }
exports.$setCommonItializationVector = $setCommonItializationVector;
function $commonInitializationVectorCopy() { return (0, data_1.$bufferFromBytes)(__CommonInitializationVector, { forceCopy: true }); }
exports.$commonInitializationVectorCopy = $commonInitializationVectorCopy;
function $encrypt(src, skey, opts) {
    if (typeof crypto_1.createCipheriv === 'undefined') {
        tserrors_1.TSError.throw(`$encrypt() : function createCipheriv() is not available in your system.`, { source: src, options: opts });
    }
    const [charset, key, algo] = _charsetKeyAndAlgo(skey, opts);
    if (!charset) {
        return null;
    }
    const source = (0, commons_1.$isstring)(src) ? charset.uint8ArrayFromString(src) : (0, data_1.$uint8ArrayFromDataLike)(src);
    if (!(0, commons_1.$length)(source)) {
        return null;
    }
    let returnValue = null;
    try {
        const addIV = !(opts === null || opts === void 0 ? void 0 : opts.noInitializationVector);
        const iv = addIV ? _randomBytes(16) : __CommonInitializationVector;
        const cipher = (0, crypto_1.createCipheriv)(algo, key, iv);
        let encrypted = addIV ? new tsdata_1.TSData(iv) : new tsdata_1.TSData();
        encrypted.appendBytes(cipher.update(source));
        encrypted.appendBytes(cipher.final());
        returnValue = !(opts === null || opts === void 0 ? void 0 : opts.dataOutput) ? encrypted.toString('hex') : encrypted;
    }
    catch (e) {
        console.log(e);
        returnValue = null;
    }
    return returnValue;
}
exports.$encrypt = $encrypt;
function $decrypt(source, skey, opts) {
    if (typeof crypto_1.createDecipheriv === 'undefined') {
        tserrors_1.TSError.throw(`$decrypt() : function createDecipheriv() is not available in your system.`, { source: source, options: opts });
    }
    const hasVector = !(opts === null || opts === void 0 ? void 0 : opts.noInitializationVector);
    const isString = (0, commons_1.$isstring)(source);
    const len = (0, commons_1.$length)(source);
    if (len % (isString ? 32 : 16) !== 0 || len < (hasVector ? (isString ? 64 : 32) : (isString ? 32 : 16))) {
        return null;
    }
    const [charset, key, algo] = _charsetKeyAndAlgo(skey, opts);
    if (!charset) {
        return null;
    }
    let returnValue = null;
    try {
        let src;
        let iv;
        if (isString) {
            src = Buffer.from(hasVector ? source.slice(32) : source, 'hex');
            iv = hasVector ? Buffer.from(source.slice(0, 32), 'hex') : __CommonInitializationVector;
        }
        else {
            src = (0, data_1.$bufferFromDataLike)(source, { start: hasVector ? 16 : 0 });
            iv = hasVector ? (0, data_1.$bufferFromDataLike)(source, { end: 16 }) : __CommonInitializationVector;
        }
        let decipher = (0, crypto_1.createDecipheriv)(algo, key, iv);
        let decrypted = new tsdata_1.TSData(decipher.update(src));
        decrypted.appendBytes(decipher.final());
        returnValue = !(opts === null || opts === void 0 ? void 0 : opts.dataOutput) ? decrypted.toString(charset) : decrypted;
    }
    catch (e) {
        returnValue = null;
    }
    return returnValue;
}
exports.$decrypt = $decrypt;
function $crc16(source, encoding) { return _crc(source, 0, crypto_tables_1.__TSCRC16ARCTable, 0xffff, encoding) >>> 0; }
exports.$crc16 = $crc16;
function $crc32(source, encoding) { return (_crc(source, 0 ^ -1, crypto_tables_1.__TSCRC32Table, 0x00ffffff, encoding) ^ -1) >>> 0; }
exports.$crc32 = $crc32;
function $hash(buf, method, encoding) {
    let ret = null;
    if (typeof crypto_1.createHash !== 'undefined') {
        try {
            const source = (0, commons_1.$isstring)(buf) ?
                (0, tscharset_1.$charset)(encoding, tscharset_1.TSCharset.binaryCharset()).uint8ArrayFromString(buf) :
                (0, data_1.$uint8ArrayFromDataLike)(buf);
            let hash = _createHash(method);
            hash.update(source);
            ret = hash.digest('hex');
        }
        catch (_a) {
            (0, utils_1.$logterm)(`Warning: crypto hash functions did fail for method '${(0, commons_1.$length)(method) ? method : 'SHA256'}'`);
            ret = null;
        }
    }
    if ((0, commons_1.$ok)(ret)) {
        return ret;
    }
    try {
        ret = $slowhash(buf, { method: method, encoding: encoding });
    }
    catch (_b) {
        (0, utils_1.$logterm)(`Warning: function $slowhash(buffer, { method:"${(0, commons_1.$length)(method) ? method : 'SHA256'}" }) did fail`);
        ret = null;
    }
    return ret;
}
exports.$hash = $hash;
function $slowhash(source, options = {}) {
    const buf = (0, commons_1.$isstring)(source) ?
        (0, tscharset_1.$charset)(options.encoding, tscharset_1.TSCharset.binaryCharset()).bufferFromString(source) :
        source;
    if (!(0, commons_1.$ok)(options.method) || options.method === exports.SHA256) {
        return $sha256(buf, options.separator, options.dataOutput);
    }
    else if (options.method === exports.SHA224) {
        return $sha224(buf, options.separator, options.dataOutput);
    }
    else if (options.method === exports.SHA384) {
        return $sha384(buf, options.separator, options.dataOutput);
    }
    else if (options.method === exports.SHA512) {
        return $sha512(buf, options.separator, options.dataOutput);
    }
    else if (options.method === exports.SHA1) {
        return $sha1(buf, options.separator, options.dataOutput);
    }
    else if (options.method === exports.MD5) {
        return $md5(buf, options.separator, options.dataOutput);
    }
    tserrors_1.TSError.throw(`$slowHash() : ${options.method} hashing method is unknown`, { source: source, options: options });
}
exports.$slowhash = $slowhash;
function $hashfile(filePath, method) {
    return __awaiter(this, void 0, void 0, function* () {
        tserrors_1.TSError.assertNotInBrowser('$hashfile');
        return new Promise((resolve, reject) => {
            let hash = _createHash(method);
            if (!(0, commons_1.$length)(filePath)) {
                return reject(null);
            }
            try {
                (0, fs_1.createReadStream)(filePath).on('data', data => hash.update(data)).on('end', () => resolve(hash.digest('hex')));
            }
            catch (e) {
                return reject(null);
            }
        });
    });
}
exports.$hashfile = $hashfile;
function $uuidhash(buf, version) { return _uuidHash($hash(buf, exports.MD5), version); }
exports.$uuidhash = $uuidhash;
function $uuidhashfile(filePath, version) {
    return __awaiter(this, void 0, void 0, function* () {
        tserrors_1.TSError.assertNotInBrowser('$uuidhashfile');
        return _uuidHash(yield $hashfile(filePath, exports.MD5), version);
    });
}
exports.$uuidhashfile = $uuidhashfile;
function $random(max) {
    let m = (0, commons_1.$unsigned)(max);
    if (!m) {
        m = types_1.UINT32_MAX;
    }
    if (typeof crypto_1.randomInt === 'function') {
        return (0, crypto_1.randomInt)(Math.min(m, types_1.UINT_MAX));
    }
    else if (typeof crypto_1.randomBytes === 'function') {
        const is32bits = m <= types_1.UINT32_MAX;
        return _randomFromBytes(m, (0, crypto_1.randomBytes)(is32bits ? 4 : 8), is32bits);
    }
    else if (typeof crypto_1.getRandomValues === 'function') {
        const is32bits = m <= types_1.UINT32_MAX;
        return _randomFromBytes(m, (0, crypto_1.getRandomValues)(Buffer.allocUnsafe(is32bits ? 4 : 8)), is32bits);
    }
    else {
        return Math.floor(Math.random() * Math.min(m, types_1.UINT_MAX));
    }
}
exports.$random = $random;
function $shuffle(values, max) {
    const ret = [];
    if ((0, commons_1.$ok)(values)) {
        const source = Array.from(values);
        let n = source.length;
        if (n > 0) {
            const m = Math.min(n, (0, commons_1.$tounsigned)(max, n));
            for (let i = 0; i < m; i++) {
                const index = $random(n);
                ret.push(source[index]);
                source.splice(index, 1);
                n--;
            }
        }
    }
    return ret;
}
exports.$shuffle = $shuffle;
;
exports.TS_MAX_PASSWORD_LENGTH = 64;
function $password(len, opts = {}) {
    if (!opts.usesLowercase && !opts.usesDigits && !opts.usesSpecials && !opts.usesUppercase) {
        opts.usesLowercase = true;
    }
    const minLen = Math.max((!opts.usesLowercase ? 0 : 1) + (!opts.usesUppercase ? 0 : 1) + (!opts.usesDigits ? 0 : 1) + (!opts.usesSpecials ? 0 : 1), 3);
    if (len < minLen) {
        tserrors_1.TSError.throw(`$password(): asked length is too short (${len}<${minLen}) for your gneration options`, Object.assign({ length: len, min: minLen, max: exports.TS_MAX_PASSWORD_LENGTH }, opts));
    }
    else if (len > exports.TS_MAX_PASSWORD_LENGTH) {
        tserrors_1.TSError.throw(`$password(): asked length is too long (${len}>${exports.TS_MAX_PASSWORD_LENGTH})`, Object.assign({ length: len, min: minLen, max: exports.TS_MAX_PASSWORD_LENGTH }, opts));
    }
    const pwd = [];
    const rbytes = _randomBytes(len);
    let i = 0;
    const base = [];
    function _randomchar(b, idx) { return b.charAt(idx % b.length); }
    function _makeBase(b) {
        base.push(b);
        pwd.push(_randomchar(b, rbytes[i++]));
    }
    if (!!opts.usesLowercase) {
        _makeBase('abcdefghijklmnopqrstuvwxyz');
    }
    if (!!opts.usesUppercase) {
        _makeBase('ABCDEFGHIJKLMNOPQRSTUVWXYZ');
    }
    if (!!opts.usesDigits) {
        _makeBase('01234567890123456789');
    }
    if (!!opts.usesSpecials) {
        _makeBase('!#$-_&*@()+/-=[]{}^:;,.');
    }
    while (i < len) {
        pwd.push(_randomchar(base[$random(base.length)], rbytes[i++]));
    }
    return $shuffle(pwd).join('');
}
exports.$password = $password;
String.prototype.crc16 = function crc16(encoding) { return $crc16(this, encoding); };
String.prototype.crc32 = function crc32(encoding) { return $crc32(this, encoding); };
String.prototype.hash = function hash(method, encoding) { return $hash(this, method, encoding); };
String.prototype.slowhash = function slowhash(options) { return $slowhash(this, options); };
Uint8Array.prototype.crc16 = function crc16() { return $crc16(this); };
Uint8Array.prototype.crc32 = function crc32() { return $crc32(this); };
Uint8Array.prototype.hash = function hash(method) { return $hash(this, method); };
Uint8Array.prototype.slowhash = function slowhash(options) { return $slowhash(this, options); };
Uint8Array.prototype.uuidhash = function uuidhash(version) { return $uuidhash(this, version); };
ArrayBuffer.prototype.crc16 = function crc16() { return $crc16(this); };
ArrayBuffer.prototype.crc32 = function crc32() { return $crc32(this); };
ArrayBuffer.prototype.hash = function hash(method) { return $hash(this, method); };
ArrayBuffer.prototype.slowhash = function slowhash(options) { return $slowhash(this, options); };
ArrayBuffer.prototype.uuidhash = function uuidhash(version) { return $uuidhash(this, version); };
(0, object_1.$declareMethod)(Array, {
    element: 'shuffle',
    implementation: function shuffle(max) { return $shuffle(this, max); }
});
function $md5(buf, separator, dataoutput) {
    let H = [0x67452301, 0xefcdab89, 0x98badcfe, 0x10325476];
    const HB = new crypto_hashBuffer_1._TSHashBuffer(buf, { integersCount: 16, separator: separator, dataoutput: dataoutput, endianness: 'LE' });
    const fn = [
        (a, b, c, d, m, k, n) => { return (_rol(n, (a + ((b & c) | ((~b) & d)) + m + k) | 0) + b) | 0; },
        (a, b, c, d, m, k, n) => { return (_rol(n, (a + ((b & d) | (c & (~d))) + m + k) | 0) + b) | 0; },
        (a, b, c, d, m, k, n) => { return (_rol(n, (a + (b ^ c ^ d) + m + k) | 0) + b) | 0; },
        (a, b, c, d, m, k, n) => { return (_rol(n, (a + ((c ^ (b | (~d)))) + m + k) | 0) + b) | 0; }
    ];
    HB.hash(M => {
        let h = [...H];
        crypto_tables_1._TSMD5Config.forEach(c => {
            h[c[0]] = fn[c[1]](h[c[2]], h[c[3]], h[c[4]], h[c[5]], M[c[6]], c[7], c[8]);
        });
        for (let i = 0; i < 4; i++) {
            H[i] = (H[i] + h[i]) >>> 0;
        }
    });
    return HB.output(H);
}
exports.$md5 = $md5;
function $sha1(buf, separator, dataoutput) {
    const K = [0x5a827999, 0x6ed9eba1, 0x8f1bbcdc, 0xca62c1d6];
    let H = [0x67452301, 0xEFCDAB89, 0x98BADCFE, 0x10325476, 0xC3D2E1F0];
    let W = new Array(80);
    const HB = new crypto_hashBuffer_1._TSHashBuffer(buf, { integersCount: 16, separator: separator, dataoutput: dataoutput });
    function _ft(s, b, c, d) {
        if (s === 0) {
            return _choice(b, c, d);
        }
        if (s === 2) {
            return _majority(b, c, d);
        }
        return b ^ c ^ d;
    }
    HB.hash(M => {
        let h = [...H];
        for (let i = 0; i < 16; i++) {
            W[i] = M[i];
        }
        for (let i = 16; i < 80; i++) {
            W[i] = _rol(1, W[i - 3] ^ W[i - 8] ^ W[i - 14] ^ W[i - 16]);
        }
        for (let i = 0; i < 80; ++i) {
            const s = Math.floor(i / 20);
            const T = _rol(5, h[0]) + _ft(s, h[1], h[2], h[3]) + h[4] + K[s] + W[i];
            h[4] = h[3];
            h[3] = h[2];
            h[2] = _rol(30, h[1]);
            h[1] = h[0];
            h[0] = T;
        }
        for (let i = 0; i < 5; i++) {
            H[i] = (H[i] + h[i]) >>> 0;
        }
    });
    return HB.output(H);
}
exports.$sha1 = $sha1;
function $sha224(buf, separator, dataoutput) {
    let H = [...crypto_tables_1._TSSHA224HTable];
    let W = new Array(64);
    const HB = new crypto_hashBuffer_1._TSHashBuffer(buf, { integersCount: 16, separator: separator, dataoutput: dataoutput });
    HB.hash(_insideHash256(H, W));
    return HB.output(H.slice(0, 7));
}
exports.$sha224 = $sha224;
function $sha256(buf, separator, dataoutput) {
    let H = [...crypto_tables_1._TSSHA256HTable];
    let W = new Array(64);
    const HB = new crypto_hashBuffer_1._TSHashBuffer(buf, { integersCount: 16, separator: separator, dataoutput: dataoutput });
    HB.hash(_insideHash256(H, W));
    return HB.output(H);
}
exports.$sha256 = $sha256;
;
function $sha256partial(buf, separator, dataoutput) {
    let H = [...crypto_tables_1._TSSHA256HTable];
    let W = new Array(64);
    const HB = new crypto_hashBuffer_1._TSHashBuffer(buf, { integersCount: 16, separator: separator, dataoutput: dataoutput });
    HB.partialHash(_insideHash256(H, W));
    return {
        partial: HB.output(H),
        calculatedBlocks: HB.blocksCount - 1,
        lastBlock: HB.lastPartialBlock()
    };
}
exports.$sha256partial = $sha256partial;
function $sha384(buf, separator, dataoutput) {
    let H = [...crypto_tables_1._TSSHA384HTable];
    let W = new Array(160);
    const HB = new crypto_hashBuffer_1._TSHashBuffer(buf, { integersCount: 32, separator: separator, dataoutput: dataoutput });
    HB.hash(_insideHash512(H, W));
    return HB.output(H.slice(0, 12));
}
exports.$sha384 = $sha384;
function $sha512(buf, separator, dataoutput) {
    let H = [...crypto_tables_1._TSSHA512HTable];
    let W = new Array(160);
    const HB = new crypto_hashBuffer_1._TSHashBuffer(buf, { integersCount: 32, separator: separator, dataoutput: dataoutput });
    HB.hash(_insideHash512(H, W));
    return HB.output(H);
}
exports.$sha512 = $sha512;
const __TSEncryptKeyLength = {
    'AES128': 16,
    'AES256': 32
};
const __TSEncryptAlgoRef = {
    'AES128': 'aes-128-cbc',
    'AES256': 'aes-256-cbc'
};
const __TSHashMethodRef = {
    'SHA224': 'sha224',
    'SHA256': 'sha256',
    'SHA384': 'sha384',
    'SHA512': 'sha512',
    'MD5': 'md5',
    'SHA1': 'sha1'
};
function _uuidHash(h, version) {
    if ((0, commons_1.$length)(h) === 32) {
        const v = (0, commons_1.$value)(version, types_1.UUIDv1);
        const s = h.toLowerCase();
        return s.slice(0, 8) + '-' + s.slice(8, 12)
            + (v === types_1.UUIDv1 ? '-' + s.slice(12, 16) + '-' + s.slice(16, 20)
                : '-4' + s.slice(13, 16) + '-' + _hnab(s.charAt(16)) + s.slice(17, 20))
            + '-' + s.slice(20, 32);
    }
    return null;
}
function _randomFromBytes(m, bytes, is32bits) {
    return is32bits ?
        (bytes.readUInt32LE(0) % m) :
        ((bytes.readUInt32LE(0) << 20) + (bytes.readUInt32LE(4) & 0x000fffff)) % Math.min(m, types_1.UINT_MAX);
}
function _algo(algo) {
    const a = (0, strings_1.$trim)(algo).toUpperCase();
    return (0, commons_1.$ok)(__TSEncryptKeyLength[a]) ? a : exports.AES256;
}
function _randomBytes(length) {
    if (typeof crypto_1.randomBytes === 'function') {
        return (0, crypto_1.randomBytes)(length);
    }
    else {
        const array = new Uint8Array(length);
        if (typeof crypto_1.getRandomValues === 'function') {
            return (0, crypto_1.getRandomValues)(array);
        }
        else if (typeof crypto_1.randomInt === 'function') {
            for (let i = 0; i < length; i++) {
                array[i] = (0, crypto_1.randomInt)(255);
            }
        }
        else {
            for (let i = 0; i < length; i++) {
                array[i] = Math.floor(Math.random() * 255);
            }
        }
        return array;
    }
}
function _createHash(method) { return (0, crypto_1.createHash)((0, commons_1.$value)(__TSHashMethodRef[(0, strings_1.$trim)(method).toUpperCase()], 'sha256')); }
function _charsetKeyAndAlgo(skey, opts) {
    const defaultCharset = tscharset_1.TSCharset.binaryCharset();
    const keyCharset = (0, tscharset_1.$charset)(opts === null || opts === void 0 ? void 0 : opts.keyEncoding, defaultCharset);
    const algo = _algo(opts === null || opts === void 0 ? void 0 : opts.algorithm);
    const key = (0, commons_1.$isstring)(skey) ? keyCharset.uint8ArrayFromString(skey) : (0, data_1.$uint8ArrayFromDataLike)(skey);
    if (key.length !== __TSEncryptKeyLength[algo]) {
        return [null, key, __TSEncryptAlgoRef[exports.AES256]];
    }
    return [(0, tscharset_1.$charset)(opts === null || opts === void 0 ? void 0 : opts.encoding, defaultCharset), key, __TSEncryptAlgoRef[algo]];
}
function _hnab(c) { return '89ab'.includes(c) ? c : '8'; }
function _crc(source, crc, table, andValue, encoding) {
    const src = (0, commons_1.$isstring)(source) ?
        (0, tscharset_1.$charset)(encoding, tscharset_1.TSCharset.binaryCharset()).uint8ArrayFromString(source) :
        (0, data_1.$uint8ArrayFromDataLike)(source);
    const len = src.length;
    for (let i = 0; i < len; i++) {
        crc = ((crc >> 8) & andValue) ^ table[(crc ^ src[i]) & 0xff];
    }
    return crc;
}
function _insideHash256(H, W) {
    function _gamma0(x) { return _ror(7, x) ^ _ror(18, x) ^ (x >>> 3); }
    function _gamma1(x) { return _ror(17, x) ^ _ror(19, x) ^ (x >>> 10); }
    function _sigma0(x) { return _ror(2, x) ^ _ror(13, x) ^ _ror(22, x); }
    function _sigma1(x) { return _ror(6, x) ^ _ror(11, x) ^ _ror(25, x); }
    return function (M) {
        let h = [...H];
        let i = 0;
        for (; i < 16; i++) {
            W[i] = M[i];
        }
        for (; i < 64; i++) {
            W[i] = (_gamma1(W[i - 2]) + W[i - 7] + _gamma0(W[i - 15]) + W[i - 16]) >>> 0;
        }
        for (i = 0; i < 64; i++) {
            const T1 = h[7] + _sigma1(h[4]) + _choice(h[4], h[5], h[6]) + crypto_tables_1.__TSSHA256KTable[i] + W[i];
            const T2 = _sigma0(h[0]) + _majority(h[0], h[1], h[2]);
            h[7] = h[6];
            h[6] = h[5];
            h[5] = h[4];
            h[4] = (h[3] + T1) >>> 0;
            h[3] = h[2];
            h[2] = h[1];
            h[1] = h[0];
            h[0] = (T1 + T2) >>> 0;
        }
        for (let i = 0; i < 8; i++) {
            H[i] = (H[i] + h[i]) >>> 0;
        }
    };
}
function _insideHash512(H, W) {
    function _sigma0(xh, xl) { return (xh >>> 28 | xl << 4) ^ (xl >>> 2 | xh << 30) ^ (xl >>> 7 | xh << 25); }
    function _sigma1(xh, xl) { return (xh >>> 14 | xl << 18) ^ (xh >>> 18 | xl << 14) ^ (xl >>> 9 | xh << 23); }
    function _gamma0(xh, xl) { return (xh >>> 1 | xl << 31) ^ (xh >>> 8 | xl << 24) ^ (xh >>> 7); }
    function _gamma0l(xh, xl) { return (xh >>> 1 | xl << 31) ^ (xh >>> 8 | xl << 24) ^ (xh >>> 7 | xl << 25); }
    function _gamma1(xh, xl) { return (xh >>> 19 | xl << 13) ^ (xl >>> 29 | xh << 3) ^ (xh >>> 6); }
    function _gamma1l(xh, xl) { return (xh >>> 19 | xl << 13) ^ (xl >>> 29 | xh << 3) ^ (xh >>> 6 | xl << 26); }
    function _carry(a, b) { return (a >>> 0) < (b >>> 0) ? 1 : 0; }
    return function (M) {
        let h = [...H];
        for (let i = 0; i < 32; i++) {
            W[i] = M[i];
        }
        for (let i = 32; i < 160; i += 2) {
            let wh = W[i - 30], wl = W[i - 29];
            const gamma0 = _gamma0(wh, wl);
            const gamma0l = _gamma0l(wl, wh);
            wh = W[i - 4];
            wl = W[i - 3];
            const gamma1 = _gamma1(wh, wl);
            const gamma1l = _gamma1l(wl, wh);
            wl = (gamma0l + W[i - 13]) | 0;
            wh = (gamma0 + W[i - 14] + _carry(wl, gamma0l)) | 0;
            wl = (wl + gamma1l) | 0;
            wh = (wh + gamma1 + _carry(wl, gamma1l)) | 0;
            const w16l = W[i - 31];
            wl = (wl + w16l) | 0;
            wh = (wh + W[i - 32] + _carry(wl, w16l)) | 0;
            W[i] = wh;
            W[i + 1] = wl;
        }
        for (let i = 0; i < 160; i += 2) {
            let wh = W[i], wl = W[i + 1];
            const sigma0l = _sigma0(h[1], h[0]);
            const KH = crypto_tables_1.__TSSHA512KTable[i], KL = crypto_tables_1.__TSSHA512KTable[i + 1];
            const choicel = _choice(h[9], h[11], h[13]);
            let T1L = (h[15] + _sigma1(h[9], h[8])) | 0;
            let T1H = (h[14] + _sigma1(h[8], h[9]) + _carry(T1L, h[15])) | 0;
            T1L = (T1L + choicel) | 0;
            T1H = (T1H + _choice(h[8], h[10], h[12]) + _carry(T1L, choicel)) | 0;
            T1L = (T1L + KL) | 0;
            T1H = (T1H + KH + _carry(T1L, KL)) | 0;
            T1L = (T1L + wl) | 0;
            T1H = (T1H + wh + _carry(T1L, wl)) | 0;
            const T2L = (sigma0l + _majority(h[1], h[3], h[5])) | 0;
            const T2H = (_sigma0(h[0], h[1]) + _majority(h[0], h[2], h[4]) + _carry(T2L, sigma0l)) | 0;
            h[14] = h[12];
            h[15] = h[13];
            h[12] = h[10];
            h[13] = h[11];
            h[10] = h[8];
            h[11] = h[9];
            h[9] = (h[7] + T1L) | 0;
            h[8] = (h[6] + T1H + _carry(h[9], h[7])) | 0;
            h[6] = h[4];
            h[7] = h[5];
            h[4] = h[2];
            h[5] = h[3];
            h[2] = h[0];
            h[3] = h[1];
            h[1] = (T1L + T2L) | 0;
            h[0] = (T1H + T2H + _carry(h[1], T1L)) | 0;
        }
        for (let i = 0; i < 16; i += 2) {
            H[i + 1] = (H[i + 1] + h[i + 1]) >>> 0;
            H[i] = (H[i] + h[i] + _carry(H[i + 1], h[i + 1])) >>> 0;
        }
    };
}
exports._insideHash512 = _insideHash512;
function _majority(x, y, z) { return (x & y) | (z & (x | y)); }
function _choice(x, y, z) { return z ^ (x & (y ^ z)); }
function _rol(n, x) { return (x << n) | (x >>> (32 - n)); }
function _ror(n, x) { return (x >>> n) | (x << (32 - n)); }
//# sourceMappingURL=crypto.js.map