/// <reference types="node" />
import { TSCharset } from "./tscharset";
import { Bytes, Nullable, StringEncoding, TSDataLike, TSEndianness, uint32, uint8 } from "./types";
export interface DataInterval {
    start?: Nullable<number>;
    end?: Nullable<number>;
}
export interface DataConversionOptions extends DataInterval {
    forceCopy?: Nullable<boolean>;
}
export declare function $bufferFromArrayBuffer(a: ArrayBuffer): Buffer;
export declare function $bufferFromBytes(source: Bytes, opts?: DataConversionOptions): Buffer;
export declare function $bufferFromDataLike(source: TSDataLike, options?: DataConversionOptions): Buffer;
export { $bufferFromArrayBuffer as $uint8ArrayFromArrayBuffer };
export declare function $uint8ArrayFromBytes(source: Bytes, opts?: DataConversionOptions): Uint8Array;
export declare function $uint8ArrayFromDataLike(source: TSDataLike, options?: DataConversionOptions): Uint8Array;
export { $bufferFromArrayBuffer as $bytesFromArrayBuffer };
export declare function $bytesFromDataLike(source: TSDataLike, opts?: DataConversionOptions): Bytes;
export declare function $arrayFromBytes(source: Bytes, opts?: DataConversionOptions): uint8[];
export declare function $arrayFromDataLike(source: TSDataLike, options?: DataConversionOptions): uint8[];
export declare function $arrayBufferFromBytes(source: Bytes, opts?: DataConversionOptions): ArrayBuffer;
export declare function $arrayBufferFromDataLike(source: TSDataLike, opts?: DataConversionOptions): ArrayBuffer;
export declare function $uint32ArrayFromDataLike(source: TSDataLike, endianness?: Nullable<TSEndianness>, complete?: Nullable<boolean>): uint32[];
export declare function $uint32ArrayFromBuffer(source: Buffer, endianness?: Nullable<TSEndianness>, complete?: Nullable<boolean>): uint32[];
export declare function $decodeBase64(input: string): Uint8Array;
export declare function $decodeBase64URL(input: string): Uint8Array;
export declare function $encodeBase64(source: TSDataLike | string, encoding?: Nullable<StringEncoding | TSCharset>): string;
export declare function $encodeBase64URL(source: TSDataLike | string, encoding?: Nullable<StringEncoding | TSCharset>): string;
export declare function $decodeHexa(s: string): Buffer;
export declare function $encodeHexa(source: TSDataLike): string;
export declare function $dataXOR(a: TSDataLike, b: TSDataLike): Buffer;
interface $bufferAspectOptions {
    prefix?: string;
    suffix?: string;
    name?: string;
    separator?: string;
    showLength?: boolean;
    transformFn?: (n: number) => string;
}
export declare function $dataAspect(source: Bytes | Uint16Array | Uint32Array, opts?: $bufferAspectOptions): string;
declare global {
    export interface String {
        toBase64: (this: string) => string;
    }
    export interface Uint8Array {
        leafInspect: (this: Uint8Array) => string;
        toBase64: (this: Uint8Array) => string;
        toHexa: (this: any) => string;
        isGenuineUint8Array: (this: Uint8Array) => boolean;
        XOR: (this: TSDataLike, other: TSDataLike) => Buffer;
    }
    export interface Uint16Array {
        leafInspect: (this: Uint16Array) => string;
    }
    export interface Uint32Array {
        leafInspect: (this: Uint32Array) => string;
    }
    export interface ArrayBuffer {
        leafInspect: (this: any) => string;
        toBase64: (this: any) => string;
        toHexa: (this: any) => string;
        XOR: (this: TSDataLike, other: TSDataLike) => Buffer;
    }
}
