"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$dataAspect = exports.$dataXOR = exports.$encodeHexa = exports.$decodeHexa = exports.$encodeBase64URL = exports.$encodeBase64 = exports.$decodeBase64URL = exports.$decodeBase64 = exports.$uint32ArrayFromBuffer = exports.$uint32ArrayFromDataLike = exports.$arrayBufferFromDataLike = exports.$arrayBufferFromBytes = exports.$arrayFromDataLike = exports.$arrayFromBytes = exports.$bytesFromDataLike = exports.$bytesFromArrayBuffer = exports.$uint8ArrayFromDataLike = exports.$uint8ArrayFromBytes = exports.$uint8ArrayFromArrayBuffer = exports.$bufferFromDataLike = exports.$bufferFromBytes = exports.$bufferFromArrayBuffer = void 0;
const commons_1 = require("./commons");
const tscharset_1 = require("./tscharset");
const tsdata_1 = require("./tsdata");
function $bufferFromArrayBuffer(a) { return ArrayBuffer.isView(a) ? Buffer.from(a.buffer, a.byteOffset, a.byteLength) : Buffer.from(a); }
exports.$bufferFromArrayBuffer = $bufferFromArrayBuffer;
exports.$uint8ArrayFromArrayBuffer = $bufferFromArrayBuffer;
exports.$bytesFromArrayBuffer = $bufferFromArrayBuffer;
function $bufferFromBytes(source, opts = {}) {
    const [sourceLen, start, end, len] = (0, commons_1.$lse)(source, opts.start, opts.end);
    if (!opts.forceCopy && source instanceof Buffer) {
        return start === 0 && end === sourceLen ? source : source.subarray(start, end);
    }
    else if (!opts.forceCopy && source instanceof Uint8Array) {
        return Buffer.from(source.buffer, source.byteOffset + start, len);
    }
    else if (start === 0 && end === sourceLen) {
        return Buffer.from(source);
    }
    const ret = Buffer.allocUnsafe(len);
    for (let i = start, j = 0; i < end; i++, j++) {
        ret[j] = source[i];
    }
    return ret;
}
exports.$bufferFromBytes = $bufferFromBytes;
function $bufferFromDataLike(source, options) {
    const [src, opts] = _toBytesOpts(source, options);
    return $bufferFromBytes(src, opts);
}
exports.$bufferFromDataLike = $bufferFromDataLike;
function $uint8ArrayFromBytes(source, opts = {}) {
    const [sourceLen, start, end, len] = (0, commons_1.$lse)(source, opts.start, opts.end);
    if (!opts.forceCopy && source instanceof Uint8Array) {
        return start === 0 && end === sourceLen ? source : source.subarray(start, end);
    }
    const ret = new Uint8Array(len);
    for (let i = start, j = 0; i < end; i++, j++) {
        ret[j] = source[i];
    }
    return ret;
}
exports.$uint8ArrayFromBytes = $uint8ArrayFromBytes;
function $uint8ArrayFromDataLike(source, options) {
    const [src, opts] = _toBytesOpts(source, options);
    return $uint8ArrayFromBytes(src, opts);
}
exports.$uint8ArrayFromDataLike = $uint8ArrayFromDataLike;
function $bytesFromDataLike(source, opts = {}) {
    if (source instanceof tsdata_1.TSData || source instanceof ArrayBuffer) {
        return $bufferFromDataLike(source, opts);
    }
    return !opts.forceCopy ? source : $bufferFromDataLike(source, opts);
}
exports.$bytesFromDataLike = $bytesFromDataLike;
function $arrayFromBytes(source, opts = {}) {
    const [sourceLen, start, end,] = (0, commons_1.$lse)(source, opts.start, opts.end);
    if (!opts.forceCopy && !(source instanceof Uint8Array)) {
        return start === 0 && end === sourceLen ? source : source.slice(start, end);
    }
    const ret = [];
    for (let i = start, j = 0; i < end; i++, j++) {
        ret[j] = source[i];
    }
    return ret;
}
exports.$arrayFromBytes = $arrayFromBytes;
function $arrayFromDataLike(source, options) {
    const [src, opts] = _toBytesOpts(source, options);
    return $arrayFromBytes(src, opts);
}
exports.$arrayFromDataLike = $arrayFromDataLike;
function $arrayBufferFromBytes(source, opts = {}) {
    const [, start, end, len] = (0, commons_1.$lse)(source, opts.start, opts.end);
    const ret = new ArrayBuffer(len);
    if (len > 0) {
        const view = new Uint8Array(ret);
        for (let i = start, j = 0; i < end; i++, j++) {
            view[j] = source[i];
        }
    }
    return ret;
}
exports.$arrayBufferFromBytes = $arrayBufferFromBytes;
function $arrayBufferFromDataLike(source, opts = {}) {
    if (source instanceof ArrayBuffer) {
        const [sourceLen, start, end,] = (0, commons_1.$lse)(source, opts.start, opts.end);
        return !opts.forceCopy && start === 0 && end === sourceLen ? source : source.slice(start, end);
    }
    else if (source instanceof tsdata_1.TSData) {
        source = source.mutableBuffer;
    }
    return $arrayBufferFromBytes(source, opts);
}
exports.$arrayBufferFromDataLike = $arrayBufferFromDataLike;
function $uint32ArrayFromDataLike(source, endianness, complete) {
    return $uint32ArrayFromBuffer($bufferFromDataLike(source), endianness, complete);
}
exports.$uint32ArrayFromDataLike = $uint32ArrayFromDataLike;
function $uint32ArrayFromBuffer(source, endianness, complete) {
    const supl = source.length % 4;
    const compl = !!complete && supl > 0 ? 1 : 0;
    const len = (source.length / 4) | 0;
    const ret = new Array(len + compl);
    if (endianness === 'LE') {
        for (let i = 0; i < len; i++)
            ret[i] = source.readInt32LE(i << 2);
        if (compl) {
            ret[len] = 0;
            for (let i = 0, n = len << 2; i < supl; i++) {
                ret[len] |= source[n + i] << (i << 3);
            }
        }
    }
    else {
        for (let i = 0; i < len; i++)
            ret[i] = source.readInt32BE(i << 2);
        if (compl) {
            ret[len] = 0;
            for (let i = 0, n = len << 2; i < supl; i++) {
                ret[len] |= source[n + i] << (24 - (i << 3));
            }
        }
    }
    return ret;
}
exports.$uint32ArrayFromBuffer = $uint32ArrayFromBuffer;
const base64KeyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
const base64URLKeyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_=";
const base64Regex = /[^A-Za-z0-9\+\/\=]/g;
const base64URLRegex = /[^A-Za-z0-9\-\_\=]/g;
function $decodeBase64(input) { return _decodeBase64(input, base64KeyStr, base64Regex); }
exports.$decodeBase64 = $decodeBase64;
function $decodeBase64URL(input) { return _decodeBase64(input, base64URLKeyStr, base64URLRegex); }
exports.$decodeBase64URL = $decodeBase64URL;
function _decodeBase64(input, reference, regex) {
    let chr1, chr2, chr3;
    let enc1, enc2, enc3, enc4;
    let i = 0;
    let size = 0;
    const len = input.length;
    input = input.replace(regex, "");
    let uint8 = new Uint8Array(input.length);
    while (i < len) {
        enc1 = reference.indexOf(input.charAt(i++));
        enc2 = reference.indexOf(input.charAt(i++));
        enc3 = reference.indexOf(input.charAt(i++));
        enc4 = reference.indexOf(input.charAt(i++));
        chr1 = (enc1 << 2) | (enc2 >> 4);
        chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
        chr3 = ((enc3 & 3) << 6) | enc4;
        uint8[size++] = (chr1 & 0xff);
        if (enc3 !== 64) {
            uint8[size++] = (chr2 & 0xff);
        }
        if (enc4 !== 64) {
            uint8[size++] = (chr3 & 0xff);
        }
    }
    return uint8.subarray(0, size);
}
function $encodeBase64(source, encoding) { return _encodeBase64(source, base64KeyStr, encoding); }
exports.$encodeBase64 = $encodeBase64;
function $encodeBase64URL(source, encoding) { return _encodeBase64(source, base64URLKeyStr, encoding); }
exports.$encodeBase64URL = $encodeBase64URL;
function _encodeBase64(source, ref, encoding) {
    const reference = (0, commons_1.$length)(ref) ? ref : base64KeyStr;
    let output = "";
    let chr1, chr2, chr3, enc1, enc2, enc3, enc4;
    const charset = (0, tscharset_1.$charset)(encoding, tscharset_1.TSCharset.binaryCharset());
    const input = (0, commons_1.$isstring)(source) ? charset.uint8ArrayFromString(source) : $uint8ArrayFromDataLike(source);
    const len = input.length;
    let i = 0;
    while (i < len) {
        chr1 = input[i++];
        chr2 = input[i++];
        chr3 = input[i++];
        enc1 = chr1 >> 2;
        enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
        enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
        enc4 = chr3 & 63;
        if (isNaN(chr2)) {
            enc3 = enc4 = 64;
        }
        else if (isNaN(chr3)) {
            enc4 = 64;
        }
        output = output + reference.charAt(enc1) + reference.charAt(enc2) + reference.charAt(enc3) + reference.charAt(enc4);
    }
    return output;
}
function $decodeHexa(s) { return Buffer.from(s, 'hex'); }
exports.$decodeHexa = $decodeHexa;
function $encodeHexa(source) { return $bufferFromDataLike(source).toString('hex'); }
exports.$encodeHexa = $encodeHexa;
function $dataXOR(a, b) {
    const len = Math.max((0, commons_1.$length)(a), (0, commons_1.$length)(b));
    const ret = Buffer.allocUnsafe(len);
    if (!len) {
        return ret;
    }
    const ab = $bytesFromDataLike(a);
    const bb = $bytesFromDataLike(b);
    for (let i = 0; i < len; i++) {
        ret[i] = (ab[i] ^ bb[i]) & 0xff;
    }
    return ret;
}
exports.$dataXOR = $dataXOR;
function $dataAspect(source, opts = { prefix: '[', suffix: ']', separator: ',', showLength: true }) {
    var _a, _b, _c;
    const fn = (0, commons_1.$isfunction)(opts.transformFn) ? opts.transformFn : (n) => n.toString();
    const n = source.length;
    let s = (0, commons_1.$ok)(opts.name) ? opts.name : source.constructor.name;
    if (opts.showLength) {
        s += `(${n})`;
    }
    if (s.length > 0) {
        s += ' ';
    }
    if ((_a = opts.prefix) === null || _a === void 0 ? void 0 : _a.length) {
        s += opts.prefix;
    }
    ;
    if (n > 0) {
        if ((_b = opts.prefix) === null || _b === void 0 ? void 0 : _b.length) {
            s += ' ';
        }
        ;
        s += fn(source[0]);
        for (let i = 1; i < n; i++) {
            s += opts.separator + ' ' + fn(source[i]);
        }
    }
    return ((_c = opts.suffix) === null || _c === void 0 ? void 0 : _c.length) ? s + ` ${opts.suffix}` : s;
}
exports.$dataAspect = $dataAspect;
Uint8Array.prototype.isGenuineUint8Array = function isGenuine() { return this.constructor.name === 'Uint8Array'; };
Uint8Array.prototype.leafInspect = function leafInspect() {
    return this.constructor.name === 'Uint8Array' ? $dataAspect(this) : '<' + $dataAspect(this, { prefix: '', suffix: '', separator: '', showLength: false, transformFn: (n) => n.toHex2() }) + '>';
};
Uint16Array.prototype.leafInspect = function leafInspect() { return $dataAspect(this); };
Uint32Array.prototype.leafInspect = function leafInspect() { return $dataAspect(this); };
ArrayBuffer.prototype.leafInspect = function leafInspect() {
    const buf = $bufferFromArrayBuffer(this);
    return 'ArrayBuffer { [Uint8Contents]: <' + $dataAspect(buf, { prefix: '', suffix: '', separator: '', showLength: false, name: '', transformFn: (n) => n.toHex2(true) }) + '>, byteLength: ' + buf.length + ' }';
};
String.prototype.toBase64 = function toBase64() { return $encodeBase64(this); };
Uint8Array.prototype.toBase64 = function toBase64() { return $encodeBase64(this); };
ArrayBuffer.prototype.toBase64 = function toBase64() { return $encodeBase64(this); };
Uint8Array.prototype.toHexa = function toHexa() { return Buffer.from(this.buffer, this.byteOffset, this.byteLength).toString('hex'); ; };
Buffer.prototype.toHexa = function toHexa() { return this.toString('hex'); };
ArrayBuffer.prototype.toHexa = function toHexa() { return $encodeHexa(this); };
Uint8Array.prototype.XOR = function xor(other) { return $dataXOR(this, other); };
ArrayBuffer.prototype.XOR = function xor(other) { return $dataXOR(this, other); };
function _toBytesOpts(source, opts = {}) {
    if (source instanceof ArrayBuffer) {
        return [$bufferFromArrayBuffer(source), Object.assign(Object.assign({}, opts), { forceCopy: false })];
    }
    else if (source instanceof tsdata_1.TSData) {
        return [source.mutableBuffer, opts];
    }
    return [source, opts];
}
//# sourceMappingURL=data.js.map