"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$timeBetweenDates = void 0;
const types_1 = require("./types");
const commons_1 = require("./commons");
const tsdate_1 = require("./tsdate");
const tsdatecomp_1 = require("./tsdatecomp");
const compare_1 = require("./compare");
const object_1 = require("./object");
const tserrors_1 = require("./tserrors");
function $timeBetweenDates(pa, pb) {
    let a = _acceptableDateParameter(pa);
    let b = _acceptableDateParameter(pb, true);
    if (typeof a === 'number' && typeof b === 'number') {
        return b - a;
    }
    if (a instanceof Date && b instanceof Date) {
        return b.timeSinceDate(a);
    }
    if (!(a instanceof tsdate_1.TSDate)) {
        a = new tsdate_1.TSDate(a);
    }
    if (!(b instanceof tsdate_1.TSDate)) {
        b = new tsdate_1.TSDate(b);
    }
    return b.timeSinceDate(a);
}
exports.$timeBetweenDates = $timeBetweenDates;
Date.prototype.leafInspect = Date.prototype.toISOString;
(0, object_1.$declareAccessor)(Date, { element: 'timeStamp', getter: function () { return (0, tsdatecomp_1.$components2timestamp)((0, tsdatecomp_1.$components)(this)); } });
Date.prototype.isEqual = function isEqual(other) {
    if (this === other) {
        return true;
    }
    if (other instanceof Date) {
        return this.getTime() === other.getTime();
    }
    if (other instanceof tsdate_1.TSDate) {
        return (0, compare_1.$datecompare)(this, other) === types_1.Same;
    }
    return false;
};
Date.prototype.compare = function compare(other) {
    if (this === other) {
        return types_1.Same;
    }
    if (other instanceof Date) {
        return (0, compare_1.$numcompare)(this.getTime(), other.getTime());
    }
    if (other instanceof tsdate_1.TSDate) {
        return (0, compare_1.$datecompare)(this, other);
    }
    return undefined;
};
Date.prototype.toArray = function toArray() { return [this]; };
Date.prototype.timeSinceDate = function timeSinceDate(other) { return (this.getTime() - other.getTime()) / 1000; };
Date.prototype.toComponents = function toComponents() { return (0, tsdatecomp_1.$components)(this); };
Date.prototype.toDate = function toDate() { return this; };
Date.prototype.toTSDate = function toTSDate() { return tsdate_1.TSDate.fromDate(this); };
function _acceptableDateParameter(a, future = false) {
    if (!(0, commons_1.$ok)(a)) {
        return !future ? tsdate_1.TSDate.past() : tsdate_1.TSDate.future();
    }
    if (a === Number.POSITIVE_INFINITY) {
        return tsdate_1.TSDate.future();
    }
    if (a === Number.NEGATIVE_INFINITY) {
        return tsdate_1.TSDate.past();
    }
    if (a instanceof Date || a instanceof tsdate_1.TSDate) {
        return a;
    }
    const t = typeof a;
    if (t === 'string') {
        return new tsdate_1.TSDate(a);
    }
    if (t === 'number') {
        if (isNaN(a)) {
            tserrors_1.TSError.throw('$timeBetweenDates() cannot handle NaN as numbers');
        }
        return a;
    }
    tserrors_1.TSError.throw(`$timeBetweenDates() cannot handle value of type ${t}`);
}
//# sourceMappingURL=date.js.map