"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$argCheck = exports.$args = exports.$parsedenv = exports.$env = void 0;
const commons_1 = require("./commons");
const utils_1 = require("./utils");
const strings_1 = require("./strings");
const tscharset_1 = require("./tscharset");
const tsparser_1 = require("./tsparser");
const tserrors_1 = require("./tserrors");
const tsurl_1 = require("./tsurl");
;
function $env(source, opts) {
    return _parsenv('$env', source, null, true, (0, commons_1.$value)(opts, {}));
}
exports.$env = $env;
function $parsedenv(source, opts) {
    const tmp = Object.assign({}, opts);
    delete tmp['parser'];
    delete tmp['acceptsUnparsed'];
    return _parsenv('$parsedenv', source, opts.parser, !!opts.acceptsUnparsed, Object.assign({}, tmp));
}
exports.$parsedenv = $parsedenv;
function _parsenv(fn, source, parserDefinition, acceptsUncheckedItems, opts) {
    const debug = !!(opts === null || opts === void 0 ? void 0 : opts.debug);
    const parse = (0, commons_1.$ok)(parserDefinition);
    const parserStructure = { _mandatory: true, _acceptsUncheckedItems: acceptsUncheckedItems };
    let parser = undefined;
    if (parse) {
        const entries = Object.entries(parserDefinition);
        const nameSet = new Set();
        let n = 0;
        for (let [name, d] of entries) {
            name = (0, strings_1.$ftrim)(name);
            if (name.length === 0) {
                tserrors_1.TSError.throw(`${fn}(): found empty name definition`, parserDefinition);
            }
            if (_normarg(name) !== name || name.length < 2) {
                tserrors_1.TSError.throw(`${fn}(): found bad name for env var '${name}'`, parserDefinition);
            }
            if (nameSet.has(name)) {
                tserrors_1.TSError.throw(`${fn}(): found duplicate name definition '${name}'`, parserDefinition);
            }
            let localParser = undefined;
            if ((0, commons_1.$isstring)(d) || (0, commons_1.$ok)(d._type)) {
                localParser = tsparser_1.TSParser.define(d);
            }
            if (!(0, commons_1.$ok)(localParser)) {
                tserrors_1.TSError.throw(`${fn}(): bad parser definition for env var '${name}'`, parserDefinition);
            }
            parserStructure[name] = d;
            n++;
        }
        if (n === 0) {
            tserrors_1.TSError.throw(`${fn}(): no definition found in environment parser`, parserDefinition);
        }
        parser = tsparser_1.TSParser.define(parserStructure);
        if (!(0, commons_1.$ok)(parser)) {
            tserrors_1.TSError.throw(`${fn}(): impossible to define environment parser structure`, parserDefinition);
        }
    }
    const merge = (0, commons_1.$ok)(opts === null || opts === void 0 ? void 0 : opts.merge);
    const ret = merge ? opts.merge : {};
    const previous = [];
    const toBeDeleted = [];
    const ref = (0, commons_1.$ok)(opts === null || opts === void 0 ? void 0 : opts.reference) ? opts.reference : {};
    const unary = {};
    const variableMax = Math.min(Math.max(1, (0, commons_1.$unsigned)(opts === null || opts === void 0 ? void 0 : opts.variableMax, 64)), 256);
    const underscoreMax = Math.min(Math.max(1, (0, commons_1.$unsigned)(opts === null || opts === void 0 ? void 0 : opts.underscoreMax, 2)), 8, variableMax);
    if (!(0, commons_1.$ok)(source)) {
        if (debug) {
            (0, utils_1.$logterm)("&R&w $env(): called with a null or undefined source  &0");
        }
        return ret;
    }
    const lines = (0, strings_1.$lines)((0, commons_1.$isstring)(source) ? source : (0, tscharset_1.$charset)(opts === null || opts === void 0 ? void 0 : opts.encoding).stringFromData(source));
    for (let i = 0, n = lines.length; i < n; i++) {
        const [key, value] = _interpretEnvLine(ret, lines[i].rtrim(), i + 1, ref, underscoreMax, variableMax, debug);
        if (key === 'void') {
            continue;
        }
        if (!key.length) {
            return null;
        }
        if (merge) {
            if ((0, commons_1.$defined)(ret[key])) {
                previous.push({ k: key, p: ret[key] });
            }
            else {
                toBeDeleted.push(key);
            }
        }
        ret[key] = value;
        if (parse) {
            unary[key] = value;
        }
    }
    if (parse) {
        const errors = debug ? [] : undefined;
        const parsedUnary = parser.interpret(unary, { context: tsparser_1.TSParserActionContext.env, errors: errors });
        if (!(0, commons_1.$ok)(parsedUnary)) {
            if (merge) {
                toBeDeleted.forEach(k => delete opts.merge[k]);
                previous.forEach(def => opts.merge[def.k] = def.p);
            }
            return null;
        }
        return Object.assign(Object.assign({}, ret), parsedUnary);
    }
    return ret;
}
function $args(definition, opts) {
    var _a;
    const entries = Object.entries(definition);
    let url = undefined;
    const ref = new Map();
    let isURL = (opts === null || opts === void 0 ? void 0 : opts.arguments) instanceof URL || (opts === null || opts === void 0 ? void 0 : opts.arguments) instanceof tsurl_1.TSURL;
    if (isURL) {
        url = (opts === null || opts === void 0 ? void 0 : opts.arguments) instanceof tsurl_1.TSURL ? opts.arguments : tsurl_1.TSURL.from(opts.arguments);
    }
    const vargs = process && process !== null && typeof process !== 'undefined' ? process === null || process === void 0 ? void 0 : process.argv : undefined;
    const isvargs = !isURL && (0, commons_1.$isarray)(vargs);
    const passedargs = (0, commons_1.$isarray)(opts === null || opts === void 0 ? void 0 : opts.arguments);
    const prefix = isURL ? '' : '-';
    const parserStructure = { _mandatory: true };
    const nameSet = new Set();
    const defaults = new Array();
    let n = 0;
    for (let [name, d] of entries) {
        name = (0, strings_1.$ftrim)(name);
        if (name.length === 0) {
            tserrors_1.TSError.throw('$arg(): found empty name definition', definition);
        }
        if (_normarg(name) !== name || name.length < 2) {
            tserrors_1.TSError.throw(`$arg(): found bad name for argument '${name}'`, definition);
        }
        let twice = nameSet.has(name);
        if (!twice) {
            let parser = undefined;
            if (typeof d === 'string') {
                twice = ref.has(prefix + name) || (prefix.length > 0 && ref.has(prefix + prefix + name));
                if (!twice) {
                    parser = tsparser_1.TSParser.define(d);
                    if ((0, commons_1.$ok)(parser)) {
                        nameSet.add(name);
                        parserStructure[name] = d;
                        const def = { name: name, positive: d === 'boolean' || d === 'boolean!' ? true : undefined };
                        ref.set(prefix + name, def);
                        if (prefix.length) {
                            ref.set(prefix + prefix + name, def);
                        }
                        n++;
                    }
                }
            }
            else {
                const ds = (0, commons_1.$ok)(d._type) ? { struct: d } : d;
                const short = (0, commons_1.$string)(ds.short);
                const isBoolean = (0, commons_1.$isstring)(ds.struct) ?
                    ds.struct === 'boolean' || ds.struct === 'boolean!' :
                    ds.struct._type === 'boolean';
                const nname = isBoolean ? (0, commons_1.$string)(ds.negative) : '';
                const nshort = isBoolean ? (0, commons_1.$string)(ds.negativeShort) : '';
                const doubledash = !ds.noDoubleDash && prefix.length > 0;
                if (nname.length === 1 || (nname.length > 1 && (_normarg(nname) !== nname || nname === name))) {
                    tserrors_1.TSError.throw(`$args(): bad definition for negative name version of argument '${name}'`, { name: name, structure: d });
                }
                if (short.length > 1 || (short.length === 1 && (_normarg(short) !== short || short === nshort))) {
                    tserrors_1.TSError.throw(`$args(): bad definition for short version of argument '${name}'`, { name: name, definition: d });
                }
                if (nshort.length > 1 || (nshort.length === 1 && (_normarg(nshort) !== nshort || short === nshort))) {
                    tserrors_1.TSError.throw(`$args(): bad definition for negative short version of argument '${name}'`, { name: name, structure: d });
                }
                twice = ref.has(prefix + name) ||
                    (doubledash && ref.has(prefix + prefix + name)) ||
                    (short.length === 1 && ref.has(prefix + short)) ||
                    (nname.length > 1 && ref.has(prefix + nname)) ||
                    (doubledash && nname.length > 1 && ref.has(prefix + prefix + nname)) ||
                    (nshort.length === 1 && ref.has(prefix + nshort));
                if (!twice) {
                    parser = tsparser_1.TSParser.define(ds.struct);
                    if ((0, commons_1.$ok)(parser)) {
                        if (isBoolean && (parser === null || parser === void 0 ? void 0 : parser.mandatory) && nname.length === 0 && nshort.length === 0) {
                            tserrors_1.TSError.throw(`$args(): argument '${name}' cannot be mandatory and have no negative version`, { name: name, structure: d });
                        }
                        const def = { name: name, positive: isBoolean ? true : undefined };
                        parserStructure[name] = ds.struct;
                        nameSet.add(name);
                        if ((0, commons_1.$ok)(ds.defaultValue)) {
                            defaults.push({ name: name, value: ds.defaultValue });
                        }
                        ref.set(prefix + name, def);
                        if (doubledash) {
                            ref.set(prefix + prefix + name, def);
                        }
                        if (prefix.length >= 0 && short.length === 1) {
                            ref.set(prefix + short, def);
                        }
                        if (nname.length > 1 || nshort.length === 1) {
                            const ndef = { name: name, positive: isBoolean ? false : undefined };
                            if (nname.length > 1) {
                                ref.set(prefix + nname, ndef);
                                if (doubledash) {
                                    ref.set(prefix + prefix + nname, ndef);
                                }
                            }
                            if (prefix.length > 0 && nshort.length === 1) {
                                ref.set(prefix + nshort, ndef);
                            }
                        }
                        n++;
                    }
                }
            }
            if (!(0, commons_1.$ok)(parser)) {
                tserrors_1.TSError.throw(`bad structure definition for argument '${name}'`, { name: name, structure: d });
            }
        }
        if (twice) {
            tserrors_1.TSError.throw(`$args(): argument '${name}' is declared twice`, definition);
        }
    }
    if (n === 0) {
        tserrors_1.TSError.throw("$args(): no arguments defined", definition);
    }
    const parser = tsparser_1.TSParser.define(parserStructure);
    let dict = null;
    let args = [];
    if (!(0, commons_1.$ok)(parser)) {
        tserrors_1.TSError.throw("$args(): unable to generate parser", { structure: parserStructure, errors: opts === null || opts === void 0 ? void 0 : opts.errors });
    }
    if (isURL) {
        dict = parser.interpret(_dictionaryFromURLQuery(url, ref, opts === null || opts === void 0 ? void 0 : opts.errors), { errors: opts === null || opts === void 0 ? void 0 : opts.errors, context: 'url' });
    }
    if (isvargs || passedargs) {
        [dict, args] = _argumentsFromVarArgs(passedargs ? opts === null || opts === void 0 ? void 0 : opts.arguments : (_a = process === null || process === void 0 ? void 0 : process.argv) === null || _a === void 0 ? void 0 : _a.slice(2), ref, opts === null || opts === void 0 ? void 0 : opts.errors);
        if ((0, commons_1.$ok)(dict)) {
            const interpretErrors = [];
            dict = (0, commons_1.$valueornull)(parser.interpret(dict, { errors: interpretErrors, context: tsparser_1.TSParserActionContext.vargs }));
            if (!(0, commons_1.$ok)(dict) && (0, commons_1.$ok)(opts === null || opts === void 0 ? void 0 : opts.errors)) {
                interpretErrors.forEach(e => {
                    var _a;
                    const m = e.match(/^object\.(\S+)\s+(.+)$/);
                    if ((0, commons_1.$ok)(m)) {
                        (_a = opts === null || opts === void 0 ? void 0 : opts.errors) === null || _a === void 0 ? void 0 : _a.push(`Argument '-${m[1]}' ${m[2]}`);
                    }
                });
            }
        }
    }
    if ((0, commons_1.$isnumber)(opts === null || opts === void 0 ? void 0 : opts.exitError) && (opts === null || opts === void 0 ? void 0 : opts.exitError) !== 0) {
        $argCheck((0, commons_1.$toint)(opts === null || opts === void 0 ? void 0 : opts.exitError), opts === null || opts === void 0 ? void 0 : opts.errors, opts === null || opts === void 0 ? void 0 : opts.processName);
    }
    if ((0, commons_1.$ok)(dict)) {
        defaults.forEach(c => { if (!(0, commons_1.$ok)(dict[c.name])) {
            dict[c.name] = c.value;
        } });
    }
    return [dict, args];
}
exports.$args = $args;
function $argCheck(exitStatus, errors, processName) {
    const err = (0, commons_1.$isarray)(errors) ? errors : ((0, commons_1.$ok)(errors) ? errors.errors : undefined);
    const n = (0, commons_1.$count)(err);
    if (n > 0) {
        (0, utils_1.$logheader)(`${(0, commons_1.$length)(processName) ? processName : 'process'} did encounter ${n > 1 ? 'errors' : 'an error'}`, undefined, '&r', '&o');
        (0, utils_1.$logterm)(`&y${err.join('.\n')}.&0\n`);
        if (exitStatus !== 0) {
            (0, utils_1.$exit)(exitStatus);
        }
    }
}
exports.$argCheck = $argCheck;
function _dictionaryFromURLQuery(url, ref, errors) {
    let doomed = false;
    const dict = {};
    for (const [key, value] of url.searchParams) {
        if (key.length > 0 && (0, commons_1.$ok)(value)) {
            const def = ref.get(key);
            if (!(0, commons_1.$ok)(def)) {
                errors === null || errors === void 0 ? void 0 : errors.push(`Unknown argument '${key}'.`);
                doomed = true;
            }
            if ((0, commons_1.$ok)(def === null || def === void 0 ? void 0 : def.positive)) {
                dict[def.name] = (def === null || def === void 0 ? void 0 : def.positive) ? value : _inversedValue(value);
            }
            else {
                const v = dict[def.name];
                if (!(0, commons_1.$ok)(v)) {
                    dict[def.name] = value;
                }
                else if ((0, commons_1.$isarray)(v)) {
                    v.push(value);
                }
                else {
                    dict[def.name] = [v, value];
                }
            }
        }
    }
    return doomed ? null : dict;
}
function _inversedValue(v) {
    if (typeof v === 'boolean') {
        return !v;
    }
    if (v === 0 || v === '0') {
        return true;
    }
    if (v === 1 || v === '1') {
        return false;
    }
    v = (0, commons_1.$string)(v);
    const s = (0, strings_1.$ftrim)(v).toLowerCase();
    if (s === '1' || s === 'true' || s === 'yes' || s === 'y') {
        return false;
    }
    if (s === '0' || s === 'false' || s === 'no' || s === 'n') {
        return true;
    }
    return v;
}
function _argumentsFromVarArgs(args, ref, errors) {
    const finalArgs = [];
    let doomed = false;
    args === null || args === void 0 ? void 0 : args.forEach(a => {
        if (a.startsWith('-')) {
            const len = a.length;
            if (len === 1) {
                errors === null || errors === void 0 ? void 0 : errors.push('Solitary dash (-) was found as an argument.');
                doomed = true;
            }
            else if (a.startsWith('--')) {
                if (len === 2) {
                    errors === null || errors === void 0 ? void 0 : errors.push('Solitary double dash (--) was found as an argument.');
                    doomed = true;
                }
                else if (!ref.has(a)) {
                    errors === null || errors === void 0 ? void 0 : errors.push(`Unknown argument '${a}'.`);
                    doomed = true;
                }
                else {
                    finalArgs.push(a);
                }
            }
            else if (!ref.has(a)) {
                let fn = 1;
                for (let i = 1; i < len; i++) {
                    if (!ref.has('-' + a[i])) {
                        errors === null || errors === void 0 ? void 0 : errors.push(`Unknown argument '-${a[i]}'.`);
                        doomed = true;
                        continue;
                    }
                    finalArgs.push('-' + a[i]);
                    fn++;
                }
                if (fn < len && len > 2) {
                    errors === null || errors === void 0 ? void 0 : errors.push(`Unknown argument '${a}'.`);
                    doomed = true;
                }
            }
            else {
                finalArgs.push(a);
            }
        }
        else {
            finalArgs.push(a);
        }
    });
    const remains = [];
    if (doomed) {
        return [null, remains];
    }
    let lastArg = '';
    let lastDef = undefined;
    const dict = {};
    doomed = false;
    finalArgs.forEach(a => {
        if (lastArg.length) {
            if (a.startsWith('-')) {
                errors === null || errors === void 0 ? void 0 : errors.push(`Found no value for argument '${lastArg}'.`);
                doomed = true;
            }
            else {
                const v = dict[lastDef.name];
                if (!(0, commons_1.$ok)(v)) {
                    dict[lastDef.name] = a;
                }
                else if ((0, commons_1.$isarray)(v)) {
                    v.push(a);
                }
                else {
                    dict[lastDef.name] = [v, a];
                }
            }
            lastArg = '';
            lastDef = undefined;
        }
        else if (a.startsWith('-')) {
            const def = ref.get(a);
            if ((0, commons_1.$ok)(def.positive)) {
                dict[def.name] = def.positive;
            }
            else {
                lastArg = a;
                lastDef = def;
            }
        }
        else {
            remains.push(a);
        }
    });
    if (lastArg.length) {
        errors === null || errors === void 0 ? void 0 : errors.push(`Found no value for argument '${lastArg}'.`);
        doomed = true;
    }
    return [doomed ? null : dict, remains];
}
function _normarg(s) { return (0, strings_1.$ascii)((0, strings_1.$normspaces)(s, { strict: true, replacer: '' })); }
function _interpretEnvLine(env, line, index, reference, underscoreMax, variableMax, debug) {
    const len = line.length;
    let p = 0;
    while (p < len && isspace(line.charCodeAt(p))) {
        p++;
    }
    if (p === len || line.charAt(p) === '#') {
        return ['void', ''];
    }
    if (line.slice(p, p + 6) === "export") {
        p += 6;
        let ws = 0;
        while (p < len && isspace(line.charCodeAt(p))) {
            p++;
            ws++;
        }
        if (p === len || line.charAt(p) === '#') {
            if (debug) {
                (0, utils_1.$logterm)(`&R&w $env(): line #${index} only contains 'export' keyword  &0`);
            }
            return ['', ''];
        }
        else if (!ws) {
            if (debug) {
                (0, utils_1.$logterm)(`&R&w $env(): line #${index} 'export' keyword must be followed at least one space character  &0`);
            }
            return ['', ''];
        }
    }
    const [DOUBLE_QUOTE, DOLLAR, QUOTE, ZERO, NINE, EQ, A, Z, BACKSLASH, UNDERSCORE, b, f, n, r, t, u, LCB, RCB] = [0x22, 0x24, 0x27, 0x30, 0x39, 0x3D, 0x41, 0x5A, 0x5C, 0x5F, 0x62, 0x66, 0x6E, 0x72, 0x74, 0x75, 0x7B, 0x7D];
    function upper(c) { return (c & 0x00df); }
    function isletter(c) { c = upper(c); return c >= A && c <= Z; }
    function isvarchar(c) { return c === UNDERSCORE || isletter(c) || (c >= ZERO && c <= NINE); }
    function isspace(c) { return c.isStrictWhiteSpace(); }
    function badchar(l, pos, tag = '') {
        const c = l.charCodeAt(pos);
        if (debug) {
            (0, utils_1.$logterm)(`&R&w $env(): found wrong character '${String.fromCharCode(c)}' (\\U${c.toHex4()}) at position ${pos + 1} and line #${index}${tag.length ? ' [' + tag + ']' : ''}  &0`);
        }
        return ['', ''];
    }
    function underscoreOverload(pos, tag = '') {
        if (debug) {
            (0, utils_1.$logterm)(`&R&w $env(): found more than ${underscoreMax} '_' character in variable or substitution at position ${pos + 1} and line #${index}${tag.length ? ' [' + tag + ']' : ''}  &0`);
        }
        return ['', ''];
    }
    function variableOverload(pos, tag = '') {
        if (debug) {
            (0, utils_1.$logterm)(`&R&w $env(): found variable or subsitution variable longer than than ${variableMax} at position ${pos + 1} and line #${index}${tag.length ? ' [' + tag + ']' : ''}  &0`);
        }
        return ['', ''];
    }
    function incompleteLine(tag = '') {
        if (debug) {
            (0, utils_1.$logterm)(`&R&w $env(): line #${index} is incomplete${tag.length ? ' [' + tag + ']' : ''}  &0\n&o"&y${line}&o"&0`);
        }
        return ['', ''];
    }
    let vl = 0;
    const keyStart = p;
    while (p < len && line.charCodeAt(p) === UNDERSCORE) {
        vl++;
        if (vl > underscoreMax) {
            return underscoreOverload(p, "var");
        }
        p++;
    }
    if (p === len) {
        return incompleteLine("underscore var only");
    }
    if (!isletter(line.charCodeAt(p))) {
        return badchar(line, p, "var start");
    }
    vl++;
    if (vl > variableMax) {
        return variableOverload(p, "var #1");
    }
    p++;
    if (p === len) {
        return incompleteLine("NO = sign #1");
    }
    while (p < len && isvarchar(line.charCodeAt(p))) {
        vl++;
        if (vl > variableMax) {
            return variableOverload(p, "var #2");
        }
        p++;
    }
    const afterKey = p;
    while (p < len && isspace(line.charCodeAt(p))) {
        p++;
    }
    if (p === len) {
        return incompleteLine("NO = sign #2");
    }
    if (line.charCodeAt(p) !== EQ) {
        return badchar(line, p, "NO =");
    }
    p++;
    while (p < len && isspace(line.charCodeAt(p))) {
        p++;
    }
    if (p === len) {
        return [line.slice(keyStart, afterKey), ''];
    }
    const c0 = line.charCodeAt(p);
    let last = undefined;
    if (c0 === DOUBLE_QUOTE || c0 === QUOTE) {
        last = c0;
        p++;
        if (p === len) {
            return incompleteLine("value never started");
        }
    }
    let p0 = p;
    let v = '';
    let State;
    (function (State) {
        State["Text"] = "TEXT";
        State["Substitution"] = "SUBS";
        State["SubstitutionLetter"] = "SUBL";
        State["SubstitutionVar"] = "SUBV";
        State["Backslash"] = "BACK";
        State["Unicode"] = "HEXA";
        State["Stop"] = "STOP";
    })(State || (State = {}));
    ;
    let state = State.Text;
    let uc = 0;
    let us = 0;
    while (p < len && state !== State.Stop) {
        const c = line.charCodeAt(p);
        switch (state) {
            case State.Text:
                if ((0, commons_1.$ok)(last) && c === last) {
                    v += line.slice(p0, p);
                    state = State.Stop;
                }
                else if (c === BACKSLASH) {
                    v += line.slice(p0, p);
                    state = State.Backslash;
                }
                else if (c === DOLLAR && last !== QUOTE) {
                    v += line.slice(p0, p);
                    state = State.Substitution;
                }
                break;
            case State.Backslash:
                state = State.Text;
                p0 = p + 1;
                switch (c) {
                    case b:
                        v += '\b';
                        break;
                    case f:
                        v += '\f';
                        break;
                    case n:
                        v += '\n';
                        break;
                    case r:
                        v += '\r';
                        break;
                    case t:
                        v += '\t';
                        break;
                    case u:
                        uc = 0;
                        us = 0;
                        state = State.Unicode;
                        break;
                    default: v += String.fromCharCode(c);
                }
                break;
            case State.Unicode:
                const hexa = c.hexaValue();
                if (hexa < 0) {
                    return badchar(line, p, `\\u[${us}]`);
                }
                uc = (uc << 4) | hexa;
                us++;
                if (us === 4) {
                    v += String.fromCharCode(uc);
                    state = State.Text;
                    p0 = p + 1;
                }
                break;
            case State.Substitution:
                if (c !== LCB) {
                    return badchar(line, p, '{');
                }
                p0 = p + 1;
                vl = 0;
                state = State.SubstitutionLetter;
                break;
            case State.SubstitutionLetter:
                if (isletter(c)) {
                    state = State.SubstitutionVar;
                    vl++;
                    if (vl > variableMax) {
                        return variableOverload(p, "substitution #1");
                    }
                }
                else if (c !== UNDERSCORE) {
                    return badchar(line, p, "substitution '_'");
                }
                vl++;
                if (vl > underscoreMax) {
                    return underscoreOverload(p, "substitution");
                }
                break;
            case State.SubstitutionVar:
                if (c === RCB) {
                    const variable = line.slice(p0, p);
                    let variableValue = env[variable];
                    if (!(0, commons_1.$ok)(variableValue)) {
                        variableValue = reference[variable];
                    }
                    variableValue = (0, commons_1.$string)(variableValue);
                    if ((0, commons_1.$length)(variableValue) > 0) {
                        v += variableValue;
                    }
                    state = State.Text;
                    p0 = p + 1;
                }
                else if (!isvarchar(c)) {
                    return badchar(line, p, '}');
                }
                vl++;
                if (vl > variableMax) {
                    return variableOverload(p, "substitution #2");
                }
                break;
        }
        p++;
    }
    if (state === State.Text) {
        if ((0, commons_1.$ok)(last)) {
            return incompleteLine(`NO string value end (${last})`);
        }
        v += line.slice(p0, p);
    }
    else if (state !== State.Stop) {
        return incompleteLine(`BAD state ${state}`);
    }
    if (p < len) {
        let pe = p;
        while (pe < len && isspace(line.charCodeAt(pe))) {
            pe++;
        }
        if (pe < len && line.charAt(pe) !== '#') {
            return badchar(line, pe, '#');
        }
    }
    return [line.slice(keyStart, afterKey), v];
}
//# sourceMappingURL=env.js.map