"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTMLContent = exports.$HTML = exports.$snakeCase = exports.$camelCase = exports.$capitalize = exports.$firstcap = exports.$normspaces = exports.$lines = exports.$right = exports.$left = exports.$trim = exports.$ftrim = exports.$rtrim = exports.$ltrim = exports.$ascii = void 0;
const commons_1 = require("./commons");
const string_tables_1 = require("./string_tables");
const tsdate_1 = require("./tsdate");
const tsphonenumber_1 = require("./tsphonenumber");
function $ascii(source) {
    const l = (0, commons_1.$length)(source);
    if (!l)
        return '';
    let s = source.replace(/≠/g, "");
    s = s.normalize("NFD").replace(/[\u0300-\u036f]|\u00a8|\u00ad|\u00b4/g, "").normalize("NFKD");
    return s.replace(/[^\x00-\x7F]/g, x => string_tables_1.FoundationASCIIConversion[x] || '');
}
exports.$ascii = $ascii;
function $ltrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationLeftTrimRegex, "") : ''; }
exports.$ltrim = $ltrim;
function $rtrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationRightTrimRegex, "") : ''; }
exports.$rtrim = $rtrim;
function $ftrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationLeftTrimRegex, "").replace(string_tables_1.FoundationRightTrimRegex, "") : ''; }
exports.$ftrim = $ftrim;
exports.$trim = $ftrim;
function $left(source, leftPart) {
    const l = (0, commons_1.$length)(source);
    let n = (0, commons_1.$unsigned)(leftPart);
    if (!n)
        (n = 1);
    return l > 0 ? (n >= l ? source : source.slice(0, n)) : '';
}
exports.$left = $left;
function $right(source, rightPart) {
    const l = (0, commons_1.$length)(source);
    let n = (0, commons_1.$unsigned)(rightPart);
    if (!n)
        (n = 1);
    return l > 0 ? (n >= l ? source : source.slice(l - n, l)) : '';
}
exports.$right = $right;
function $lines(s, useOnlyASCIISeparators = false) {
    const len = (0, commons_1.$length)(s);
    const ret = [];
    if (len > 0) {
        let LCState;
        (function (LCState) {
            LCState[LCState["CR"] = 0] = "CR";
            LCState[LCState["NewLine"] = 1] = "NewLine";
            LCState[LCState["Other"] = 2] = "Other";
        })(LCState || (LCState = {}));
        ;
        const [CR, LF, FF, NEL, LS, PS] = [0x000D, 0x000A, 0x000C, 0x0085, 0x02028, 0x02029];
        const isOtherLineSeparator = useOnlyASCIISeparators ? (_) => false : (c) => c === LS || c === PS || c === NEL || c === FF;
        let lcstate = LCState.Other;
        let lastPosition = 0, pos = 0;
        while (pos < len) {
            const c = s.charCodeAt(pos);
            if (lcstate === LCState.CR) {
                ret.push(s.slice(lastPosition, pos - 1));
                if (c === LF) {
                    lastPosition = pos + 1;
                    lcstate = LCState.NewLine;
                }
                else if (isOtherLineSeparator(c)) {
                    ret.push('');
                    lastPosition = pos + 1;
                    lcstate = LCState.NewLine;
                }
                else {
                    lcstate = LCState.Other;
                    lastPosition = pos--;
                }
            }
            else if (c === CR) {
                lcstate = LCState.CR;
            }
            else if (c === LF || isOtherLineSeparator(c)) {
                ret.push(s.slice(lastPosition, pos));
                lastPosition = pos + 1;
                lcstate = LCState.NewLine;
            }
            else {
                lcstate = LCState.Other;
            }
            pos++;
        }
        switch (lcstate) {
            case LCState.CR:
                ret.push(s.slice(lastPosition, pos - 1));
                ret.push('');
                break;
            case LCState.NewLine:
                ret.push('');
                break;
            case LCState.Other:
                if (lastPosition < pos) {
                    ret.push(s.slice(lastPosition, pos));
                }
                break;
        }
    }
    else if ((0, commons_1.$ok)(s)) {
        ret.push('');
    }
    return ret;
}
exports.$lines = $lines;
function $normspaces(s, opts = {}) {
    return $ftrim(s).replace(opts.strict ? string_tables_1.FoundationFindStrictWhitespacesRegex : string_tables_1.FoundationFindAllWhitespacesRegex, (0, commons_1.$value)(opts.replacer, " "));
}
exports.$normspaces = $normspaces;
function $firstcap(s) { return _capitalize(s, 1); }
exports.$firstcap = $firstcap;
function $capitalize(s) { return _capitalize(s); }
exports.$capitalize = $capitalize;
function $camelCase(s) { return _camelCase(s); }
exports.$camelCase = $camelCase;
function $snakeCase(s) { return _snakeCase(s); }
exports.$snakeCase = $snakeCase;
function $HTML(s, reference = string_tables_1.FoundationHTMLEncoding) {
    const len = (0, commons_1.$length)(s);
    let ret = "";
    for (let i = 0; i < len; i++) {
        const c = s.charCodeAt(i);
        if (c <= 0xff) {
            ret += reference[c];
        }
        else {
            ret += '&#' + c;
        }
    }
    return ret;
}
exports.$HTML = $HTML;
class HTMLContent extends String {
}
exports.HTMLContent = HTMLContent;
String.prototype.ascii = function ascii() { return $ascii(this); };
String.prototype.camelCase = function camelCase() { return _camelCase(this); };
String.prototype.capitalize = function capitalize() { return $capitalize(this); };
String.prototype.doubleEscape = function doubleEscape(c) { return _doubleEscape(this, c); };
String.prototype.firstCap = function firstCap() { return $firstcap(this); };
String.prototype.ftrim = function ftrim() { return $ftrim(this); };
String.prototype.isDate = function isDate() { return (0, commons_1.$ok)((0, commons_1.$isodate)(this)); };
String.prototype.isEmail = function isEmail() { return (0, commons_1.$ok)((0, commons_1.$email)(this)); };
String.prototype.isNewLine = function isNewLine() { return string_tables_1.FoundationNewLineStringCodeSet.has(this); };
String.prototype.isStrictWhiteSpace = function isStrictWhiteSpace() { return string_tables_1.FoundationStrictWhiteSpacesStringCodeSet.has(this); };
String.prototype.isUrl = function isUrl() { return (0, commons_1.$ok)((0, commons_1.$url)(this)); };
String.prototype.isUUID = function isUUID() { return (0, commons_1.$ok)((0, commons_1.$UUID)(this)); };
String.prototype.isWhiteSpace = function isWhiteSpace() { return string_tables_1.FoundationWhiteSpacesStringCodeSet.has(this); };
String.prototype.left = function left(leftPart) { return $left(this, leftPart); };
String.prototype.lines = function lines(useOnlyASCIISeparators) { return $lines(this, useOnlyASCIISeparators); };
String.prototype.ltrim = function ltrim() { return $ltrim(this); };
String.prototype.normalizeSpaces = function normalizeSpaces(opts) { return $normspaces(this, opts); };
String.prototype.right = function right(rightPart) { return $right(this, rightPart); };
String.prototype.rtrim = function rtrim() { return $rtrim(this); };
String.prototype.singular = function singular() { return this.toUnsigned() === 1; };
String.prototype.snakeCase = function snakeCase() { return _snakeCase(this); };
String.prototype.toDate = function toDate() { return (0, commons_1.$isdate)(this) ? new Date(this) : null; };
String.prototype.toHTML = function toHTML() { return $HTML(this); };
String.prototype.toHTMLContent = function toHTMLContent() { return new HTMLContent(this); };
String.prototype.toInt = function toInt(defaultValue) { return (0, commons_1.$toint)(this, defaultValue); };
String.prototype.toTSDate = function toTSDate() { return tsdate_1.TSDate.fromIsoString(this); };
String.prototype.toPhoneNumber = function toPhoneNumber(defaultCountry) { return tsphonenumber_1.TSPhoneNumber.fromString(this, defaultCountry); };
String.prototype.toUnsigned = function toUnsigned(defaultValue) { return (0, commons_1.$tounsigned)(this, defaultValue); };
HTMLContent.prototype.toHTML = function toHTML() { return $HTML('' + this, string_tables_1.FoundationHTMLStructureEncoding); };
function _doubleEscape(source, esc) {
    let ret = '';
    const esc2 = esc + esc;
    for (let c of source) {
        ret += c === esc ? esc2 : c;
    }
    return ret;
}
function _snakeCase(source) {
    const s = $ascii($normspaces(source, { strict: true, replacer: '' }));
    const len = s.length;
    let ret = '';
    let lastCharWasDash = false;
    for (let i = 0; i < len; i++) {
        const c = s.charAt(i);
        if (c === '_' || c === '-') {
            lastCharWasDash = true;
            continue;
        }
        else if (lastCharWasDash) {
            ret += '_';
        }
        ret += c.toLowerCase();
        lastCharWasDash = false;
    }
    return ret;
}
function _camelCase(source) {
    const s = $ascii($normspaces(source, { strict: true, replacer: '' }));
    const len = s.length;
    let ret = '';
    let lastCharWasDash = false;
    for (let i = 0; i < len; i++) {
        const c = s.charAt(i);
        if (c === '_' || c === '-') {
            lastCharWasDash = true;
            continue;
        }
        else if (lastCharWasDash && c >= 'a' && c <= 'z') {
            ret += c.toUpperCase();
        }
        else {
            ret += c;
        }
        lastCharWasDash = false;
    }
    return ret;
}
function _capitalize(s, max = 0) {
    const len = (0, commons_1.$length)(s);
    let ret = "";
    if (!max) {
        max = len;
    }
    ;
    let lastCharWasNotLetter = true;
    let n = 0;
    for (let i = 0; i < len; i++) {
        const c = s.charAt(i);
        const isLetter = _charAssimilableAsLetter(c);
        if (isLetter && lastCharWasNotLetter && n < max) {
            ret += c.toUpperCase();
            n++;
        }
        else {
            ret += c;
        }
        lastCharWasNotLetter = !isLetter;
    }
    return ret;
}
function _charAssimilableAsLetter(c) {
    c = $ascii(c);
    if (c.length) {
        const v = c.charCodeAt(0) & ~32;
        return v >= 65 && v <= 90;
    }
    return false;
}
//# sourceMappingURL=strings.js.map