import { TSClone, TSLeafInspect, TSObject } from "./tsobject";
import { Comparison, Nullable, uint, uint8 } from "./types";
export interface TSColorToStringOptions {
    removeAlpha?: Nullable<boolean>;
    uppercase?: Nullable<boolean>;
    shortestCSS?: Nullable<boolean>;
    rgbaCSSLike?: Nullable<boolean>;
    colorSpace?: Nullable<TSColorSpace>;
}
export declare enum TSColorSpace {
    RGB = "RGB",
    CMYK = "CMYK",
    Grayscale = "GRAYSCALE"
}
export declare enum TSToGrayScaleMode {
    Brightess = 0,
    Luminosity = 1,
    Luminance = 2
}
declare const customInspectSymbol: unique symbol;
export declare class TSColor implements TSObject, TSLeafInspect, TSClone<TSColor> {
    readonly colorSpace: TSColorSpace;
    private _channels;
    private _alpha;
    private _name?;
    private static __colorsCache;
    private static readonly __colorCacheMaxSize;
    static black(): TSColor;
    static white(): TSColor;
    static cyan(): TSColor;
    static magenta(): TSColor;
    static yellow(): TSColor;
    static red(): TSColor;
    static green(): TSColor;
    static blue(): TSColor;
    static fromString(rgbStringColor: Nullable<string>): TSColor | null;
    static rgb(stringColor: string): TSColor;
    static rgb(colorDefinition: number): TSColor;
    static rgb(R: number, G: number, B: number, A?: number): TSColor;
    static rgbcomponents(r: number, g: number, b: number, opacity?: number): TSColor;
    static cmyk(C: number, M: number, Y: number, K: number, opacity?: number): TSColor;
    static grayscale(whiteIntensity: number, opacity?: number): TSColor;
    static logColorCache(): void;
    private constructor();
    rgb(): [uint8, uint8, uint8];
    rgbComponents(): [number, number, number];
    cmykComponents(): [number, number, number, number];
    grayComponent(): number;
    get name(): string;
    clone(): TSColor;
    get red(): uint8;
    get green(): uint8;
    get blue(): uint8;
    get alpha(): uint8;
    get transparency(): uint8;
    get cyan(): number;
    get magenta(): number;
    get yellow(): number;
    get black(): number;
    get opacity(): number;
    get gray(): number;
    get luminosity(): number;
    get luminance(): number;
    get isPale(): boolean;
    hsl(): [number, number, number];
    hsb(): [number, number, number];
    toAlpha(newAlpha: uint8): TSColor;
    toOpacity(newOpacity: number): TSColor;
    toRGB(): TSColor;
    toCMYK(): TSColor;
    toGrayscale(mode?: TSToGrayScaleMode): TSColor;
    lighterColor(): TSColor;
    darkerColor(): TSColor;
    lightestColor(): TSColor;
    darkestColor(): TSColor;
    matchingColor(): TSColor;
    toNumber(): number;
    toUnsigned(): uint;
    isSimilar(other: TSColor): boolean;
    leafInspect(): string;
    [customInspectSymbol](depth: number, inspectOptions: any, inspect: any): string;
    isEqual(other: any): boolean;
    compare(other: any): Comparison;
    toString(opts?: TSColorToStringOptions): string;
    toJSON(): any;
    toArray(): number[];
    private _modifiedColorsColor;
    private static readonly __TSWebColorNames;
    private static readonly __WebColorsHexParsers;
    private static _cacheRGBColor;
    private static _cachedRGBColor;
    private static _parseHexColorString;
}
export interface TSColorConstructor {
    new (colorSpace: TSColorSpace, channels: number[], alpha: number, name?: string): TSColor;
}
export {};
