"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSCountry = void 0;
const commons_1 = require("./commons");
const tsdefaults_1 = require("./tsdefaults");
const types_1 = require("./types");
const countries_json_1 = __importDefault(require("./countries.json"));
const compare_1 = require("./compare");
const strings_1 = require("./strings");
const customInspectSymbol = Symbol.for('nodejs.util.inspect.custom');
class TSCountry {
    constructor(info, locales, nativeLanguage) {
        this.continent = info.continent;
        this.alpha2Code = info.alpha2Code;
        this.alpha3Code = info.alpha3Code;
        this.names = info.names;
        this.domains = info.domains;
        this.EEC = info.EEC;
        this.spokenLanguages = info.spokenLanguages;
        this.phonePlan = Object.assign({ minDigits: 10, maxDigits: 10, areaCodes: [] }, info.phonePlan);
        this.currency = info.currency;
        this.locales = (0, commons_1.$ok)(info.specificLocales) ? Object.assign(Object.assign({}, locales), info.specificLocales) : Object.assign({}, locales);
        this.nativeLanguage = (0, commons_1.$value)(nativeLanguage, (0, commons_1.$count)(this.spokenLanguages) > 0 ? this.spokenLanguages[0] : this.locales.language);
        this._state = info.state;
        this._vatValidator = TSCountry.CountriesVATInfo[info.alpha2Code];
    }
    static loadCountries(localesMap, managedLanguages) {
        if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
            TSCountry.__countriesMap = new Map();
            TSCountry.__countriesAlpha2Map = new Map();
            TSCountry.__countriesAlpha3Map = new Map();
            TSCountry.__dialCodeMap = new Map();
            TSCountry.__countries = [];
            countries_json_1.default.forEach(info => {
                var _a;
                let loc = (0, commons_1.$ok)(info.localeLanguage) ? localesMap.get(info.localeLanguage) : TSCountry.__EULocale;
                if (!(0, commons_1.$ok)(loc)) {
                    loc = TSCountry.__EULocale;
                }
                const c = new TSCountry(info, loc, info.localeLanguage);
                let codes = TSCountry.__dialCodeMap.get(c.phonePlan.dialCode);
                if (!(0, commons_1.$ok)(codes)) {
                    codes = new Array;
                    TSCountry.__dialCodeMap.set(c.phonePlan.dialCode, codes);
                }
                codes.push(c);
                TSCountry.__countries.push(c);
                TSCountry.__countriesMap.set(c.alpha2Code, c).set(c.alpha3Code, c);
                TSCountry.__countriesAlpha3Map.set(c.alpha3Code, c.alpha2Code);
                TSCountry.__countriesAlpha2Map.set(c.alpha2Code, c.alpha3Code);
                managedLanguages.forEach(l => TSCountry.__countriesMap.set((0, strings_1.$ascii)(c.names[l].toUpperCase()), c));
                (_a = info.aliases) === null || _a === void 0 ? void 0 : _a.forEach(a => TSCountry.__countriesMap.set((0, strings_1.$ascii)(a.toUpperCase()), c));
            });
        }
        return TSCountry.__countriesMap;
    }
    static countriesForDialCode(dc) {
        const array = (0, commons_1.$ok)(dc) ? TSCountry.__dialCodeMap.get(dc) : undefined;
        return (0, commons_1.$value)(array, []);
    }
    static country(c) {
        c = (0, strings_1.$ftrim)(c);
        if (c.length) {
            if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
                tsdefaults_1.TSDefaults.defaults();
            }
            const ret = TSCountry.__countriesMap.get((0, strings_1.$ascii)(c.toUpperCase()));
            if ((0, commons_1.$ok)(ret)) {
                return ret;
            }
        }
        return null;
    }
    static alpha2CodeForAlpha3Code(c) { return (0, commons_1.$length)(c) === 3 ? (0, commons_1.$valueornull)(TSCountry.__countriesAlpha3Map.get(c)) : null; }
    static alpha3CodeForAlpha2Code(c) { return (0, commons_1.$length)(c) === 2 ? (0, commons_1.$valueornull)(TSCountry.__countriesAlpha2Map.get(c)) : null; }
    static alpha2Codes() { return TSCountry.__countriesAlpha2Map.keysArray(); }
    static alpha3Codes() { return TSCountry.__countriesAlpha3Map.keysArray(); }
    static countryForVATNumber(vatNumber) {
        vatNumber = (0, strings_1.$trim)(vatNumber).toUpperCase();
        if (vatNumber.length >= 3) {
            const vatCountry = TSCountry.country(vatNumber.slice(0, 2));
            if ((0, commons_1.$ok)(vatCountry) && vatCountry.validateVATNumber(vatNumber) !== null) {
                return vatCountry;
            }
        }
        return null;
    }
    static countries() {
        if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
            tsdefaults_1.TSDefaults.defaults();
        }
        return [...TSCountry.__countries];
    }
    get label() { return this.names[types_1.Languages.en]; }
    get name() { return this.names[(0, tsdefaults_1.$language)()]; }
    get nativeName() { return this.names[this.nativeLanguage]; }
    get continentLabel() { return (0, tsdefaults_1.$continentName)(this.continent, types_1.Languages.en); }
    get continentName() { return (0, tsdefaults_1.$continentName)(this.continent); }
    get nativeContinentName() { return (0, tsdefaults_1.$continentName)(this.continent, this.nativeLanguage); }
    get language() { return this.locales.language; }
    get languageName() { return this.locales.names[(0, tsdefaults_1.$language)()]; }
    get nativeLanguageName() { return this.locales.names[this.nativeLanguage]; }
    get state() { return (0, commons_1.$value)(TSCountry.country(this._state), this); }
    clone() { return this; }
    translatedName(lang) {
        const ret = this.names[(0, commons_1.$ok)(lang) ? lang : (0, tsdefaults_1.$language)()];
        return (0, commons_1.$length)(ret) ? ret : null;
    }
    validateVATNumber(vatNumber) {
        vatNumber = (0, strings_1.$trim)(vatNumber).toUpperCase();
        if (vatNumber.length < 3) {
            return null;
        }
        return (0, commons_1.$ok)(this._vatValidator) ? this._vatValidator(vatNumber) : undefined;
    }
    toString() { return this.alpha2Code; }
    toJSON() { return this.alpha2Code; }
    toArray() { return [this.alpha2Code, this.alpha3Code]; }
    isEqual(other) { return this === other; }
    compare(other) {
        if (this === other) {
            return types_1.Same;
        }
        if (other instanceof TSCountry) {
            return (0, compare_1.$compare)(this.alpha2Code, other.alpha2Code);
        }
        return undefined;
    }
    leafInspect() { return `<${this.label.capitalize()} (${this.alpha2Code})>`; }
    [customInspectSymbol](depth, inspectOptions, inspect) {
        return this.leafInspect();
    }
}
exports.TSCountry = TSCountry;
TSCountry.__EULocale = {
    language: types_1.Languages.en,
    names: { fr: "anglais", en: "english", de: "englisch", it: "inglese", es: "inglés", pt: "inglês", nl: "Engels", el: "αγγλικά" },
    months: ["January", "February", "March", "April", "May", "June", "July", "August", "Septembre", "Octobre", "Novembre", "Decembre"],
    shortMonths: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
    days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
    shortDays: ["Sun.", "Mon.", "Tue.", "Wed.", "Thu.", "Fri.", "Sat."],
    startingWeekDay: 1,
    dateTimeFormat: "%d/%m/%Y %H:%M:%S",
    dateFormat: "%d/%m/%Y",
    shortDateFormat: "%d/%m/%y",
    shortDateTimeFormat: "%d/%m/%y %H:%M:%S",
    timeFormat: "%H:%M:%S",
    partialTimeFormat: "%H:%M",
    ampm: ['AM', 'PM'],
    continentNames: { 'AF': 'Africa', 'AN': 'Antartica', 'AS': 'Asia', 'EU': 'Europe', 'NA': 'North America', 'OC': 'Oceania', 'SA': 'South America' },
    unitNames: {
        byte: ['Byte', 'Bytes', 'B'],
        second: ['second', 'seconds', 's'],
        minute: ['minute', 'minutes', 'm'],
        hour: ['hour', 'hours', 'h'],
        day: ['day', 'days', 'd'],
        month: ['month', 'months'],
        year: ['year', 'years']
    }
};
TSCountry.CountriesVATInfo = {
    AT: (s) => {
        const m = s.match(/^AT\s*U\s*([0-9]{8})$/);
        return (0, commons_1.$count)(m) === 2 ? `ATU${m[1]}` : null;
    },
    BE: (s) => {
        const m = s.match(/^BE\s*(0)?([0-9]{9})$/);
        return (0, commons_1.$count)(m) === 3 ? `BE0${m[2]}` : null;
    },
    BG: (s) => {
        const m = s.match(/^BG\s*([0-9]{9,10})$/);
        return (0, commons_1.$count)(m) === 2 ? `BG${m[1]}` : null;
    },
    HR: (s) => {
        const m = s.match(/^HR\s*([0-9]{11})$/);
        return (0, commons_1.$count)(m) === 2 ? `HR${m[1]}` : null;
    },
    CY: (s) => {
        const m = s.match(/^CY\s*([0-9]){8}\s*([A-Z])$/);
        return (0, commons_1.$count)(m) === 3 ? `CY${m[1]}${m[2]}` : null;
    },
    CZ: (s) => {
        const m = s.match(/^CZ\s*([0-9]{8,10})$/);
        return (0, commons_1.$count)(m) === 2 ? `CZ${m[1]}` : null;
    },
    DK: (s) => {
        const m = s.match(/^DK\s*([0-9]{8})$/);
        return (0, commons_1.$count)(m) === 2 ? `DK${m[1]}` : null;
    },
    EE: (s) => {
        const m = s.match(/^EE\s*([0-9]{9})$/);
        return (0, commons_1.$count)(m) === 2 ? `EE${m[1]}` : null;
    },
    FI: (s) => {
        const m = s.match(/^FI\s*([0-9]{8})$/);
        return (0, commons_1.$count)(m) === 2 ? `FI${m[1]}` : null;
    },
    FR: (s) => {
        const m = s.match(/^FR\s*([0-9A-Z]{2})\s*([0-9]{9})$/);
        return (0, commons_1.$count)(m) === 3 ? `FR${m[1]}${m[2]}` : null;
    },
    DE: (s) => {
        const m = s.match(/^DE\s*([0-9]{9})$/);
        return (0, commons_1.$count)(m) === 2 ? `DE${m[1]}` : null;
    },
    GR: (s) => {
        const m = s.match(/^(EL|GR)\s*([0-9]{9})$/);
        return (0, commons_1.$count)(m) === 3 ? `EL${m[2]}` : null;
    },
    HU: (s) => {
        const m = s.match(/^HU\s*([0-9]{8})$/);
        return (0, commons_1.$count)(m) === 2 ? `HU${m[1]}` : null;
    },
    IE: (s) => {
        const m = s.match(/^IE\s*([0-9]{7})\s*([A-Z]{2})$/);
        return (0, commons_1.$count)(m) === 3 ? `IE${m[1]}${m[2]}` : null;
    },
    IT: (s) => {
        const m = s.match(/^IT\s*([0-9]{11})$/);
        return (0, commons_1.$count)(m) === 2 ? `IT${m[1]}` : null;
    },
    LV: (s) => {
        const m = s.match(/^LV\s*([0-9]{11})$/);
        return (0, commons_1.$count)(m) === 2 ? `LV${m[1]}` : null;
    },
    LT: (s) => {
        const m = s.match(/^LT\s*([0-9]{9}|[0-9]{12})$/);
        return (0, commons_1.$count)(m) === 2 ? `LT${m[1]}` : null;
    },
    LU: (s) => {
        const m = s.match(/^LU\s*([0-9]{8})$/);
        return (0, commons_1.$count)(m) === 2 ? `LU${m[1]}` : null;
    },
    MT: (s) => {
        const m = s.match(/^MT\s*([0-9]{8})$/);
        return (0, commons_1.$count)(m) === 2 ? `MT${m[1]}` : null;
    },
    NL: (s) => {
        const m = s.match(/^NL\s*([0-9]{9})\s*B\s*([0-9]{2})$/);
        return (0, commons_1.$count)(m) === 3 ? `NL${m[1]}B${m[2]}` : null;
    },
    PL: (s) => {
        const m = s.match(/^PL\s*([0-9]{10})$/);
        return (0, commons_1.$count)(m) === 2 ? `PL${m[1]}` : null;
    },
    PT: (s) => {
        const m = s.match(/^PT\s*([0-9]{3})\s*([0-9]{3})\s*([0-9]{3})$/);
        return (0, commons_1.$count)(m) === 4 ? `PT${m[1]}${m[2]}${m[3]}` : null;
    },
    RO: (s) => {
        const m = s.match(/^RO\s*([0-9]{10})$/);
        return (0, commons_1.$count)(m) === 2 ? `RO${m[1]}` : null;
    },
    SK: (s) => {
        const m = s.match(/^SK\s*([0-9]{10})$/);
        return (0, commons_1.$count)(m) === 2 ? `SK${m[1]}` : null;
    },
    SI: (s) => {
        const m = s.match(/^SI\s*([0-9]{9})$/);
        return (0, commons_1.$count)(m) === 2 ? `SI${m[1]}` : null;
    },
    ES: (s) => {
        const m = s.match(/^ES\s*([0-9A-Z])\s*([0-9]{7})\s*([0-9A-Z])$/);
        return (0, commons_1.$count)(m) === 4 ? `ES${m[1]}${m[2]}${m[3]}` : null;
    },
    SE: (s) => {
        const m = s.match(/^SE\s*([0-9]{12})$/);
        return (0, commons_1.$count)(m) === 2 ? `SE${m[1]}` : null;
    }
};
//# sourceMappingURL=tscountry.js.map