"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$insettimestamp = exports.$weekOfYear = exports.$dayOfWeekFromTimestamp = exports.$dayOfYear = exports.$timestampWithoutTime = exports.$hourFromTimestamp = exports.$minuteFromTimestamp = exports.$secondFromTimestamp = exports.$timestamp = exports.$timeisvalid = exports.$dayisvalid = exports.$isleap = exports.TSMinTimeStamp = exports.TSMaxTimeStamp = exports.TSSecsFrom19700101To20010101 = exports.TSSecsFrom00010101To20010101 = exports.TSDaysFrom00010101To20010101 = exports.TSDaysFrom00000229To20010101 = exports.TSWeek = exports.TSDay = exports.TSHour = exports.TSMinute = exports.TSDate = void 0;
const tsdatecomp_1 = require("./tsdatecomp");
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const types_1 = require("./types");
const number_1 = require("./number");
const tserrors_1 = require("./tserrors");
const customInspectSymbol = Symbol.for('nodejs.util.inspect.custom');
class TSDate {
    constructor() {
        this.toTSDate = this.clone;
        this.toEpochTimestamp = this.toEpoch;
        let n = arguments.length;
        if (n >= 3) {
            if (!$dayisvalid(arguments[0], arguments[1], arguments[2])) {
                tserrors_1.TSError.throw(`TSDate.constructor() : Bad year, month or day argument`, { arguments: Array.from(arguments) });
            }
            if (n !== 3 && n !== 6) {
                tserrors_1.TSError.throw(`TSDate.constructor() : Wrong number (${n}) of argumens`, { arguments: Array.from(arguments) });
            }
            if (n === 6) {
                if (!$timeisvalid(arguments[3], arguments[4], arguments[5])) {
                    tserrors_1.TSError.throw(`TSDate.constructor() : Bad hours, minutes or day seconds`, { arguments: Array.from(arguments) });
                }
                this._timestamp = _insetTimeStamp($timestamp(arguments[0], arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]));
            }
            else {
                this._timestamp = _insetTimeStamp($timestamp(arguments[0], arguments[1], arguments[2], 0, 0, 0));
            }
        }
        else if (n === 2) {
            tserrors_1.TSError.throw(`TSDate.constructor() : Wrong number (${n}) of argumens`, { arguments: Array.from(arguments) });
        }
        else {
            let t = arguments[0];
            if (t === Number.POSITIVE_INFINITY) {
                this._timestamp = exports.TSMaxTimeStamp;
            }
            else if (t === Number.NEGATIVE_INFINITY) {
                this._timestamp = exports.TSMinTimeStamp;
            }
            else if ((0, commons_1.$isnumber)(t)) {
                this._timestamp = $insettimestamp(t);
            }
            else if (t instanceof TSDate) {
                this._timestamp = t.timestamp;
            }
            else {
                let comps = null;
                if ((0, commons_1.$isstring)(t)) {
                    if (t === TSDate.ZULU) {
                        this._timestamp = (0, number_1.$div)(Date.now(), 1000) - exports.TSSecsFrom19700101To20010101;
                        return;
                    }
                    else if (t === TSDate.FUTURE) {
                        this._timestamp = exports.TSMaxTimeStamp;
                        return;
                    }
                    else if (t === TSDate.PAST) {
                        this._timestamp = exports.TSMinTimeStamp;
                        return;
                    }
                    else if (t === TSDate.EPOCH) {
                        this._timestamp = -exports.TSSecsFrom19700101To20010101;
                        return;
                    }
                    else {
                        comps = (0, tsdatecomp_1.$isostring2components)(t);
                    }
                }
                else if (!(0, commons_1.$ok)(t) || t instanceof Date) {
                    comps = (0, tsdatecomp_1.$components)(t);
                }
                if (!(0, commons_1.$ok)(comps)) {
                    tserrors_1.TSError.throw(`Bad TSDate constructor unique argument: ${t}`, { argument: t });
                }
                this._timestamp = _insetTimeStamp((0, tsdatecomp_1.$components2timestamp)(comps));
            }
        }
    }
    static past() { return new TSDate(TSDate.PAST); }
    static future() { return new TSDate(TSDate.FUTURE); }
    static zulu() { return new TSDate(TSDate.ZULU); }
    static epoch() { return new TSDate(TSDate.EPOCH); }
    static isDateSource(value) { return (0, commons_1.$isnumber)(value) || (0, commons_1.$isdate)(value) || value === TSDate.PAST || value === TSDate.FUTURE || value === Number.POSITIVE_INFINITY || value === Number.NEGATIVE_INFINITY; }
    static fromZulu(s) {
        if (!(0, commons_1.$ok)(s)) {
            return null;
        }
        return this.fromComponents({
            year: s.getUTCFullYear(),
            month: (s.getUTCMonth() + 1),
            day: s.getUTCDate(),
            hour: s.getUTCHours(),
            minute: s.getUTCMinutes(),
            second: s.getUTCSeconds(),
            dayOfWeek: s.getUTCDay()
        });
    }
    static fromEpoch(t) {
        return (0, commons_1.$isnumber)(t) ? new TSDate(t - exports.TSSecsFrom19700101To20010101) : null;
    }
    static from(date) {
        if (!(0, commons_1.$ok)(date)) {
            return null;
        }
        if (date === TSDate.FUTURE || date === TSDate.PAST || date === Number.POSITIVE_INFINITY || date === Number.NEGATIVE_INFINITY) {
            return new TSDate(date);
        }
        if ((0, commons_1.$isnumber)(date)) {
            return this.fromTimeStamp(date);
        }
        if (date instanceof Date) {
            return this.fromDate(date);
        }
        return this.fromIsoString(date);
    }
    static fromTimeStamp(t) {
        return (0, commons_1.$isnumber)(t) ? new TSDate(t) : null;
    }
    static fromDate(d) {
        return (0, commons_1.$ok)(d) ? new TSDate(d) : null;
    }
    static fromComponents(comp) {
        if (!(0, tsdatecomp_1.$componentsarevalid)(comp)) {
            return null;
        }
        const c = comp;
        return new TSDate($timestamp(c.year, c.month, c.day, c.hour, c.minute, c.second));
    }
    static fromIsoString(s) {
        if (!(0, commons_1.$ok)(s)) {
            return null;
        }
        return this.fromComponents((0, tsdatecomp_1.$parsedatetime)(s, tsdatecomp_1.TSDateForm.ISO8601));
    }
    static fromString(s, form = tsdatecomp_1.TSDateForm.Standard) {
        return this.fromComponents((0, tsdatecomp_1.$parsedatetime)(s, form));
    }
    static fromDateString(s, form = tsdatecomp_1.TSDateForm.Standard) {
        return this.fromComponents((0, tsdatecomp_1.$parsedate)(s, form));
    }
    clone() { return this; }
    get timestamp() { return this._timestamp; }
    isLeap() { return $isleap((0, tsdatecomp_1.$components)(this._timestamp).year); }
    isFuture() { return this._timestamp >= exports.TSMaxTimeStamp; }
    isPast() { return this._timestamp <= exports.TSMinTimeStamp; }
    isFinite() { return !this.isPast() && !this.isFuture(); }
    dateWithoutTime() { return new TSDate($timestampWithoutTime(this._timestamp)); }
    timeSinceDate(d) { return this._timestamp - d.timestamp; }
    get year() { return (0, tsdatecomp_1.$components)(this._timestamp).year; }
    get month() { return (0, tsdatecomp_1.$components)(this._timestamp).month; }
    get day() { return (0, tsdatecomp_1.$components)(this._timestamp).day; }
    get hour() { return $hourFromTimestamp(this._timestamp); }
    get minute() { return $minuteFromTimestamp(this._timestamp); }
    get second() { return $secondFromTimestamp(this._timestamp); }
    week(offset = 0) { return $weekOfYear(this._timestamp, offset); }
    dayOfYear() { return $dayOfYear(this._timestamp); }
    dayOfWeek(offset = 0) { return $dayOfWeekFromTimestamp(this._timestamp, offset); }
    lastDayOfMonth() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return _lastDayOfMonth(c.year, c.month); }
    dateByAdding(years, months = 0, days = 0, hours = 0, minutes = 0, seconds = 0) {
        if (!(0, commons_1.$isint)(years) || !(0, commons_1.$isint)(months) || !(0, commons_1.$isint)(days) || !(0, commons_1.$isint)(hours) || !(0, commons_1.$isint)(minutes) || !(0, commons_1.$isint)(seconds)) {
            tserrors_1.TSError.throw(`TSDate.dateByAdding() has at lease one non integer argument`, { years: years, months: months, days: days, hours: hours, minutes: minutes, seconds: seconds });
        }
        let ts = this._timestamp + days * exports.TSDay + hours * exports.TSHour + minutes * exports.TSMinute + seconds;
        if (!years && !months) {
            return new TSDate(ts);
        }
        let comp = (0, tsdatecomp_1.$components)(ts);
        const yearsInMonths = (0, number_1.$div)(Math.abs(months), 12);
        years += months > 0 ? yearsInMonths : -yearsInMonths;
        months += yearsInMonths * (months > 0 ? -12 : 12);
        years += comp.year;
        months += comp.month;
        if (months > 12) {
            years++;
            months -= 12;
        }
        if (months < 1) {
            years--;
            months += 12;
        }
        if (years < 1) {
            return TSDate.past();
        }
        if (comp.day > TSDaysInMonth[months]) {
            comp.day = (months == 2 && $isleap(years) ? 29 : TSDaysInMonth[months]);
        }
        return new TSDate($timestamp(years, months, comp.day, comp.hour, comp.minute, comp.second));
    }
    dateByAddingYears(years) { return this.dateByAdding(years); }
    dateByAddingMonths(months) { return this.dateByAdding(0, months); }
    dateByAddingWeeks(weeks) { return new TSDate(this._timestamp + weeks * exports.TSWeek); }
    dateByAddingDays(days) { return new TSDate(this._timestamp + days * exports.TSDay); }
    dateByAddingHours(hours) { return new TSDate(this._timestamp + hours * exports.TSHour); }
    dateByAddingMinutes(mn) { return new TSDate(this._timestamp + mn * exports.TSMinute); }
    dateByAddingSeconds(seconds) { return new TSDate(this._timestamp + seconds); }
    dateByAddingTime(time) { return new TSDate(this._timestamp + time); }
    firstDateOfYear() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, 1, 1); }
    lastDateOfYear() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, 12, 31); }
    firstDateOfMonth() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, c.month, 1); }
    lastDateOfMonth() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, c.month, _lastDayOfMonth(c.year, c.month)); }
    firstDateOfWeek(offset = 0) {
        const ts = $timestampWithoutTime(this._timestamp);
        const dayOfWeek = $dayOfWeekFromTimestamp(ts, offset);
        return new TSDate(ts - dayOfWeek * exports.TSDay);
    }
    ;
    daysSinceDate(aDate) {
        return (0, number_1.$div)($timestampWithoutTime(this._timestamp) - $timestampWithoutTime(aDate._timestamp), exports.TSDay);
    }
    toNumber() { return this._timestamp; }
    toEpoch() { return this._timestamp + exports.TSSecsFrom19700101To20010101; }
    toComponents() { return (0, tsdatecomp_1.$components)(this._timestamp); }
    toDate() { return (0, tsdatecomp_1.$components2date)((0, tsdatecomp_1.$components)(this._timestamp)); }
    toIsoString(format = tsdatecomp_1.TSDateForm.ISO8601) {
        return (0, tsdatecomp_1.$components2string)((0, tsdatecomp_1.$components)(this._timestamp), format);
    }
    toISOString() {
        return (0, tsdatecomp_1.$components2StringWithOffset)((0, tsdatecomp_1.$components)(this._timestamp), { forceZ: true, milliseconds: 0 });
    }
    compare(other) {
        if (this === other) {
            return types_1.Same;
        }
        if (other instanceof TSDate) {
            return (0, compare_1.$numcompare)(this._timestamp, other.timestamp);
        }
        if (other instanceof Date) {
            return (0, compare_1.$datecompare)(this, other);
        }
        return undefined;
    }
    isEqual(other) {
        if (this === other) {
            return true;
        }
        if (other instanceof TSDate) {
            return this._timestamp === other.timestamp;
        }
        if (other instanceof Date) {
            return (0, compare_1.$datecompare)(other, this) === types_1.Same;
        }
        return false;
    }
    toString(format, locale) {
        if (!(0, commons_1.$ok)(format) && !(0, commons_1.$ok)(locale)) {
            return this.toIsoString();
        }
        return this._toString(false, format, locale);
    }
    toLocaleString(format, locale) {
        return this._toString(true, format, locale);
    }
    toJSON() { return this.toIsoString(); }
    toArray() { return [this]; }
    leafInspect() { return this.toISOString(); }
    [customInspectSymbol](depth, inspectOptions, inspect) {
        return this.leafInspect();
    }
    _toString(localTime, format, locale) {
        const offset = localTime ? -(this.toDate()).getTimezoneOffset() * exports.TSMinute : 0;
        if ((0, commons_1.$isstring)(format)) {
            return (0, commons_1.$value)((0, tsdatecomp_1.$components2stringformat)((0, tsdatecomp_1.$components)(this._timestamp + offset), format, locale), '');
        }
        if (!(0, commons_1.$ok)(format)) {
            format = tsdatecomp_1.TSDateForm.Standard;
        }
        return (0, tsdatecomp_1.$components2string)((0, tsdatecomp_1.$components)(this._timestamp + offset), format);
    }
}
exports.TSDate = TSDate;
TSDate.FUTURE = 'future';
TSDate.PAST = 'past';
TSDate.ZULU = 'zulu';
TSDate.EPOCH = 'epoch';
exports.TSMinute = 60;
exports.TSHour = 3600;
exports.TSDay = 86400;
exports.TSWeek = 604800;
exports.TSDaysFrom00000229To20010101 = 730792;
exports.TSDaysFrom00010101To20010101 = 730485;
exports.TSSecsFrom00010101To20010101 = 63113904000;
exports.TSSecsFrom19700101To20010101 = 978307200;
exports.TSMaxTimeStamp = 8639021692800;
exports.TSMinTimeStamp = -exports.TSSecsFrom00010101To20010101;
function $isleap(y) {
    return !(0, commons_1.$isint)(y) || y % 4 ? false : (y % 100 ? (y > 7 ? true : false) : (y % 400 || y < 1600 ? false : true));
}
exports.$isleap = $isleap;
function $dayisvalid(year, month, day) {
    if (!(0, commons_1.$isint)(day) || !(0, commons_1.$isint)(month) || !(0, commons_1.$isint)(year) || day < 1 || day > 31 || month < 1 || month > 12) {
        return false;
    }
    if (day > TSDaysInMonth[month]) {
        return (month === 2 && day === 29 && $isleap(year)) ? true : false;
    }
    return true;
}
exports.$dayisvalid = $dayisvalid;
function $timeisvalid(hour, minute, second) {
    return ((0, commons_1.$isint)(hour) && (0, commons_1.$isint)(minute) && (0, commons_1.$isnumber)(second) && hour >= 0 && hour < 24 && minute >= 0 && minute < 60 && second >= 0 && second < 60);
}
exports.$timeisvalid = $timeisvalid;
function $timestamp(year, month, day, hours = 0, minutes = 0, seconds = 0) {
    if (!$dayisvalid(year, month, day)) {
        tserrors_1.TSError.throw(`$timestamp(${year}, ${month}, ${day}): wrong day definition arguments`, { arguments: Array.from(arguments) });
    }
    if (!$timeisvalid(hours, minutes, seconds)) {
        tserrors_1.TSError.throw(`$timestamp(${year}, ${month}, ${day}, ${hours}, ${minutes}, ${seconds}): wrong time arguments`, { arguments: Array.from(arguments) });
    }
    if (month < 3) {
        month += 12;
        year--;
    }
    let leaps = Math.floor(year / 4) - Math.floor(year / 100) + Math.floor(year / 400);
    return Math.floor((day + TSDaysInPreviousMonth[month] + 365 * year + leaps - exports.TSDaysFrom00000229To20010101) * exports.TSDay) + hours * exports.TSHour + minutes * exports.TSMinute + seconds;
}
exports.$timestamp = $timestamp;
function $secondFromTimestamp(ts) { return (((0, commons_1.$toint)(ts) + exports.TSSecsFrom00010101To20010101) % exports.TSMinute); }
exports.$secondFromTimestamp = $secondFromTimestamp;
function $minuteFromTimestamp(ts) { return (0, number_1.$div)(((0, commons_1.$toint)(ts) + exports.TSSecsFrom00010101To20010101) % exports.TSHour, exports.TSMinute); }
exports.$minuteFromTimestamp = $minuteFromTimestamp;
function $hourFromTimestamp(ts) { return (0, number_1.$div)(((0, commons_1.$toint)(ts) + exports.TSSecsFrom00010101To20010101) % exports.TSDay, exports.TSHour); }
exports.$hourFromTimestamp = $hourFromTimestamp;
function $timestampWithoutTime(ts) {
    ts = (0, commons_1.$toint)(ts);
    const overhead = (ts + exports.TSSecsFrom00010101To20010101) % exports.TSDay;
    return ts - overhead;
}
exports.$timestampWithoutTime = $timestampWithoutTime;
function $dayOfYear(ts) {
    let c = (0, tsdatecomp_1.$components)(ts);
    return Math.floor((ts - $timestamp(c.year, 1, 1)) / exports.TSDay) + 1;
}
exports.$dayOfYear = $dayOfYear;
function $dayOfWeekFromTimestamp(ts, offset = 0) {
    return (Math.floor(ts / exports.TSDay) + exports.TSDaysFrom00010101To20010101 + 8 - offset) % 7;
}
exports.$dayOfWeekFromTimestamp = $dayOfWeekFromTimestamp;
function $weekOfYear(ts, offset = 0) {
    let c = (0, tsdatecomp_1.$components)(ts);
    offset %= 7;
    let ref = _yearReference(c.year, offset);
    if (ts < ref) {
        ref = _yearReference(c.year - 1, offset);
    }
    let week = Math.floor((ts - ref) / exports.TSWeek) + 1;
    if (week === 53) {
        ref += 52 * exports.TSWeek;
        c = (0, tsdatecomp_1.$components)(ref);
        if (c.day >= 29) {
            week = 1;
        }
    }
    return week;
}
exports.$weekOfYear = $weekOfYear;
function $insettimestamp(ts) { return _insetTimeStamp((0, commons_1.$toint)(ts)); }
exports.$insettimestamp = $insettimestamp;
const TSDaysInMonth = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
const TSDaysInPreviousMonth = [0, 0, 0, 0, 31, 61, 92, 122, 153, 184, 214, 245, 275, 306, 337];
function _insetTimeStamp(ts) { return Math.min(Math.max(exports.TSMinTimeStamp, ts), exports.TSMaxTimeStamp); }
function _lastDayOfMonth(year, month) {
    return (month === 2 && $isleap(year)) ? 29 : TSDaysInMonth[month];
}
function _yearReference(y, offset = 0) {
    const firstDayOfYear = $timestamp(y, 1, 1);
    let d = $dayOfWeekFromTimestamp(firstDayOfYear, offset);
    d = (d <= 3 ? -d : 7 - d);
    return firstDayOfYear + d * exports.TSDay;
}
//# sourceMappingURL=tsdate.js.map