"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$durationNumber2StringFormat = exports.$durationDescription = exports.$duration2String = exports.$duration = exports.$durationcomponents = exports.$components2stringformat = exports.$components2StringWithOffset = exports.$components2string = exports.$components2date = exports.$components2timestamp = exports.$isostring2components = exports.$parsedate = exports.$parsedatetime = exports.$parsetime = exports.$componentshavetime = exports.$componentsarevalid = exports.$components = exports.$timecomponents = exports.TSDateRep = exports.TSDateForm = void 0;
const commons_1 = require("./commons");
const tsdefaults_1 = require("./tsdefaults");
const tsdate_1 = require("./tsdate");
const types_1 = require("./types");
const tscountry_1 = require("./tscountry");
const number_1 = require("./number");
const strings_1 = require("./strings");
const utils_1 = require("./utils");
const tserrors_1 = require("./tserrors");
var TSDateForm;
(function (TSDateForm) {
    TSDateForm[TSDateForm["Standard"] = 0] = "Standard";
    TSDateForm[TSDateForm["English"] = 1] = "English";
    TSDateForm[TSDateForm["Computer"] = 2] = "Computer";
    TSDateForm[TSDateForm["ISO8601"] = 3] = "ISO8601";
    TSDateForm[TSDateForm["ISO8601C"] = 4] = "ISO8601C";
    TSDateForm[TSDateForm["ISO8601L"] = 5] = "ISO8601L";
})(TSDateForm = exports.TSDateForm || (exports.TSDateForm = {}));
var TSDateRep;
(function (TSDateRep) {
    TSDateRep["LocalTime"] = "lt";
    TSDateRep["LocalDate"] = "ld";
    TSDateRep["LocalDateTime"] = "ldt";
    TSDateRep["ShortLocalDate"] = "sld";
    TSDateRep["ShortLocalDateTime"] = "sldt";
})(TSDateRep = exports.TSDateRep || (exports.TSDateRep = {}));
function $timecomponents(source) {
    if (!(0, commons_1.$ok)(source)) {
        source = new Date();
    }
    else if (source instanceof tsdate_1.TSDate) {
        source = source.timestamp;
    }
    if ((0, commons_1.$isnumber)(source)) {
        const timestamp = source;
        return {
            hour: (0, tsdate_1.$hourFromTimestamp)(timestamp),
            minute: (0, tsdate_1.$minuteFromTimestamp)(timestamp),
            second: (0, tsdate_1.$secondFromTimestamp)(timestamp),
        };
    }
    const d = source;
    return {
        hour: d.getHours(),
        minute: d.getMinutes(),
        second: d.getSeconds(),
    };
}
exports.$timecomponents = $timecomponents;
function $components(source) {
    if (!(0, commons_1.$ok)(source)) {
        source = new Date();
    }
    else if (source instanceof tsdate_1.TSDate) {
        source = source.timestamp;
    }
    if (source === Number.NEGATIVE_INFINITY) {
        source = tsdate_1.TSMinTimeStamp;
    }
    else if (source === Number.POSITIVE_INFINITY) {
        source = tsdate_1.TSMaxTimeStamp;
    }
    if ((0, commons_1.$isnumber)(source)) {
        const timestamp = Math.max(Math.min(source, tsdate_1.TSMaxTimeStamp), tsdate_1.TSMinTimeStamp);
        let Z = Math.floor(timestamp / tsdate_1.TSDay) + tsdate_1.TSDaysFrom00000229To20010101;
        let gg = Z - 0.25;
        let CENTURY = Math.floor(gg / 36524.25);
        let CENTURY_MQUART = CENTURY - Math.floor(CENTURY / 4);
        let ALLDAYS = gg + CENTURY_MQUART;
        let Y = Math.floor(ALLDAYS / 365.25);
        let Y365 = Math.floor(Y * 365.25);
        let DAYS_IN_Y = CENTURY_MQUART + Z - Y365;
        let MONTH_IN_Y = Math.floor((5 * DAYS_IN_Y + 456) / 153);
        return {
            day: Math.floor(DAYS_IN_Y - Math.floor((153 * MONTH_IN_Y - 457) / 5)),
            hour: (0, tsdate_1.$hourFromTimestamp)(timestamp),
            minute: (0, tsdate_1.$minuteFromTimestamp)(timestamp),
            second: (0, tsdate_1.$secondFromTimestamp)(timestamp),
            dayOfWeek: ((Z + 2) % 7),
            month: (MONTH_IN_Y > 12 ? MONTH_IN_Y - 12 : MONTH_IN_Y),
            year: (MONTH_IN_Y > 12 ? Y + 1 : Y)
        };
    }
    if (!(source instanceof Date)) {
        tserrors_1.TSError.throw(`$components() source is not a valid number, nor a Date nor a TSDate`, { source: source });
    }
    const d = source;
    return {
        year: d.getFullYear(),
        month: (d.getMonth() + 1),
        day: d.getDate(),
        hour: d.getHours(),
        minute: d.getMinutes(),
        second: d.getSeconds(),
        dayOfWeek: d.getDay()
    };
}
exports.$components = $components;
function $componentsarevalid(comp) {
    if (!(0, commons_1.$ok)(comp)) {
        return false;
    }
    ;
    const c = comp;
    return (0, tsdate_1.$dayisvalid)(c.year, c.month, c.day) && (0, tsdate_1.$timeisvalid)(c.hour, c.minute, c.second);
}
exports.$componentsarevalid = $componentsarevalid;
function $componentshavetime(c) {
    return c.hour > 0 || c.minute > 0 || c.second > 0;
}
exports.$componentshavetime = $componentshavetime;
function $parsetime(s) {
    if (!(0, commons_1.$length)(s)) {
        return null;
    }
    const m = s.match(/^\s*(\d{1,6})(\s*[:.]\s*(\d{1,2})(\s*[:.]\s*(\d{1,2}))?)?\s*$/);
    if (!(0, commons_1.$ok)(m)) {
        return null;
    }
    const res = m;
    const c = { hour: types_1.UINT_MIN, minute: types_1.UINT_MIN, second: types_1.UINT_MIN };
    _parseTime(c, res, 0, res.length);
    return (0, tsdate_1.$timeisvalid)(c.hour, c.minute, c.second) ? c : null;
}
exports.$parsetime = $parsetime;
function $parsedatetime(s, form = TSDateForm.Standard) {
    return _parsedt(s, /^\s*(\d{1,8})([/\-.](\d{1,2})([/\-.](\d{1,4})(\s+(\d{1,6})(\s*[:.]\s*(\d{1,2})(\s*[:.]\s*(\d{1,2}))?)?)?)?)?\s*$/, form);
}
exports.$parsedatetime = $parsedatetime;
function $parsedate(s, form = TSDateForm.Standard) {
    return _parsedt(s, /^\s*(\d{1,8})([/\-.](\d{1,2})([/\-.](\d{1,4}))?)?\s*$/, form, { noTime: true });
}
exports.$parsedate = $parsedate;
const NO_TIME_ISO_REGEX = /^([0-9]{2,6})\-([0-9]{1,2})\-([0-9]{1,2})$/;
const TIME_ISO_REGEX = /^([0-9]{2,6})\-([0-9]{1,2})\-([0-9]{1,2})([Tt]([0-9]{1,2})(:([0-9]{1,2})(:([0-9]{1,2})(\.000)?)?)?([zZ]|\+00(:?00)?)?)?$/;
const COMPACT_NO_TIME_ISO_REGEX = /^([0-9]{4})([0-9]{2})([0-9]{2})$/;
const COMPACT_TIME_ISO_REGEX = /^([0-9]{4})([0-9]{2})([0-9]{2})([Tt]([0-9]{2})(([0-9]{2})(([0-9]{2})(\.000)?)?)?([zZ]|\+00(:?00)?)?)?$/;
function $isostring2components(source, opts = {}) {
    const s = (0, strings_1.$ftrim)(source);
    if (!s.length) {
        return null;
    }
    let m = undefined;
    if (opts.noTime) {
        m = s.match(NO_TIME_ISO_REGEX);
        if (!(0, commons_1.$ok)(m)) {
            m = s.match(COMPACT_NO_TIME_ISO_REGEX);
        }
    }
    else {
        m = s.match(TIME_ISO_REGEX);
        if (!(0, commons_1.$ok)(m)) {
            m = s.match(COMPACT_TIME_ISO_REGEX);
        }
    }
    if (!(0, commons_1.$ok)(m)) {
        return null;
    }
    const res = m;
    const l = res.length;
    const c = {
        year: _yearFrom(res[1]),
        month: (0, commons_1.$unsigned)(res[2]),
        day: (0, commons_1.$unsigned)(res[3]),
        hour: l > 5 ? (0, commons_1.$unsigned)(res[5]) : types_1.UINT_MIN,
        minute: l > 7 ? (0, commons_1.$unsigned)(res[7]) : types_1.UINT_MIN,
        second: l > 9 ? (0, commons_1.$unsigned)(res[9]) : types_1.UINT_MIN
    };
    return $componentsarevalid(c) ? c : null;
}
exports.$isostring2components = $isostring2components;
function $components2timestamp(c) {
    return (0, tsdate_1.$timestamp)(c.year, c.month, c.day, c.hour, c.minute, c.second);
}
exports.$components2timestamp = $components2timestamp;
function $components2date(c) {
    return new Date(c.year, c.month - 1, c.day, c.hour, c.minute, c.second, 0);
}
exports.$components2date = $components2date;
function $components2string(c, form = TSDateForm.Standard) {
    switch (form) {
        case TSDateForm.ISO8601:
            return `${(0, number_1.$fpad4)(c.year)}-${(0, number_1.$fpad2)(c.month)}-${(0, number_1.$fpad2)(c.day)}T${(0, number_1.$fpad2)(c.hour)}:${(0, number_1.$fpad2)(c.minute)}:${(0, number_1.$fpad2)(c.second)}`;
        case TSDateForm.ISO8601C:
            return `${(0, number_1.$fpad4)(c.year)}${(0, number_1.$fpad2)(c.month)}${(0, number_1.$fpad2)(c.day)}T${(0, number_1.$fpad2)(c.hour)}${(0, number_1.$fpad2)(c.minute)}${(0, number_1.$fpad2)(c.second)}`;
        case TSDateForm.ISO8601L:
            return `${(0, number_1.$fpad)(c.year, 6)}-${(0, number_1.$fpad2)(c.month)}-${(0, number_1.$fpad2)(c.day)}T${(0, number_1.$fpad2)(c.hour)}:${(0, number_1.$fpad2)(c.minute)}:${(0, number_1.$fpad2)(c.second)}`;
        case TSDateForm.Standard:
            return $componentshavetime(c) ? `${(0, number_1.$fpad2)(c.day)}/${(0, number_1.$fpad2)(c.month)}/${(0, number_1.$fpad4)(c.year)}-${(0, number_1.$fpad2)(c.hour)}:${(0, number_1.$fpad2)(c.minute)}:${(0, number_1.$fpad2)(c.second)}`
                : `${(0, number_1.$fpad2)(c.day)}/${(0, number_1.$fpad2)(c.month)}/${(0, number_1.$fpad4)(c.year)}`;
        case TSDateForm.English:
            return $componentshavetime(c) ? `${(0, number_1.$fpad2)(c.month)}/${(0, number_1.$fpad2)(c.day)}/${(0, number_1.$fpad4)(c.year)}-${(0, number_1.$fpad2)(c.hour)}:${(0, number_1.$fpad2)(c.minute)}:${(0, number_1.$fpad2)(c.second)}`
                : `${(0, number_1.$fpad2)(c.month)}/${(0, number_1.$fpad2)(c.day)}/${(0, number_1.$fpad4)(c.year)}`;
        case TSDateForm.Computer:
            return $componentshavetime(c) ? `${(0, number_1.$fpad4)(c.year)}/${(0, number_1.$fpad2)(c.month)}/${(0, number_1.$fpad4)(c.day)}-${(0, number_1.$fpad2)(c.hour)}:${(0, number_1.$fpad2)(c.minute)}:${(0, number_1.$fpad2)(c.second)}`
                : `${(0, number_1.$fpad4)(c.year)}/${(0, number_1.$fpad2)(c.month)}/${(0, number_1.$fpad4)(c.day)}`;
    }
}
exports.$components2string = $components2string;
;
function $components2StringWithOffset(c, opts = {}) {
    if (!(0, commons_1.$ok)(opts.form)) {
        opts.form = TSDateForm.ISO8601;
    }
    if (!(0, commons_1.$ok)(opts.minutesOffset)) {
        opts.minutesOffset = 0;
    }
    if (!(0, commons_1.$isint)(opts.minutesOffset)) {
        tserrors_1.TSError.throw('$components2StringWithOffset() : output time zone offset must be a plain unsigned integer', { timeZoneOffset: opts.minutesOffset });
    }
    const m = opts.minutesOffset;
    const a = Math.abs(m);
    if (a % 15 !== 0) {
        tserrors_1.TSError.throw('$components2StringWithOffset() : output time zone offset must be multiple of 15', { timeZoneOffset: opts.minutesOffset });
    }
    if ((0, commons_1.$ok)(opts.milliseconds) && !(0, commons_1.$isunsigned)(opts.milliseconds, 999)) {
        tserrors_1.TSError.throw('$components2StringWithOffset() : milliseconds offset should be in 0...999 range', { millisecondsOffset: opts.milliseconds });
    }
    let s = $components2string(c, opts.form);
    if ((0, commons_1.$ok)(opts.milliseconds)) {
        s += '.' + (0, number_1.$fpad3)(opts.milliseconds);
    }
    if (opts.form === TSDateForm.ISO8601C) {
        if (!a) {
            return s + 'Z';
        }
        return a % 60 === 0 ? `${s}${m < 0 ? '-' : '+'}${(0, number_1.$fpad2)((0, number_1.$div)(a, 60))}` :
            `${s}${m < 0 ? '-' : '+'}${(0, number_1.$fpad2)((0, number_1.$div)(a, 60))}${(0, number_1.$fpad2)((a % 60))}`;
    }
    if (!a && opts.forceZ) {
        return s + 'Z';
    }
    return `${s}${m < 0 ? '-' : '+'}${(0, number_1.$fpad2)((0, number_1.$div)(a, 60))}:${(0, number_1.$fpad2)((a % 60))}`;
}
exports.$components2StringWithOffset = $components2StringWithOffset;
function $components2stringformat(comp, format = '', locale) {
    if (!$componentsarevalid(comp)) {
        return null;
    }
    const ts = $components2timestamp(comp);
    const trs = !(0, commons_1.$ok)(locale) || (0, commons_1.$isstring)(locale) || (locale instanceof tscountry_1.TSCountry) ? (0, tsdefaults_1.$locales)(locale) : locale;
    let fmtlen = (0, commons_1.$length)(format);
    let ret = "";
    let escape = false;
    let skipIncludedIfNoTime = false;
    if (!fmtlen) {
        format = trs.dateTimeFormat;
    }
    else {
        switch (format) {
            case TSDateRep.LocalDate:
                format = trs.dateFormat;
                break;
            case TSDateRep.ShortLocalDate:
                format = trs.shortDateFormat;
                break;
            case TSDateRep.LocalDateTime:
                format = trs.dateTimeFormat;
                break;
            case TSDateRep.ShortLocalDateTime:
                format = trs.shortDateTimeFormat;
                break;
            case TSDateRep.LocalTime:
                format = trs.timeFormat;
                break;
            default:
                break;
        }
    }
    fmtlen = format.length;
    for (let i = 0; i < fmtlen; i++) {
        const c = format.charAt(i);
        if (escape) {
            escape = false;
            if (skipIncludedIfNoTime) {
                if (c === ']') {
                    skipIncludedIfNoTime = false;
                }
            }
            else {
                switch (c) {
                    case '%':
                        ret += '%';
                        break;
                    case 'a':
                        ret += trs.shortDays[(0, tsdate_1.$dayOfWeekFromTimestamp)(ts)];
                        break;
                    case 'A':
                        ret += trs.days[(0, tsdate_1.$dayOfWeekFromTimestamp)(ts)];
                        break;
                    case 'b':
                        ret += trs.shortMonths[comp.month - 1];
                        break;
                    case 'B':
                        ret += trs.months[comp.month - 1];
                        break;
                    case 'd':
                        ret += (0, number_1.$fpad2)(comp.day);
                        break;
                    case 'e':
                        ret += comp.day;
                        break;
                    case 'f':
                        ret += (0, tsdate_1.$dayOfWeekFromTimestamp)(ts);
                        break;
                    case 'F':
                        ret += (0, tsdate_1.$dayOfWeekFromTimestamp)(ts, trs.startingWeekDay);
                        break;
                    case 'H':
                        ret += (0, number_1.$fpad2)(comp.hour);
                        break;
                    case 'J':
                        ret += (0, number_1.$fpad2)((comp.hour > 12 ? comp.hour - 12 : comp.hour));
                        break;
                    case 'I':
                        ret += comp.hour;
                        break;
                    case 'K':
                        ret += comp.hour > 12 ? comp.hour - 12 : comp.hour;
                        break;
                    case 'm':
                        ret += (0, number_1.$fpad2)(comp.month);
                        break;
                    case 'M':
                        ret += (0, number_1.$fpad2)(comp.minute);
                        break;
                    case 'N':
                        ret += comp.minute;
                        break;
                    case 'n':
                        ret += comp.month;
                        break;
                    case 'p': {
                        const s = $components2stringformat(comp, trs.partialTimeFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'P':
                        ret += trs.ampm[comp.hour <= 12 ? 0 : 1];
                        break;
                    case 'q':
                        ret += (0, number_1.$fpad3)((0, tsdate_1.$dayOfYear)(ts));
                        break;
                    case 'r':
                        ret += (0, tsdate_1.$dayOfYear)(ts);
                        break;
                    case 'S':
                        ret += (0, number_1.$fpad2)(comp.second);
                        break;
                    case 'T':
                        ret += comp.second;
                        break;
                    case 't': {
                        const s = $components2stringformat(comp, trs.timeFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'v':
                        ret += (0, number_1.$fpad2)((0, tsdate_1.$weekOfYear)(ts, trs.startingWeekDay));
                        break;
                    case 'w':
                        ret += (0, tsdate_1.$weekOfYear)(ts, trs.startingWeekDay);
                        break;
                    case 'x': {
                        const s = $components2stringformat(comp, trs.shortDateFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'X': {
                        const s = $components2stringformat(comp, trs.dateFormat, locale);
                        if (!(0, commons_1.$ok)(s)) {
                            return null;
                        }
                        ret += s;
                        break;
                    }
                    case 'y':
                        ret += (0, number_1.$fpad2)((comp.year % 100));
                        break;
                    case 'Y':
                        ret += (0, number_1.$fpad4)(comp.year);
                        break;
                    case 'z':
                        ret += comp.year;
                        break;
                    case '[':
                        skipIncludedIfNoTime = comp.hour == 0 && comp.minute == 0 && comp.second == 0;
                        break;
                    case ']':
                        skipIncludedIfNoTime = false;
                        break;
                    default:
                        ret += '%';
                        ret += c;
                        break;
                }
            }
        }
        else if (c === '%') {
            escape = true;
        }
        else {
            escape = false;
            if (!skipIncludedIfNoTime) {
                ret += c;
            }
        }
    }
    if (escape && !skipIncludedIfNoTime) {
        ret += '%';
    }
    return ret;
}
exports.$components2stringformat = $components2stringformat;
function $durationcomponents(duration) {
    if ((0, commons_1.$ok)(duration) && duration < 0) {
        tserrors_1.TSError.throw('$durationcomponents() : duration must be positive or 0', { duration: duration });
    }
    let time = (0, commons_1.$tounsigned)(duration);
    let d, h, m;
    d = (0, number_1.$div)(time, tsdate_1.TSDay);
    time -= d * tsdate_1.TSDay;
    h = (0, number_1.$div)(time, tsdate_1.TSHour);
    time -= h * tsdate_1.TSHour;
    m = (0, number_1.$div)(time, tsdate_1.TSMinute);
    return {
        days: d,
        hours: h,
        minutes: m,
        seconds: (time - m * tsdate_1.TSMinute)
    };
}
exports.$durationcomponents = $durationcomponents;
function $duration(comps) {
    return comps.days * tsdate_1.TSDay + comps.hours * tsdate_1.TSHour + comps.minutes * tsdate_1.TSMinute + comps.seconds;
}
exports.$duration = $duration;
function $duration2String(comps, format) {
    return $durationNumber2StringFormat($duration(comps), format);
}
exports.$duration2String = $duration2String;
function $durationDescription(comps, opts) {
    let duration = typeof comps === 'number' ? comps : $duration(comps);
    let c = (0, commons_1.$isunsigned)(duration) ? $durationcomponents(duration) : null;
    if ((0, commons_1.$ok)(c)) {
        switch (opts === null || opts === void 0 ? void 0 : opts.depth) {
            case 'days':
                if ((duration % tsdate_1.TSDay) >= tsdate_1.TSDay / 2) {
                    c.days++;
                }
                c.hours = types_1.UINT_MIN;
                c.minutes = types_1.UINT_MIN;
                c.seconds = types_1.UINT_MIN;
                break;
            case 'days-cut':
                c.hours = types_1.UINT_MIN;
                c.minutes = types_1.UINT_MIN;
                c.seconds = types_1.UINT_MIN;
                break;
            case 'hours':
                if ((duration % tsdate_1.TSHour) >= tsdate_1.TSHour / 2) {
                    c.hours++;
                    if (c.hours === 24) {
                        c.days++;
                        c.hours = types_1.UINT_MIN;
                    }
                }
                c.minutes = types_1.UINT_MIN;
                c.seconds = types_1.UINT_MIN;
                break;
            case 'hours-cut':
                c.minutes = types_1.UINT_MIN;
                c.seconds = types_1.UINT_MIN;
                break;
            case 'minutes':
                if (c.seconds >= 30) {
                    c.minutes++;
                    if (c.minutes === 60) {
                        c.hours++;
                        c.minutes = types_1.UINT_MIN;
                    }
                    if (c.hours === 24) {
                        c.days++;
                        c.hours = types_1.UINT_MIN;
                    }
                }
                c.seconds = types_1.UINT_MIN;
                break;
            case 'minutes-cut':
                c.seconds = types_1.UINT_MIN;
                break;
            default:
                break;
        }
        const a = [];
        const locale = opts === null || opts === void 0 ? void 0 : opts.locale;
        function _addRep(key, value, defaultDefinition) {
            const def = (0, commons_1.$value)((0, tsdefaults_1.$unitDefinition)(key, locale), defaultDefinition);
            a.push(`${value} ${value === 1 ? def === null || def === void 0 ? void 0 : def.singular : def.plural}`);
        }
        if (!!(opts === null || opts === void 0 ? void 0 : opts.noDays) && c.days > 0) {
            c.hours = (c.hours + 24 * c.days);
            c.days = types_1.UINT_MIN;
        }
        if (c.days > 0) {
            _addRep('day', c.days, { singular: 'day', plural: 'days', unit: '' });
        }
        if (c.hours > 0) {
            _addRep('hour', c.hours, { singular: 'hour', plural: 'hours', unit: '' });
        }
        if (c.minutes > 0) {
            _addRep('minute', c.minutes, { singular: 'minute', plural: 'minutes', unit: '' });
        }
        if (c.seconds > 0) {
            _addRep('second', c.seconds, { singular: 'second', plural: 'seconds', unit: '' });
        }
        return a.join(' ');
    }
    return '';
}
exports.$durationDescription = $durationDescription;
function $durationNumber2StringFormat(duration, format) {
    let State;
    (function (State) {
        State[State["Standard"] = 0] = "Standard";
        State[State["StandardEscape"] = 1] = "StandardEscape";
        State[State["DaysPart"] = 2] = "DaysPart";
        State[State["DaysPartEscape"] = 3] = "DaysPartEscape";
        State[State["HoursPart"] = 4] = "HoursPart";
        State[State["HoursPartEscape"] = 5] = "HoursPartEscape";
        State[State["SecondsPart"] = 6] = "SecondsPart";
        State[State["SecondsPartEscape"] = 7] = "SecondsPartEscape";
        State[State["SubdaysPart"] = 8] = "SubdaysPart";
        State[State["SubdaysPartEscape"] = 9] = "SubdaysPartEscape";
        State[State["SubhoursPart"] = 10] = "SubhoursPart";
        State[State["SubhoursPartEscape"] = 11] = "SubhoursPartEscape";
    })(State || (State = {}));
    ;
    const comp = $durationcomponents(duration);
    if (!(0, commons_1.$length)(format)) {
        format = "%(%d-%)%H:%M%{:%S%}";
    }
    let fmtlen = format.length;
    let state = State.Standard;
    let ret = "";
    let elsePart = false;
    const subhour = comp.minutes > 0 || comp.seconds > 0;
    const subday = comp.hours > 0 || subhour;
    let stack = [];
    const debug = (0, tsdefaults_1.$default)("debugDurationStateAutomat");
    if (debug) {
        (0, utils_1.$logterm)(`Format:      "${format}"`);
        (0, utils_1.$logterm)(`comp.days:    ${comp.days}`);
        (0, utils_1.$logterm)(`comp.hours:   ${comp.hours}`);
        (0, utils_1.$logterm)(`comp.minutes: ${comp.minutes}`);
        (0, utils_1.$logterm)(`comp.seconds: ${comp.seconds}`);
        (0, utils_1.$logterm)(`subhour:      ${subhour}`);
        (0, utils_1.$logterm)(`subday:       ${subday}`);
    }
    function _pop() { const p = stack.pop(); return [p.state, p.elsePart]; }
    function _push(newState) { stack.push({ state: state, elsePart: elsePart }); return [newState, false]; }
    function _default(c) { ret += '%'; if (c !== '%') {
        ret += c;
    } }
    function _d() { ret += comp.days; }
    function _D() { ret += (0, number_1.$fpad2)(comp.days); }
    function _E() { ret += (0, number_1.$fpad3)(comp.days); }
    function _h() { ret += comp.hours; }
    function _H() { ret += (0, number_1.$fpad2)(comp.hours); }
    function _i() { ret += (comp.days * 24 + comp.hours); }
    function _I() { ret += (0, number_1.$fpad2)((comp.days * 24 + comp.hours)); }
    function _J() { ret += (0, number_1.$fpad3)((comp.days * 24 + comp.hours)); }
    function _m() { ret += comp.minutes; }
    function _M() { ret += (0, number_1.$fpad2)(comp.minutes); }
    function _s() { ret += comp.seconds; }
    function _S() { ret += (0, number_1.$fpad2)(comp.seconds); }
    for (let i = 0; i < fmtlen; i++) {
        const c = format.charAt(i);
        if (debug)
            (0, utils_1.$logterm)(`stack.count = ${stack.length}, state = ${state}, elsePart = ${elsePart}, char[${i}] = '${c}'`);
        switch (state) {
            case State.Standard:
                elsePart = false;
                if (c === '%') {
                    state = State.StandardEscape;
                }
                else {
                    ret += c;
                }
                break;
            case State.StandardEscape:
                state = State.Standard;
                switch (c) {
                    case '%':
                        ret += '%';
                        break;
                    case '(':
                        [state, elsePart] = _push(State.DaysPart);
                        break;
                    case '{':
                        [state, elsePart] = _push(State.SecondsPart);
                        break;
                    case '[':
                        [state, elsePart] = _push(State.SubdaysPart);
                        break;
                    case '≤':
                        [state, elsePart] = _push(State.HoursPart);
                        break;
                    case '<':
                        [state, elsePart] = _push(State.SubhoursPart);
                        break;
                    case 'D':
                        _D();
                        break;
                    case 'd':
                        _d();
                        break;
                    case 'E':
                        _E();
                        break;
                    case 'H':
                        _H();
                        break;
                    case 'h':
                        _h();
                        break;
                    case 'I':
                        _I();
                        break;
                    case 'i':
                        _i();
                        break;
                    case 'J':
                        _J();
                        break;
                    case 'M':
                        _M();
                        break;
                    case 'm':
                        _m();
                        break;
                    case 'S':
                        _S();
                        break;
                    case 's':
                        _s();
                        break;
                    default:
                        ret += '%', ret += c;
                        break;
                }
                break;
            case State.DaysPart:
                if (c === '%') {
                    state = State.DaysPartEscape;
                }
                else if ((comp.days > 0 && !elsePart) || (!comp.days && elsePart)) {
                    ret += c;
                }
                break;
            case State.DaysPartEscape:
                if (c === ')') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = state = State.DaysPart;
                    if (c === 'p') {
                        elsePart = !elsePart;
                    }
                    else if (comp.days > 0 && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'D':
                                _D();
                                break;
                            case 'd':
                                _d();
                                break;
                            case 'E':
                                _E();
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (elsePart && !comp.days) {
                        _default(c);
                    }
                }
                break;
            case State.HoursPart:
                if (c === '%') {
                    state = State.HoursPartEscape;
                }
                else if ((comp.hours > 0 && !elsePart) || (!comp.hours && elsePart)) {
                    ret += c;
                }
                break;
            case State.HoursPartEscape:
                if (c === '≥') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.HoursPart;
                    if (c === 'q') {
                        elsePart = !elsePart;
                    }
                    else if (comp.hours > 0 && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'H':
                                _H();
                                break;
                            case 'h':
                                _h();
                                break;
                            case 'I':
                                _I();
                                break;
                            case 'i':
                                _i();
                                break;
                            case 'J':
                                _J();
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (elsePart && !comp.hours) {
                        _default(c);
                    }
                }
                break;
            case State.SecondsPart:
                if (c === '%') {
                    state = State.SecondsPartEscape;
                }
                else if ((comp.seconds > 0 && !elsePart) || (!comp.seconds && elsePart)) {
                    ret += c;
                }
                break;
            case State.SecondsPartEscape:
                if (c === '}') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.SecondsPart;
                    if (c === 'b') {
                        elsePart = !elsePart;
                    }
                    else if (comp.seconds > 0 && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'S':
                                _S();
                                break;
                            case 's':
                                _s();
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (elsePart && !comp.seconds) {
                        _default(c);
                    }
                }
                break;
            case State.SubdaysPart:
                if (c === '%') {
                    state = State.SubdaysPartEscape;
                }
                else if ((subday && !elsePart) || (!subday && elsePart)) {
                    ret += c;
                }
                break;
            case State.SubdaysPartEscape:
                if (c === ']') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.SubdaysPart;
                    if (c === '!') {
                        elsePart = !elsePart;
                    }
                    else if (subday && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'H':
                                _H();
                                break;
                            case 'h':
                                _h();
                                break;
                            case 'I':
                                _I();
                                break;
                            case 'i':
                                _i();
                                break;
                            case 'J':
                                _J();
                                break;
                            case 'M':
                                _M();
                                break;
                            case 'm':
                                _m();
                                break;
                            case 'S':
                                _S();
                                break;
                            case 's':
                                _s();
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '<':
                                [state, elsePart] = _push(State.SubhoursPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (!subday && elsePart) {
                        _default(c);
                    }
                }
                break;
            case State.SubhoursPart:
                if (c === '%') {
                    state = State.SubhoursPartEscape;
                }
                else if ((subhour && !elsePart) || (!subhour && elsePart)) {
                    ret += c;
                }
                break;
            case State.SubhoursPartEscape:
                if (c === '>') {
                    [state, elsePart] = _pop();
                }
                else {
                    state = State.SubhoursPart;
                    if (c === 'c') {
                        elsePart = !elsePart;
                    }
                    else if (subhour && !elsePart) {
                        switch (c) {
                            case '%':
                                ret += '%';
                                break;
                            case 'M':
                                _M();
                                break;
                            case 'm':
                                _m();
                                break;
                            case 'S':
                                _S();
                                break;
                            case 's':
                                _s();
                                break;
                            case '(':
                                [state, elsePart] = _push(State.DaysPart);
                                break;
                            case '{':
                                [state, elsePart] = _push(State.SecondsPart);
                                break;
                            case '[':
                                [state, elsePart] = _push(State.SubdaysPart);
                                break;
                            case '≤':
                                [state, elsePart] = _push(State.HoursPart);
                                break;
                            default:
                                ret += '%', ret += c;
                                break;
                        }
                    }
                    else if (!subhour && elsePart) {
                        _default(c);
                    }
                }
                break;
        }
        if (debug) {
            (0, utils_1.$logterm)(`state after:${state}, composed string = "${ret}"`);
        }
    }
    switch (state) {
        case State.Standard:
        case State.DaysPart:
        case State.SecondsPart:
        case State.SubdaysPart:
        case State.SubhoursPart:
        case State.HoursPart:
            break;
        case State.StandardEscape:
            ret += '%';
            break;
        case State.DaysPartEscape:
            if ((comp.days > 0 && !elsePart) || (!comp.days && elsePart)) {
                ret += '%';
            }
            break;
        case State.HoursPartEscape:
            if ((comp.hours > 0 && !elsePart) || (!comp.hours && elsePart)) {
                ret += '%';
            }
            break;
        case State.SecondsPartEscape:
            if ((comp.seconds > 0 && !elsePart) || (!comp.days && elsePart)) {
                ret += '%';
            }
            break;
        case State.SubdaysPartEscape:
            if ((subday && !elsePart) || (!subday && elsePart)) {
                ret += '%';
            }
            break;
        case State.SubhoursPartEscape:
            if ((subhour && !elsePart) || (!subhour && elsePart)) {
                ret += '%';
            }
            break;
    }
    if (debug) {
        (0, utils_1.$logterm)(`$durationNumber2StringFormat() returned string = "${ret}"`);
    }
    return ret;
}
exports.$durationNumber2StringFormat = $durationNumber2StringFormat;
function _yearFrom(ys) {
    const y = (0, commons_1.$unsigned)(ys);
    return ys.length > 2 ? y : _adjustYear(y);
}
function _adjustYear(y) {
    const currentYear = new Date().getFullYear();
    const century = (0, number_1.$div)(currentYear, 100);
    return (y > currentYear % 100 + 20 ? y + (century - 1) * 100 : y + century * 100);
}
function _completeWithToday(c) {
    if (c.day) {
        if (c.month) {
            if (!c.year) {
                c.year = (new Date().getFullYear());
            }
        }
        else if (!c.year) {
            const d = new Date();
            c.month = (d.getMonth() + 1);
            c.year = d.getFullYear();
        }
    }
}
function _parsedt(s, regexp, form = TSDateForm.Standard, opts = {}) {
    if (form === TSDateForm.ISO8601 || form === TSDateForm.ISO8601C || form === TSDateForm.ISO8601L) {
        return $isostring2components(s, opts);
    }
    if (!(0, commons_1.$length)(s)) {
        return null;
    }
    const m = s.match(regexp);
    if (!(0, commons_1.$ok)(m)) {
        return null;
    }
    const res = m;
    const l = res.length;
    const c = { year: types_1.UINT_MIN, month: types_1.UINT_MIN, day: types_1.UINT_MIN, hour: types_1.UINT_MIN, minute: types_1.UINT_MIN, second: types_1.UINT_MIN };
    switch (form) {
        case TSDateForm.Standard:
            const ds = (0, commons_1.$unsigned)(res[1]);
            if (l === 2) {
                if (ds >= 1010001) {
                    c.year = (ds % 10000);
                    c.month = (0, number_1.$div)(ds % 1000000, 10000);
                    c.day = (0, number_1.$div)(ds, 1000000);
                }
                else if (ds >= 10101) {
                    c.year = _adjustYear(ds % 100);
                    c.month = (0, number_1.$div)(ds % 10000, 100);
                    c.day = (0, number_1.$div)(ds, 10000);
                }
                else if (ds >= 101) {
                    c.month = (ds % 100);
                    c.day = (0, number_1.$div)(ds, 100);
                    _completeWithToday(c);
                }
                else {
                    c.day = ds;
                    _completeWithToday(c);
                }
            }
            else {
                c.day = ds;
                c.month = (0, commons_1.$unsigned)(res[3]);
                if (l >= 6) {
                    c.year = _yearFrom(res[5]);
                    if (l > 6) {
                        _parseTime(c, res, 5, l - 5);
                    }
                }
                else {
                    _completeWithToday(c);
                }
            }
            break;
        case TSDateForm.English:
            const me = (0, commons_1.$unsigned)(res[1]);
            if (l === 2) {
                if (me >= 1010001) {
                    c.year = (me % 10000);
                    c.day = (0, number_1.$div)(me % 1000000, 10000);
                    c.month = (0, number_1.$div)(me, 1000000);
                }
                else if (me >= 10101) {
                    c.year = _adjustYear(me % 100);
                    c.day = (0, number_1.$div)(me % 10000, 100);
                    c.month = (0, number_1.$div)(me, 10000);
                }
                else if (me >= 101) {
                    c.day = (me % 100);
                    c.month = (0, number_1.$div)(me, 100);
                }
            }
            else {
                c.month = me;
                c.day = (0, commons_1.$unsigned)(res[3]);
                if (l >= 6) {
                    c.year = _yearFrom(res[5]);
                    if (l > 6) {
                        _parseTime(c, res, 5, l - 5);
                    }
                }
                else {
                    _completeWithToday(c);
                }
            }
            break;
        case TSDateForm.Computer:
            const yc = (0, commons_1.$unsigned)(res[1]);
            if (l === 2) {
                if (res[1].length !== 8) {
                    return null;
                }
                c.year = (0, number_1.$div)(yc, 10000);
                c.month = ((0, number_1.$div)(yc, 100) % 100);
                c.day = (0, number_1.$div)(yc, 100);
            }
            else if (l >= 6) {
                c.year = yc;
                c.month = (0, commons_1.$unsigned)(res[3]);
                c.day = (0, commons_1.$unsigned)(res[5]);
                if (l > 6) {
                    _parseTime(c, res, 5, l - 5);
                }
            }
            else {
                return null;
            }
            break;
        default:
            return null;
    }
    return $componentsarevalid(c) ? c : null;
}
function _parseTime(c, m, start, len) {
    let v = (0, commons_1.$unsigned)(m[start + 1]);
    if (len === 2) {
        switch (m[start + 1].length) {
            case 1:
            case 2:
                c.hour = v;
                break;
            case 3:
            case 4:
                c.hour = (0, number_1.$div)(v, 100);
                c.minute = (v % 100);
                break;
            case 5:
            case 6:
                c.hour = (0, number_1.$div)(v, 10000);
                c.minute = ((0, number_1.$div)(v, 100) % 100);
                c.second = (v % 100);
                break;
        }
    }
    else {
        c.hour = v;
        c.minute = (0, commons_1.$unsigned)(m[start + 2]);
        if (len > 4) {
            c.second = (0, commons_1.$unsigned)(m[start + 4]);
        }
    }
}
//# sourceMappingURL=tsdatecomp.js.map