"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$subclassReponsabililty = exports.TSHttpError = exports.TSError = exports.TSUniqueError = void 0;
const commons_1 = require("./commons");
const strings_1 = require("./strings");
const tsrequest_1 = require("./tsrequest");
const utils_1 = require("./utils");
class TSUniqueError extends Error {
    constructor(message) {
        super(message);
        this.name = message;
    }
    static genericError() {
        if (!(0, commons_1.$ok)(this.__genericInstance)) {
            this.__genericInstance = new TSUniqueError('GenericSingletonError');
        }
        return this.__genericInstance;
    }
    static timeoutError() {
        if (!(0, commons_1.$ok)(this.__timeoutInstance)) {
            this.__timeoutInstance = new TSUniqueError('TimeoutSingletonError');
        }
        return this.__timeoutInstance;
    }
    leafInspect() { return this.name; }
}
exports.TSUniqueError = TSUniqueError;
class TSError extends Error {
    static throw() {
        const n = arguments.length;
        switch (n) {
            case 0: throw new this(`${this.name} did throw`);
            case 1: throw new this(arguments[0]);
            case 2: throw new this(arguments[0], arguments[1]);
            default: throw new this(arguments[0], arguments[1], arguments[3]);
        }
    }
    static assert() {
        const n = arguments.length;
        switch (n) {
            case 0: break;
            case 1:
                if (!arguments[0]) {
                    throw new this(`${this.name}.assert() did fail`);
                }
                break;
            case 1:
                if (!arguments[0]) {
                    throw new this(arguments[1]);
                }
                break;
            case 2:
                if (!arguments[0]) {
                    throw new this(arguments[1], arguments[2]);
                }
                break;
            default:
                if (!arguments[0]) {
                    throw new this(arguments[0], arguments[1], arguments[3]);
                }
                break;
        }
    }
    constructor() {
        const n = arguments.length;
        const s = n > 0 ? (0, strings_1.$ftrim)(arguments[0]) : "";
        super(s.length ? s : TSError.DefaultMessage);
        this._errorCode = NaN;
        switch (n) {
            case 1: break;
            case 2:
                if (typeof arguments[1] === 'number') {
                    this.errorCode = arguments[1];
                }
                else if ((0, commons_1.$ok)(arguments[1])) {
                    this.info = arguments[1];
                }
                break;
            case 3:
                if (typeof arguments[1] === 'number') {
                    this.errorCode = arguments[1];
                }
                else if ((0, commons_1.$ok)(arguments[1])) {
                    this.info = arguments[1];
                }
                if (typeof arguments[2] === 'number') {
                    this.errorCode = arguments[2];
                }
                else if ((0, commons_1.$ok)(arguments[2])) {
                    this.info = arguments[2];
                }
                break;
            default:
                break;
        }
    }
    static assertIntParam(v, fn, param) { this.assert(!(0, commons_1.$ok)(v) || (0, commons_1.$isint)(v), `parameter '${param}' of ${fn}() must be an integer`, { functionName: fn, param: param, value: v }); }
    static assertUnsignedParam(v, fn, param) { this.assert(!(0, commons_1.$ok)(v) || (0, commons_1.$isunsigned)(v), `parameter '${param}' of ${fn}() must be an unsigned`, { functionName: fn, param: param, value: v }); }
    static assertNotInBrowser(fn) { this.assert(!(0, utils_1.$inbrowser)(), `unavailable ${fn}() ${fn.includes('.') ? 'method' : 'function'} in browser`, { functionName: fn }); }
    get errorCode() { return this._errorCode; }
    set errorCode(code) { if ((0, commons_1.$isint)(code)) {
        this._errorCode = code;
    } }
    get status() { return (0, commons_1.$ok)(this._errorCode) && !isNaN(this._errorCode) && Object.values(tsrequest_1.Resp).includes(this._errorCode) ? this._errorCode : tsrequest_1.Resp.InternalError; }
    set status(s) { if (Object.values(tsrequest_1.Resp).includes(s)) {
        this._errorCode = s;
    } }
    entries() { return Object.entries({ name: this.name, errorCode: this.errorCode, message: this.message, info: this.info }); }
}
exports.TSError = TSError;
exports.TSHttpError = TSError;
TSError.DefaultMessage = "TSError did throw";
function $subclassReponsabililty(instance, method) {
    const c = instance.constructor;
    if (c === Function) {
        TSError.throw(`implementation of static method ${instance.name}.${method.name}() is subclasses reponsabillity.`, {
            isStatic: true,
            object: instance,
            method: method
        });
    }
    TSError.throw(`implementation of method ${c.name}.${method.name}() is subclasses reponsabillity.`, {
        isStatic: false,
        object: instance,
        method: method
    });
}
exports.$subclassReponsabililty = $subclassReponsabililty;
//# sourceMappingURL=tserrors.js.map