import { TSCharset } from "./tscharset";
import { TSData } from "./tsdata";
import { TSFusionContextType, TSFusionTreeNode } from "./tsfusionnode";
import { Bytes, Nullable, StringEncoding, TSDataLike, TSDictionary, uint, uint8 } from "./types";
export type TSFusionProcedure = (data: any, rootData: any, localContext: TSDictionary, globalContext: TSDictionary, system: TSDictionary) => any;
export declare enum TSCharRole {
    UnsignificantStarterChar = 0,
    StarterChar = 1,
    InternalChar = 2,
    ForbiddenChar = 3
}
interface TSFusionTemplateOptions {
    startingMark?: string;
    endingMark?: string;
    separator?: string;
    debugParsing?: boolean;
    procedures?: TSDictionary<TSFusionProcedure>;
    globalFunctions?: TSDictionary<Function>;
    addStandardGlobalFunctions?: Nullable<boolean>;
}
export interface TSDataTemplateOptions extends TSFusionTemplateOptions {
    encoding?: StringEncoding | TSCharset;
}
export declare abstract class TSFusionTemplate {
    source: TSData;
    startingMark: number[];
    separator: number[];
    endingMark: number[];
    protected _capacity: uint;
    protected _treeRoot: TSFusionTreeNode;
    protected _procedures: TSDictionary<TSFusionProcedure>;
    protected _globalFunctions: Nullable<TSDictionary<Function>>;
    protected _addStandardGlobalFunctions: boolean;
    protected _variables: Map<string, Set<TSFusionContextType>>;
    protected _contextsVariables: Map<TSFusionContextType, Set<string>>;
    static readonly DefaultStartingMark = "{{";
    static readonly DefaultSeparator = ":";
    static readonly DefaultEndingMark = "}}";
    abstract pushFusionData(data: Bytes): void;
    abstract pushFusionValue(value: NonNullable<any>): void;
    static fromData(source: TSDataLike, encoding?: Nullable<StringEncoding | TSCharset>, opts?: TSFusionTemplateOptions): TSDataTemplate | null;
    static fromHTMLData(source: TSDataLike, opts?: TSFusionTemplateOptions): TSHTMLTemplate | null;
    static fromString(source: string, opts?: TSFusionTemplateOptions): TSStringTemplate | null;
    protected constructor(src: TSDataLike, opts?: TSFusionTemplateOptions, paramCharset?: TSCharset);
    get capacity(): number;
    set capacity(n: number);
    validateSeparators(sm: uint8[], em: uint8[], sepa: uint8[]): void;
    variablesForType(type: TSFusionContextType): string[];
    isVariableOfType(s: string, type: TSFusionContextType): boolean;
    get variables(): string[];
    get userVariables(): string[];
    get globalVariables(): string[];
    get systemVariables(): string[];
    get localVariables(): string[];
    get rootVariables(): string[];
    get procedures(): string[];
    isVariable(s: string): boolean;
    isUserVariable(s: string): boolean;
    isGlobalVariable(s: string): boolean;
    isSystemVariable(s: string): boolean;
    isLocalVariable(s: string): boolean;
    isRootVariable(s: string): boolean;
    isProcedure(s: string): boolean;
}
export declare class TSStringTemplate extends TSFusionTemplate {
    private static __stringCharset;
    private _outputString;
    static fromString(source: string, opts?: TSFusionTemplateOptions): TSStringTemplate | null;
    constructor(source: string, options?: TSFusionTemplateOptions);
    pushFusionData(data: Bytes): void;
    pushFusionValue(value: NonNullable<any>): void;
    fusionWithDataContext(data: any, globalContext: TSDictionary, errors?: string[]): string | null;
    fusionStringWithDataContext: (data: any, globalContext: TSDictionary, errors?: string[]) => string | null;
}
export declare abstract class TSGenericDataTemplate extends TSFusionTemplate {
    protected _outputData: TSData | undefined;
    pushFusionData(data: Bytes): void;
    fusionWithDataContext(data: any, globalContext: TSDictionary, errors?: string[]): TSData | null;
}
export declare class TSHTMLTemplate extends TSGenericDataTemplate {
    private _htmlCharset;
    static fromHTMLData(src: TSDataLike, opts?: TSFusionTemplateOptions): TSHTMLTemplate | null;
    constructor(src: TSDataLike, opts?: TSFusionTemplateOptions);
    pushFusionValue(value: NonNullable<any>): void;
    fusionStringWithDataContext(data: any, globalContext: TSDictionary, errors?: string[]): string | null;
}
export declare class TSDataTemplate extends TSGenericDataTemplate {
    protected _charset: TSCharset;
    static fromData(src: TSDataLike, encoding?: Nullable<StringEncoding | TSCharset>, opts?: TSFusionTemplateOptions): TSDataTemplate | null;
    constructor(source: TSDataLike, charset: TSCharset, opts?: TSFusionTemplateOptions);
    get charset(): TSCharset;
    pushFusionValue(value: NonNullable<any>): void;
    fusionStringWithDataContext(data: any, globalContext: TSDictionary, errors?: string[]): string | null;
}
export {};
