import { TSFusionProcedure, TSFusionTemplate } from "./tsfusion";
import { Bytes, Nullable, TSDictionary } from "./types";
export declare enum TSFusionContextType {
    Local = "local",
    Global = "global",
    Root = "root",
    System = "system",
    Procedure = "procedure",
    User = "user"
}
export declare enum TSFusionNodeType {
    Root = "root",
    Data = "data",
    Variable = "variable",
    Test = "test-variable",
    NegativeTest = "nagative-test-variable",
    Enumeration = "enumeration-variable"
}
export interface TSFusionEnumeration {
    fusionEnumeration(): any[];
}
export interface TSFusionOptions {
    globalContext?: Nullable<TSDictionary>;
    procedures?: Nullable<TSDictionary<TSFusionProcedure>>;
    globalFunctions?: Nullable<TSDictionary<Function>>;
    errors?: Nullable<string[]>;
    addStandardGlobalFunctions?: Nullable<boolean>;
}
export declare class TSFusionTreeNode {
    type: TSFusionNodeType;
    value?: string | Bytes;
    nodes?: TSFusionTreeNode[];
    parameters?: any[];
    parent?: TSFusionTreeNode;
    contextType: TSFusionContextType;
    private _localContext;
    private _pathCache;
    static root(): TSFusionTreeNode;
    private constructor();
    get label(): string;
    get isContainerVariable(): boolean;
    pushData(data: Bytes): void;
    pushVariable(name: string, type: TSFusionNodeType, contextType: TSFusionContextType, parameters?: any[]): TSFusionTreeNode;
    fusion(template: TSFusionTemplate, data: any, options?: TSFusionOptions): boolean;
    private _cleanLocalContexts;
    private _fusion;
}
declare global {
    export interface Set<T> extends TSFusionEnumeration {
    }
    export interface Map<K, V> extends TSFusionEnumeration {
    }
}
