"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$conformsToFrame = exports.$conformsToSize = exports.$comformsToPoint = exports.TSIN = exports.TSMM = exports.TSCM = exports.TSAssertFormat = exports.TSCompareSizes = exports.TSValidSize = exports.TSArea = exports.TSEqualSizes = exports.TSEqualPoints = exports.TSPixels2Inches = exports.TSInches2Pixels = exports.TSPixels2mm = exports.TSPixels2cm = exports.TScm2Pixels = exports.TSmm2Pixels = exports.TSRect = exports.TSDocumentFormats = exports.TSRectEdge = void 0;
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const tserrors_1 = require("./tserrors");
const types_1 = require("./types");
var TSRectEdge;
(function (TSRectEdge) {
    TSRectEdge[TSRectEdge["TSMinXEdge"] = 0] = "TSMinXEdge";
    TSRectEdge[TSRectEdge["TSMinYEdge"] = 1] = "TSMinYEdge";
    TSRectEdge[TSRectEdge["TSMaxXEdge"] = 2] = "TSMaxXEdge";
    TSRectEdge[TSRectEdge["TSMaxYEdge"] = 3] = "TSMaxYEdge";
})(TSRectEdge = exports.TSRectEdge || (exports.TSRectEdge = {}));
;
exports.TSDocumentFormats = {
    'min': { w: TSmm2Pixels(100), h: TSmm2Pixels(100) },
    'max': { w: TSmm2Pixels(1200), h: TSmm2Pixels(1200) },
    'letter': { w: TSInches2Pixels(8.5), h: TSInches2Pixels(11) },
    'letter-landscape': { w: TSInches2Pixels(11), h: TSInches2Pixels(8.5) },
    'legal': { w: TSInches2Pixels(8.5), h: TSInches2Pixels(14) },
    'legal-landscape': { w: TSInches2Pixels(14), h: TSInches2Pixels(8.5) },
    'tabloid': { w: TSInches2Pixels(11), h: TSInches2Pixels(17) },
    'ledger': { w: TSInches2Pixels(17), h: TSInches2Pixels(11) },
    'gov-letter': { w: TSInches2Pixels(8), h: TSInches2Pixels(10.5) },
    'gov-letter-landscape': { w: TSInches2Pixels(10.5), h: TSInches2Pixels(8) },
    'gov-legal': { w: TSInches2Pixels(8.5), h: TSInches2Pixels(13) },
    'gov-legal-landscape': { w: TSInches2Pixels(13), h: TSInches2Pixels(8.5) },
    'half-letter': { w: TSInches2Pixels(5.5), h: TSInches2Pixels(8.5) },
    'half-letter-landscape': { w: TSInches2Pixels(8.5), h: TSInches2Pixels(5.5) },
    'junior-legal': { w: TSInches2Pixels(5), h: TSInches2Pixels(8) },
    'junior-legal-landscape': { w: TSInches2Pixels(8), h: TSInches2Pixels(5) },
    'arch-a': { w: TSInches2Pixels(9), h: TSInches2Pixels(12) },
    'arch-a-landscape': { w: TSInches2Pixels(12), h: TSInches2Pixels(9) },
    'arch-b': { w: TSInches2Pixels(12), h: TSInches2Pixels(18) },
    'arch-b-landscape': { w: TSInches2Pixels(18), h: TSInches2Pixels(12) },
    'arch-c': { w: TSInches2Pixels(18), h: TSInches2Pixels(24) },
    'arch-c-landscape': { w: TSInches2Pixels(24), h: TSInches2Pixels(18) },
    'arch-d': { w: TSInches2Pixels(24), h: TSInches2Pixels(36) },
    'arch-d-landscape': { w: TSInches2Pixels(36), h: TSInches2Pixels(24) },
    'arch-e': { w: TSInches2Pixels(36), h: TSInches2Pixels(48) },
    'arch-e-landscape': { w: TSInches2Pixels(48), h: TSInches2Pixels(36) },
    'arch-e1': { w: TSInches2Pixels(30), h: TSInches2Pixels(42) },
    'arch-e1-landscape': { w: TSInches2Pixels(42), h: TSInches2Pixels(30) },
    'arch-e2': { w: TSInches2Pixels(26), h: TSInches2Pixels(38) },
    'arch-e2-landscape': { w: TSInches2Pixels(38), h: TSInches2Pixels(26) },
    'arch-e3': { w: TSInches2Pixels(27), h: TSInches2Pixels(39) },
    'arch-e3-landscape': { w: TSInches2Pixels(39), h: TSInches2Pixels(27) },
    'ansi-c': { w: TSInches2Pixels(17), h: TSInches2Pixels(22) },
    'ansi-c-landscape': { w: TSInches2Pixels(22), h: TSInches2Pixels(17) },
    'ansi-d': { w: TSInches2Pixels(22), h: TSInches2Pixels(34) },
    'ansi-d-landscape': { w: TSInches2Pixels(34), h: TSInches2Pixels(22) },
    'ansi-e': { w: TSInches2Pixels(34), h: TSInches2Pixels(44) },
    'ansi-e-landscape': { w: TSInches2Pixels(44), h: TSInches2Pixels(34) },
    '4a0': { w: TSmm2Pixels(1682), h: TSmm2Pixels(2378) },
    '4a0-landscape': { w: TSmm2Pixels(2378), h: TSmm2Pixels(1682) },
    '2a0': { w: TSmm2Pixels(1189), h: TSmm2Pixels(1682) },
    '2a0-landscape': { w: TSmm2Pixels(1682), h: TSmm2Pixels(1189) },
    'a0+': { w: TSmm2Pixels(914), h: TSmm2Pixels(1292) },
    'a0+landscape': { w: TSmm2Pixels(1292), h: TSmm2Pixels(914) },
    'a0': { w: TSmm2Pixels(841), h: TSmm2Pixels(1189) },
    'a0-landscape': { w: TSmm2Pixels(1189), h: TSmm2Pixels(841) },
    'a1+': { w: TSmm2Pixels(608), h: TSmm2Pixels(914) },
    'a1+landscape': { w: TSmm2Pixels(914), h: TSmm2Pixels(608) },
    'a1': { w: TSmm2Pixels(594), h: TSmm2Pixels(841) },
    'a1-landscape': { w: TSmm2Pixels(841), h: TSmm2Pixels(594) },
    'a2': { w: TSmm2Pixels(420), h: TSmm2Pixels(594) },
    'a2-landscape': { w: TSmm2Pixels(594), h: TSmm2Pixels(420) },
    'a3+': { w: TSmm2Pixels(329), h: TSmm2Pixels(483) },
    'a3+landscape': { w: TSmm2Pixels(483), h: TSmm2Pixels(329) },
    'a3': { w: TSmm2Pixels(297), h: TSmm2Pixels(420) },
    'a3-landscape': { w: TSmm2Pixels(420), h: TSmm2Pixels(297) },
    'a4': { w: TSmm2Pixels(210), h: TSmm2Pixels(297) },
    'a4-landscape': { w: TSmm2Pixels(297), h: TSmm2Pixels(210) },
    'a5': { w: TSmm2Pixels(148), h: TSmm2Pixels(210) },
    'a5-landscape': { w: TSmm2Pixels(210), h: TSmm2Pixels(148) },
    'a6': { w: TSmm2Pixels(105), h: TSmm2Pixels(148) },
    'a6-landscape': { w: TSmm2Pixels(148), h: TSmm2Pixels(105) },
    'a7': { w: TSmm2Pixels(74), h: TSmm2Pixels(105) },
    'a7-landscape': { w: TSmm2Pixels(105), h: TSmm2Pixels(74) },
    'a8': { w: TSmm2Pixels(52), h: TSmm2Pixels(74) },
    'a8-landscape': { w: TSmm2Pixels(74), h: TSmm2Pixels(52) },
    'a9': { w: TSmm2Pixels(37), h: TSmm2Pixels(52) },
    'a9-landscape': { w: TSmm2Pixels(52), h: TSmm2Pixels(37) },
    'a10': { w: TSmm2Pixels(26), h: TSmm2Pixels(37) },
    'a10-landscape': { w: TSmm2Pixels(37), h: TSmm2Pixels(26) },
    'a11': { w: TSmm2Pixels(18), h: TSmm2Pixels(26) },
    'a11-landscape': { w: TSmm2Pixels(26), h: TSmm2Pixels(18) },
    'a12': { w: TSmm2Pixels(13), h: TSmm2Pixels(18) },
    'a12-landscape': { w: TSmm2Pixels(18), h: TSmm2Pixels(13) },
    'a13': { w: TSmm2Pixels(9), h: TSmm2Pixels(13) },
    'a13-landscape': { w: TSmm2Pixels(13), h: TSmm2Pixels(9) },
    'b0+': { w: TSmm2Pixels(1118), h: TSmm2Pixels(1580) },
    'b0+landscape': { w: TSmm2Pixels(1580), h: TSmm2Pixels(1118) },
    'b0': { w: TSmm2Pixels(1000), h: TSmm2Pixels(1414) },
    'b0-landscape': { w: TSmm2Pixels(1414), h: TSmm2Pixels(1000) },
    'b1+': { w: TSmm2Pixels(720), h: TSmm2Pixels(1020) },
    'b1+landscape': { w: TSmm2Pixels(1020), h: TSmm2Pixels(720) },
    'b1': { w: TSmm2Pixels(707), h: TSmm2Pixels(1000) },
    'b1-landscape': { w: TSmm2Pixels(1000), h: TSmm2Pixels(707) },
    'b2+': { w: TSmm2Pixels(520), h: TSmm2Pixels(720) },
    'b2+landscape': { w: TSmm2Pixels(720), h: TSmm2Pixels(520) },
    'b2': { w: TSmm2Pixels(500), h: TSmm2Pixels(707) },
    'b2-landscape': { w: TSmm2Pixels(707), h: TSmm2Pixels(500) },
    'b3': { w: TSmm2Pixels(353), h: TSmm2Pixels(500) },
    'b3-landscape': { w: TSmm2Pixels(500), h: TSmm2Pixels(353) },
    'b4': { w: TSmm2Pixels(250), h: TSmm2Pixels(353) },
    'b4-landscape': { w: TSmm2Pixels(353), h: TSmm2Pixels(250) },
    'b5': { w: TSmm2Pixels(176), h: TSmm2Pixels(250) },
    'b5-landscape': { w: TSmm2Pixels(250), h: TSmm2Pixels(176) },
    'b6': { w: TSmm2Pixels(125), h: TSmm2Pixels(176) },
    'b6-landscape': { w: TSmm2Pixels(176), h: TSmm2Pixels(125) },
    'b7': { w: TSmm2Pixels(88), h: TSmm2Pixels(125) },
    'b7-landscape': { w: TSmm2Pixels(125), h: TSmm2Pixels(88) },
    'b8': { w: TSmm2Pixels(62), h: TSmm2Pixels(88) },
    'b8-landscape': { w: TSmm2Pixels(88), h: TSmm2Pixels(62) },
    'b9': { w: TSmm2Pixels(44), h: TSmm2Pixels(62) },
    'b9-landscape': { w: TSmm2Pixels(62), h: TSmm2Pixels(44) },
    'b10': { w: TSmm2Pixels(31), h: TSmm2Pixels(44) },
    'b10-landscape': { w: TSmm2Pixels(44), h: TSmm2Pixels(31) },
    'b11': { w: TSmm2Pixels(22), h: TSmm2Pixels(31) },
    'b11-landscape': { w: TSmm2Pixels(31), h: TSmm2Pixels(22) },
    'b12': { w: TSmm2Pixels(15), h: TSmm2Pixels(22) },
    'b12-landscape': { w: TSmm2Pixels(22), h: TSmm2Pixels(15) },
    'b13': { w: TSmm2Pixels(11), h: TSmm2Pixels(15) },
    'b13-landscape': { w: TSmm2Pixels(15), h: TSmm2Pixels(11) },
    'c0': { w: TSmm2Pixels(917), h: TSmm2Pixels(1297) },
    'c0-landscape': { w: TSmm2Pixels(1297), h: TSmm2Pixels(917) },
    'c1': { w: TSmm2Pixels(648), h: TSmm2Pixels(917) },
    'c1-landscape': { w: TSmm2Pixels(917), h: TSmm2Pixels(648) },
    'c2': { w: TSmm2Pixels(458), h: TSmm2Pixels(648) },
    'c2-landscape': { w: TSmm2Pixels(648), h: TSmm2Pixels(458) },
    'c3': { w: TSmm2Pixels(324), h: TSmm2Pixels(458) },
    'c3-landscape': { w: TSmm2Pixels(458), h: TSmm2Pixels(324) },
    'c4': { w: TSmm2Pixels(229), h: TSmm2Pixels(324) },
    'c4-landscape': { w: TSmm2Pixels(324), h: TSmm2Pixels(229) },
    'c5': { w: TSmm2Pixels(162), h: TSmm2Pixels(229) },
    'c5-landscape': { w: TSmm2Pixels(229), h: TSmm2Pixels(162) },
    'c6': { w: TSmm2Pixels(114), h: TSmm2Pixels(162) },
    'c6-landscape': { w: TSmm2Pixels(162), h: TSmm2Pixels(114) },
    'c7': { w: TSmm2Pixels(81), h: TSmm2Pixels(114) },
    'c7-landscape': { w: TSmm2Pixels(114), h: TSmm2Pixels(81) },
    'c8': { w: TSmm2Pixels(57), h: TSmm2Pixels(81) },
    'c8-landscape': { w: TSmm2Pixels(81), h: TSmm2Pixels(57) },
    'c9': { w: TSmm2Pixels(40), h: TSmm2Pixels(57) },
    'c9-landscape': { w: TSmm2Pixels(57), h: TSmm2Pixels(40) },
    'c10': { w: TSmm2Pixels(28), h: TSmm2Pixels(40) },
    'c10-landscape': { w: TSmm2Pixels(40), h: TSmm2Pixels(28) },
    'ra0': { w: TSmm2Pixels(860), h: TSmm2Pixels(1220) },
    'ra0-landscape': { w: TSmm2Pixels(1220), h: TSmm2Pixels(860) },
    'ra1': { w: TSmm2Pixels(610), h: TSmm2Pixels(860) },
    'ra1-landscape': { w: TSmm2Pixels(860), h: TSmm2Pixels(610) },
    'ra2': { w: TSmm2Pixels(430), h: TSmm2Pixels(610) },
    'ra2-landscape': { w: TSmm2Pixels(610), h: TSmm2Pixels(430) },
    'ra3': { w: TSmm2Pixels(305), h: TSmm2Pixels(430) },
    'ra3-landscape': { w: TSmm2Pixels(430), h: TSmm2Pixels(305) },
    'ra4': { w: TSmm2Pixels(215), h: TSmm2Pixels(305) },
    'ra4-landscape': { w: TSmm2Pixels(305), h: TSmm2Pixels(215) },
    'sra0': { w: TSmm2Pixels(900), h: TSmm2Pixels(1280) },
    'sra0-landscape': { w: TSmm2Pixels(1280), h: TSmm2Pixels(900) },
    'sra1': { w: TSmm2Pixels(640), h: TSmm2Pixels(900) },
    'sra1-landscape': { w: TSmm2Pixels(900), h: TSmm2Pixels(640) },
    'sra2': { w: TSmm2Pixels(450), h: TSmm2Pixels(640) },
    'sra2-landscape': { w: TSmm2Pixels(640), h: TSmm2Pixels(450) },
    'sra3': { w: TSmm2Pixels(320), h: TSmm2Pixels(450) },
    'sra3-landscape': { w: TSmm2Pixels(450), h: TSmm2Pixels(320) },
    'sra4': { w: TSmm2Pixels(225), h: TSmm2Pixels(320) },
    'sra4-landscape': { w: TSmm2Pixels(320), h: TSmm2Pixels(225) },
    'sra1+': { w: TSmm2Pixels(660), h: TSmm2Pixels(920) },
    'sra1+landscape': { w: TSmm2Pixels(920), h: TSmm2Pixels(660) },
    'sra2+': { w: TSmm2Pixels(480), h: TSmm2Pixels(650) },
    'sra2+landscape': { w: TSmm2Pixels(650), h: TSmm2Pixels(480) },
    'sra3+': { w: TSmm2Pixels(320), h: TSmm2Pixels(460) },
    'sra3+landscape': { w: TSmm2Pixels(460), h: TSmm2Pixels(320) },
    'sra3++': { w: TSmm2Pixels(320), h: TSmm2Pixels(464) },
    'sra3++landscape': { w: TSmm2Pixels(464), h: TSmm2Pixels(320) },
    'a0u': { w: TSmm2Pixels(880), h: TSmm2Pixels(1230) },
    'a0u-landscape': { w: TSmm2Pixels(1230), h: TSmm2Pixels(880) },
    'a1u': { w: TSmm2Pixels(625), h: TSmm2Pixels(880) },
    'a1u-landscape': { w: TSmm2Pixels(880), h: TSmm2Pixels(625) },
    'a2u': { w: TSmm2Pixels(450), h: TSmm2Pixels(625) },
    'a2u-landscape': { w: TSmm2Pixels(625), h: TSmm2Pixels(450) },
    'a3u': { w: TSmm2Pixels(330), h: TSmm2Pixels(450) },
    'a3u-landscape': { w: TSmm2Pixels(450), h: TSmm2Pixels(330) },
    'a4u': { w: TSmm2Pixels(240), h: TSmm2Pixels(330) },
    'a4u-landscape': { w: TSmm2Pixels(330), h: TSmm2Pixels(240) },
    'demi-raisin': { w: TSmm2Pixels(325), h: TSmm2Pixels(500) },
    'demi-raisin-landscape': { w: TSmm2Pixels(500), h: TSmm2Pixels(325) },
    'raisin': { w: TSmm2Pixels(500), h: TSmm2Pixels(650) },
    'raisin-landscape': { w: TSmm2Pixels(650), h: TSmm2Pixels(500) },
    'double-raisin': { w: TSmm2Pixels(650), h: TSmm2Pixels(1000) },
    'double-raisin-landscape': { w: TSmm2Pixels(1000), h: TSmm2Pixels(650) },
    'jesus': { w: TSmm2Pixels(560), h: TSmm2Pixels(760) },
    'jesus-landscape': { w: TSmm2Pixels(760), h: TSmm2Pixels(560) },
    'soleil': { w: TSmm2Pixels(600), h: TSmm2Pixels(800) },
    'soleil-landscape': { w: TSmm2Pixels(800), h: TSmm2Pixels(600) },
    'univers': { w: TSmm2Pixels(1000), h: TSmm2Pixels(1130) },
    'univers-landscape': { w: TSmm2Pixels(1130), h: TSmm2Pixels(1000) },
    'din-d0': { w: TSmm2Pixels(771), h: TSmm2Pixels(1090) },
    'din-d0-landscape': { w: TSmm2Pixels(1090), h: TSmm2Pixels(771) },
    'din-d1': { w: TSmm2Pixels(545), h: TSmm2Pixels(771) },
    'din-d1-landscape': { w: TSmm2Pixels(771), h: TSmm2Pixels(545) },
    'din-d2': { w: TSmm2Pixels(385), h: TSmm2Pixels(545) },
    'din-d2-landscape': { w: TSmm2Pixels(545), h: TSmm2Pixels(385) },
    'din-d3': { w: TSmm2Pixels(272), h: TSmm2Pixels(385) },
    'din-d3-landscape': { w: TSmm2Pixels(385), h: TSmm2Pixels(272) },
    'din-d4': { w: TSmm2Pixels(192), h: TSmm2Pixels(272) },
    'din-d4-landscape': { w: TSmm2Pixels(272), h: TSmm2Pixels(192) },
    'din-d5': { w: TSmm2Pixels(136), h: TSmm2Pixels(192) },
    'din-d5-landscape': { w: TSmm2Pixels(192), h: TSmm2Pixels(136) },
    'din-d6': { w: TSmm2Pixels(96), h: TSmm2Pixels(136) },
    'din-d6-landscape': { w: TSmm2Pixels(136), h: TSmm2Pixels(96) },
    'din-d7': { w: TSmm2Pixels(68), h: TSmm2Pixels(96) },
    'din-d7-landscape': { w: TSmm2Pixels(96), h: TSmm2Pixels(68) },
    'din-d8': { w: TSmm2Pixels(48), h: TSmm2Pixels(68) },
    'din-d8-landscape': { w: TSmm2Pixels(68), h: TSmm2Pixels(48) },
    'envelope-dl': { w: TSmm2Pixels(110), h: TSmm2Pixels(220) },
    'envelope-b4': { w: TSmm2Pixels(250), h: TSmm2Pixels(353) },
    'envelope-b5': { w: TSmm2Pixels(176), h: TSmm2Pixels(250) },
    'envelope-b6': { w: TSmm2Pixels(125), h: TSmm2Pixels(176) },
    'envelope-c3': { w: TSmm2Pixels(324), h: TSmm2Pixels(458) },
    'envelope-c4': { w: TSmm2Pixels(229), h: TSmm2Pixels(324) },
    'envelope-c4m': { w: TSmm2Pixels(318), h: TSmm2Pixels(229) },
    'envelope-c5': { w: TSmm2Pixels(162), h: TSmm2Pixels(229) },
    'envelope-c6/c5': { w: TSmm2Pixels(114), h: TSmm2Pixels(229) },
    'envelope-c6': { w: TSmm2Pixels(114), h: TSmm2Pixels(162) },
    'envelope-c64m': { w: TSmm2Pixels(318), h: TSmm2Pixels(114) },
    'envelope-c7/c6': { w: TSmm2Pixels(81), h: TSmm2Pixels(162) },
    'envelope-c7': { w: TSmm2Pixels(81), h: TSmm2Pixels(114) },
    'envelope-ce4': { w: TSmm2Pixels(229), h: TSmm2Pixels(310) },
    'envelope-ce64': { w: TSmm2Pixels(114), h: TSmm2Pixels(310) },
    'envelope-e4': { w: TSmm2Pixels(220), h: TSmm2Pixels(312) },
    'envelope-ec45': { w: TSmm2Pixels(220), h: TSmm2Pixels(229) },
    'envelope-ec5': { w: TSmm2Pixels(155), h: TSmm2Pixels(229) },
    'envelope-e5': { w: TSmm2Pixels(115), h: TSmm2Pixels(220) },
    'envelope-e56': { w: TSmm2Pixels(155), h: TSmm2Pixels(155) },
    'envelope-e6': { w: TSmm2Pixels(110), h: TSmm2Pixels(155) },
    'envelope-e65': { w: TSmm2Pixels(110), h: TSmm2Pixels(220) },
    'envelope-r7': { w: TSmm2Pixels(120), h: TSmm2Pixels(135) },
    'envelope-s4': { w: TSmm2Pixels(250), h: TSmm2Pixels(330) },
    'envelope-s5': { w: TSmm2Pixels(185), h: TSmm2Pixels(255) },
    'envelope-s65': { w: TSmm2Pixels(110), h: TSmm2Pixels(225) },
    'envelope-x5': { w: TSmm2Pixels(105), h: TSmm2Pixels(216) },
    'envelope-ex5': { w: TSmm2Pixels(155), h: TSmm2Pixels(216) },
    'folio': { w: TSInches2Pixels(12), h: TSInches2Pixels(19) },
    'folio-landscape': { w: TSInches2Pixels(19), h: TSInches2Pixels(12) },
    'quarto': { w: TSInches2Pixels(9.5), h: TSInches2Pixels(12) },
    'quarto-landscape': { w: TSInches2Pixels(12), h: TSInches2Pixels(9.5) },
    'octavo': { w: TSInches2Pixels(6), h: TSInches2Pixels(9) },
    'octavo-landscape': { w: TSInches2Pixels(9), h: TSInches2Pixels(6) },
    'a-format': { w: TSmm2Pixels(110), h: TSmm2Pixels(178) },
    'a-format-landscape': { w: TSmm2Pixels(178), h: TSmm2Pixels(110) },
    'b-format': { w: TSmm2Pixels(129), h: TSmm2Pixels(198) },
    'b-format-landscape': { w: TSmm2Pixels(198), h: TSmm2Pixels(129) },
    'c-format': { w: TSmm2Pixels(135), h: TSmm2Pixels(216) },
    'c-format-landscape': { w: TSmm2Pixels(216), h: TSmm2Pixels(135) },
};
const customInspectSymbol = Symbol.for('nodejs.util.inspect.custom');
class TSRect {
    constructor() {
        this.containedInRect = this.containedIn;
        this.intersectsRect = this.intersects;
        this.intersectionRect = this.intersection;
        this.unionRect = this.union;
        this.offsetRect = this.offset;
        this.insetRect = this.inset;
        this.integralRect = this.integral;
        this.divideRect = this.divide;
        this.leafInspect = this.toString;
        const n = arguments.length;
        switch (n) {
            case 0:
                this.x = this.y = this.w = this.h = 0;
                break;
            case 1: {
                if ((0, commons_1.$isstring)(arguments[0])) {
                    const s = exports.TSDocumentFormats[arguments[0]];
                    this._setInternalValues('bad document format parameter', 0, 0, s === null || s === void 0 ? void 0 : s.w, s === null || s === void 0 ? void 0 : s.h, (0, commons_1.$ok)(s));
                }
                else if ((0, commons_1.$isarray)(arguments[0])) {
                    const a = arguments[0];
                    this._setInternalValues('bad rect definition array parameter', a[0], a[1], a[2], a[3], a.length === 4);
                }
                else if ((arguments[0] instanceof TSRect)) {
                    const r = arguments[0];
                    this._setInternalValues('bad TSRect parameter', r.minX, r.minY, r.width, r.height);
                }
                else if ($conformsToFrame(arguments[0])) {
                    const f = arguments[0];
                    this._setInternalValues('bad TSFrame parameter', f.x, f.y, f.w, f.h);
                }
                else {
                    tserrors_1.TSError.throw('TSRect.constructor() : should have a TSRect, a TSFrame or a document format parameter', { arguments: Array.from(arguments) });
                }
                break;
            }
            case 2:
                if ($comformsToPoint(arguments[0])) {
                    const s = (0, commons_1.$isstring)(arguments[1]) ? exports.TSDocumentFormats[arguments[1]] : arguments[1];
                    if ($conformsToSize(s)) {
                        const p = arguments[0];
                        this._setInternalValues('bad TSPoint+(TSSize or TSDocumentFormat) 2 parameters', p.x, p.y, s.w, s.h);
                    }
                    else {
                        tserrors_1.TSError.throw('TSRect.constructor() : 2nd parameeter is not a TSSize', { arguments: Array.from(arguments) });
                    }
                }
                else {
                    tserrors_1.TSError.throw('TSRect.constructor() : 1st parameter is not a TSPoint', { arguments: Array.from(arguments) });
                }
                break;
            case 3:
                if ((0, commons_1.$isnumber)(arguments[0]) && (0, commons_1.$isnumber)(arguments[1]) && ((0, commons_1.$isstring)(arguments[2]) || $conformsToSize(arguments[2]))) {
                    const s = (0, commons_1.$isstring)(arguments[2]) ? exports.TSDocumentFormats[arguments[2]] : arguments[2];
                    this._setInternalValues('bad [number, number, size or format] parameters', arguments[0], arguments[1], s.w, s.h);
                }
                else if ($comformsToPoint(arguments[0]) && (0, commons_1.$isnumber)(arguments[1]) && (0, commons_1.$isnumber)(arguments[2])) {
                    const p = arguments[0];
                    this._setInternalValues('bad [point, number, number] parameters', p.x, p.y, arguments[1], arguments[2]);
                }
                else {
                    tserrors_1.TSError.throw('TSRect.constructor() : bad TSPoint, TSSize or number parameter', { arguments: Array.from(arguments) });
                }
                break;
            case 4:
                this._setInternalValues('one or more bad number parameter', arguments[0], arguments[1], arguments[2], arguments[3]);
                break;
            default:
                tserrors_1.TSError.throw('TSRect.constructor() : too much arguments', { arguments: Array.from(arguments) });
        }
    }
    get origin() { return { x: this.minX, y: this.minY }; }
    get size() { return { w: this.width, h: this.height }; }
    get width() { return this.maxX - this.minX; }
    get height() { return this.maxY - this.minY; }
    get frame() { return { x: this.minX, y: this.minY, w: this.width, h: this.height }; }
    get minX() { return this.x; }
    get minY() { return this.y; }
    get maxX() { return this.x + this.w; }
    get maxY() { return this.y + this.h; }
    get midX() { return this.x + this.w / 2; }
    get midY() { return this.y + this.h / 2; }
    get isEmpty() { return this.w > 0 && this.h > 0 ? false : true; }
    contains(p) {
        if (p instanceof TSRect) {
            const r = p;
            return !r.isEmpty && this.minX <= r.minX && this.minY <= r.minY && this.maxX >= r.maxX && this.maxY >= r.maxY;
        }
        if ((0, commons_1.$isarray)(p)) {
            const a = p;
            if ((a.length === 2 || a.length === 4) && (0, commons_1.$isnumber)(a[0]) && (0, commons_1.$isnumber)(a[1]) && (a.length === 2 || ((0, commons_1.$isnumber)(a[2]) && (0, commons_1.$isnumber)(a[3]) && a[2] >= 0 && a[3] >= 0))) {
                return a.length === 2 ? this.contains({ x: a[0], y: a[1] }) : this.contains(new TSRect(a[0], a[1], a[2], a[3]));
            }
            return false;
        }
        return $comformsToPoint(p) ? p.x >= this.minX && p.x <= this.maxX && p.y >= this.minY && p.y <= this.maxY : false;
    }
    containsPoint(p) {
        if ((0, commons_1.$isarray)(p) && p.length !== 2) {
            return false;
        }
        return this.contains(p);
    }
    containsRect(p) {
        if ((0, commons_1.$isarray)(p) && p.length !== 4) {
            return false;
        }
        return this.contains(p);
    }
    containedIn(r) {
        if (!(0, commons_1.$ok)(r)) {
            return false;
        }
        try {
            r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        }
        catch (_a) {
            return false;
        }
        return r.contains(this);
    }
    intersects(r) {
        if (!(0, commons_1.$ok)(r)) {
            return false;
        }
        try {
            r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        }
        catch (_a) {
            return false;
        }
        return this.maxX <= r.minX || r.maxX <= this.minX || this.maxY <= r.minY || r.maxY <= this.minY || this.isEmpty || r.isEmpty ? false : true;
    }
    intersection(r) {
        let rect = new TSRect();
        if (!(0, commons_1.$ok)(r)) {
            return rect;
        }
        try {
            r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        }
        catch (_a) {
            return rect;
        }
        if (this.maxX <= r.minX || r.maxX <= this.minX || this.maxY <= r.minY || r.maxY <= this.minY) {
            return rect;
        }
        rect.x = Math.max(this.minX, r.minX);
        rect.y = Math.max(this.minY, r.minY);
        rect.w = Math.min(this.maxX, r.maxX) - rect.x;
        rect.h = Math.min(this.maxY, r.maxY) - rect.y;
        return rect;
    }
    union(r) {
        if (!(0, commons_1.$ok)(r)) {
            return this.clone();
        }
        r = (0, commons_1.$isarray)(r) ? new TSRect(r) : r;
        if (this.isEmpty) {
            return r.clone();
        }
        if (r.isEmpty) {
            return this.clone();
        }
        let rect = new TSRect();
        rect.x = Math.min(this.minX, r.minX);
        rect.y = Math.min(this.minY, r.minY);
        rect.w = Math.max(this.maxX, r.maxX) - rect.x;
        rect.h = Math.max(this.maxY, r.maxY) - rect.y;
        return rect;
    }
    offset(xOffset, yOffset) {
        if (!(0, commons_1.$isnumber)(xOffset) || !(0, commons_1.$isnumber)(yOffset)) {
            tserrors_1.TSError.throw('TSRect.offset() : invalid offset parameter', { xOffset: xOffset, yOffset: yOffset });
        }
        return new TSRect(this.minX + xOffset, this.minY + yOffset, this.width, this.height);
    }
    inset(insetWidth, insetHeight) {
        if (!(0, commons_1.$isnumber)(insetWidth) || !(0, commons_1.$isnumber)(insetHeight)) {
            tserrors_1.TSError.throw('TSRect.inset() : invalid inset parameter', { insetWidth: insetWidth, insetHeight: insetHeight });
        }
        return new TSRect(this.minX - insetWidth, this.minY - insetHeight, this.width + insetWidth * 2, this.height + insetHeight * 2);
    }
    integral() {
        let rect = new TSRect();
        rect.x = Math.floor(this.minX);
        rect.y = Math.floor(this.minY);
        if (this.isEmpty) {
            return rect;
        }
        rect.w = Math.ceil(this.maxX) - rect.x;
        rect.h = Math.ceil(this.maxY) - rect.y;
        return rect;
    }
    divide(amount, edge) {
        const x = this.minX;
        const y = this.minY;
        const w = this.width;
        const h = this.height;
        if (amount === Number.POSITIVE_INFINITY) {
            amount = Math.max(w, h);
        }
        if (!(0, commons_1.$isnumber)(amount) || amount < 0) {
            amount = 0;
        }
        switch (edge) {
            case TSRectEdge.TSMinXEdge:
                return amount > w ?
                    [new TSRect(x, y, w, h), new TSRect(this.maxX, y, 0, h)] :
                    [new TSRect(x, y, amount, h), new TSRect(x + amount, y, w - amount, h)];
            case TSRectEdge.TSMinYEdge:
                return amount > h ?
                    [new TSRect(x, y, w, h), new TSRect(x, this.maxY, w, 0)] :
                    [new TSRect(x, y, w, amount), new TSRect(x, y + amount, w, h - amount)];
            case TSRectEdge.TSMaxXEdge:
                return amount > w ?
                    [new TSRect(x, y, w, h), new TSRect(x, y, 0, h)] :
                    [new TSRect(this.maxX - amount, y, amount, h), new TSRect(x, y, w - amount, h)];
            case TSRectEdge.TSMaxYEdge:
                return amount > h ?
                    [new TSRect(x, y, w, h), new TSRect(x, y, w, 0)] :
                    [new TSRect(x, this.maxY - amount, w, amount), new TSRect(x, y, w, h - amount)];
            default:
                tserrors_1.TSError.throw('TSRect.divide() : bad edge parameter', { amount: amount, edge: edge });
        }
    }
    clone() { return new TSRect(this.minX, this.minY, this.width, this.height); }
    closedPolygon(updatesYCoordinatesfirst = false) {
        const x1 = this.minX, y1 = this.minY, x2 = this.maxX, y2 = this.maxY;
        return updatesYCoordinatesfirst ?
            [{ x: x1, y: y1 }, { x: x1, y: y2 }, { x: x2, y: y2 }, { x: x2, y: y1 }, { x: x1, y: y1 }] :
            [{ x: x1, y: y1 }, { x: x2, y: y1 }, { x: x2, y: y2 }, { x: x1, y: y2 }, { x: x1, y: y1 }];
    }
    isEqual(other) { return this === other || (other instanceof TSRect && other.minX === this.minX && other.maxX === this.maxX && other.minY === this.minY && other.maxY === this.maxY); }
    compare(other) {
        if (other === this) {
            return types_1.Same;
        }
        if (!(other instanceof TSRect)) {
            return undefined;
        }
        const area = this.width * this.height;
        const otherArea = other.width * other.height;
        if (area === otherArea) {
            const aX = this.minX;
            const aY = this.minY;
            const bX = other.minX;
            const bY = other.minY;
            if (aX === bX) {
                return aY < bY ? types_1.Ascending : (aY > bY ? types_1.Descending : types_1.Same);
            }
            if (aY === other.y) {
                return aX < bX ? types_1.Ascending : types_1.Descending;
            }
            if (aX < bX && aY < bY) {
                return types_1.Ascending;
            }
            if (aX > bX && aY > bY) {
                return types_1.Descending;
            }
            return undefined;
        }
        return area < otherArea ? types_1.Ascending : types_1.Descending;
    }
    toJSON() { return this.frame; }
    toString() {
        return `{ x = ${this.minX}, y = ${this.minY}), w:${this.width}, h:${this.height} }`;
    }
    toArray() { return [this.minX, this.minY, this.maxX, this.maxY]; }
    [customInspectSymbol](depth, inspectOptions, inspect) {
        return this.leafInspect();
    }
    _setInternalValues(message, x, y, w, h, _moreTest = true) {
        if (!!_moreTest && (0, commons_1.$isnumber)(x) && (0, commons_1.$isnumber)(y) && (0, commons_1.$isnumber)(w) && (0, commons_1.$isnumber)(h) && w >= 0 && h >= 0) {
            this.x = x;
            this.y = y;
            this.w = w;
            this.h = h;
        }
        else {
            tserrors_1.TSError.throw(`TSRect.constructor() : ${message}`, { x: x, y: y, w: w, h: h });
        }
    }
}
exports.TSRect = TSRect;
function TSmm2Pixels(mm) { return mm * 45 / 16; }
exports.TSmm2Pixels = TSmm2Pixels;
function TScm2Pixels(cm) { return cm * 225 / 8; }
exports.TScm2Pixels = TScm2Pixels;
function TSPixels2cm(pixels) { return pixels * 8 / 225; }
exports.TSPixels2cm = TSPixels2cm;
function TSPixels2mm(pixels) { return pixels * 16 / 45; }
exports.TSPixels2mm = TSPixels2mm;
function TSInches2Pixels(inches) { return inches * 72; }
exports.TSInches2Pixels = TSInches2Pixels;
function TSPixels2Inches(pixels) { return pixels / 72; }
exports.TSPixels2Inches = TSPixels2Inches;
function TSEqualPoints(A, B) { return A.x === B.x && A.y === B.y; }
exports.TSEqualPoints = TSEqualPoints;
function TSEqualSizes(A, B) { return A.w === B.w && A.h === B.h; }
exports.TSEqualSizes = TSEqualSizes;
function TSArea(A) { return A.w * A.h; }
exports.TSArea = TSArea;
function TSValidSize(A) { return (0, commons_1.$ok)(A) && A.w >= 0 && A.h >= 0; }
exports.TSValidSize = TSValidSize;
function TSCompareSizes(A, B) { return (0, compare_1.$numcompare)(TSArea(A), TSArea(B)); }
exports.TSCompareSizes = TSCompareSizes;
;
function TSAssertFormat(format, opts = {}) {
    const size = (0, commons_1.$isstring)(format) ? exports.TSDocumentFormats[format] : format;
    const defaultSize = TSValidSize(opts.defaultSize) ? opts.defaultSize : exports.TSDocumentFormats.a4;
    if (!TSValidSize(size)) {
        if (opts.invalidSizeRaise) {
            tserrors_1.TSError.throw('TSAssertFormat() : invalid size format', { size: size });
        }
        return defaultSize;
    }
    const min = TSValidSize(opts.minimalSize) ? opts.minimalSize : exports.TSDocumentFormats.min;
    if (size.w < min.w || size.h < min.h) {
        if (opts.oversizeRaise) {
            tserrors_1.TSError.throw('TSAssertFormat() : size format too small', { size: size, minimalSize: min });
        }
        return min;
    }
    const max = TSValidSize(opts.maximalSize) ? opts.maximalSize : exports.TSDocumentFormats.max;
    if (size.w > max.w || size.h > max.h) {
        if (opts.oversizeRaise) {
            tserrors_1.TSError.throw('TSAssertFormat() : size format too large', { size: size, maximalSize: max });
        }
        return max;
    }
    return size;
}
exports.TSAssertFormat = TSAssertFormat;
exports.TSCM = TScm2Pixels(1);
exports.TSMM = TSmm2Pixels(1);
exports.TSIN = TSInches2Pixels(1);
const LocalPointProperties = ['x', 'y'];
function $comformsToPoint(v) { return v instanceof TSRect || (0, commons_1.$hasproperties)(v, LocalPointProperties); }
exports.$comformsToPoint = $comformsToPoint;
const LocalSizeProperties = ['w', 'h'];
function $conformsToSize(v) { return v instanceof TSRect || (0, commons_1.$hasproperties)(v, LocalSizeProperties); }
exports.$conformsToSize = $conformsToSize;
function $conformsToFrame(v) { return $comformsToPoint(v) && $conformsToSize(v); }
exports.$conformsToFrame = $conformsToFrame;
//# sourceMappingURL=tsgeometry.js.map