import { Nullable, TSDictionary } from "./types";
export type TSLeafOptionalNode = 'boolean' | 'charset' | 'color' | 'continent' | 'country' | 'currency' | 'data' | 'date' | 'email' | 'hexa' | 'int' | 'int8' | 'int16' | 'int32' | 'ipaddress' | 'ipv4' | 'ipv6' | 'jsdate' | 'language' | 'native' | 'number' | 'paper' | 'path' | 'phone' | 'string' | 'uint8' | 'uint16' | 'uint32' | 'unsigned' | 'uuid' | 'url';
export type TSMandatoryLeafNode = 'boolean!' | 'charset!' | 'color!' | 'continent!' | 'country!' | 'currency!' | 'data!' | 'date!' | 'email!' | 'hexa!' | 'int!' | 'int8!' | 'int16!' | 'int32!' | 'ipaddress!' | 'ipv4!' | 'ipv6!' | 'jsdate!' | 'language!' | 'native!' | 'number!' | 'paper!' | 'path!' | 'phone!' | 'string!' | 'uint8!' | 'uint16!' | 'uint32!' | 'unsigned!' | 'uuid!' | 'url!';
export type TSParserNodeType = TSLeafOptionalNode | 'array' | 'object';
export type TSNativeValue = Nullable<string | number | boolean>;
export declare enum TSParserActionContext {
    url = "url",
    json = "json",
    vargs = "vargs",
    env = "env"
}
export type TSParserActionContextType = keyof typeof TSParserActionContext;
export interface TSParserOptions {
    errors?: Nullable<string[]>;
    context?: Nullable<TSParserActionContextType>;
    acceptsUncheckedItems?: Nullable<boolean>;
}
export interface TSParserActionOptions extends TSParserOptions {
    options?: Nullable<TSDictionary>;
}
export type TSParserAction<T> = (v: any, opts?: Nullable<TSParserActionOptions>) => T;
export type TSParserChecker = TSParserAction<boolean>;
export type TSParserTransformer = TSParserAction<any>;
export type TSParserNatifier = TSParserAction<TSNativeValue>;
export type TSExtendedLeafNode = {
    _type: TSLeafOptionalNode;
    _mandatory?: boolean;
    _default?: any;
    _enum?: TSDictionary<number | string> | Array<string | number>;
    _exportAsEnum?: boolean;
    _checker?: TSParserChecker;
    _transformer?: TSParserTransformer;
    _natifier?: TSParserNatifier;
    _options?: TSDictionary;
};
export type TSLeafNode = TSLeafOptionalNode | TSMandatoryLeafNode | TSExtendedLeafNode;
export type TSExtendedArrayNode = {
    _itemsType: TSNode;
    _mandatory?: boolean;
    _min?: number;
    _max?: number;
    _checker?: TSParserChecker;
    _transformer?: TSParserTransformer;
    _natifier?: TSParserNatifier;
};
export type TSArrayNode = [TSNode, number?, number?];
export declare enum TSCase {
    standard = "standard",
    lowercase = "lowercase",
    uppercase = "uppercase"
}
export type TSObjectNode = {
    [key: string]: TSNode;
} | {
    _keysType?: TSLeafOptionalNode;
    _valueItemsType?: TSNode;
    _acceptsUncheckedItems?: boolean;
    _checker?: TSParserChecker;
    _transformer?: TSParserTransformer;
    _natifier?: TSParserNatifier;
    _mandatory?: boolean;
    _keysCase?: Nullable<TSCase>;
    _aliases?: Nullable<Map<string, string>>;
    _outputAliases?: Nullable<Map<string, string>>;
    _aliasUnsensitive?: boolean;
};
export type TSNode = TSLeafNode | TSObjectNode | TSArrayNode | TSExtendedArrayNode;
export declare abstract class TSParser {
    readonly errors: string[];
    readonly mandatory: boolean;
    protected _check: TSParserChecker;
    protected _transform: TSParserTransformer;
    protected _natify?: TSParserNatifier;
    static define(rootNode: TSNode, errors?: Nullable<Array<string>>): TSParser | null;
    get isValid(): boolean;
    validate(value: any, opts?: Nullable<TSParserOptions>): boolean;
    interpret(value: any, opts?: Nullable<TSParserOptions>): any;
    encode(value: any, opts?: Nullable<TSParserOptions>): any;
    parse(source: Nullable<string>, errors?: Nullable<string[]>): any;
    stringify(value: any, spaces?: string | number | undefined, errors?: Nullable<string[]>): string | null;
    abstract rawInterpret(_: any, opts?: Nullable<TSParserOptions>): any;
    abstract rawEncode(_: any, opts?: Nullable<TSParserOptions>): any;
    abstract nodeType(): TSParserNodeType;
    abstract toJSON(): object;
    toString(): string;
    includedDefaultValue(): any;
    protected _validate(value: any, path: string, opts?: Nullable<TSParserOptions>): boolean;
    protected constructor(mandatory: boolean, checker: Nullable<(v: any) => boolean>, transformer: Nullable<(v: any) => any>, natifier: Nullable<(v: any) => TSNativeValue>);
}
export declare function isLeafParserType(s: Nullable<string>): boolean;
export declare function $bool(v: any, opts?: Nullable<TSParserActionOptions>): boolean;
export declare function $validateParsedValue(v: NonNullable<any>, type: TSLeafOptionalNode, opts?: Nullable<TSParserActionOptions>): boolean;
