"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSRequest = exports.NO_HEADERS = exports.NO_BODY = exports.RespType = exports.Verb = exports.Resp = exports.$query = exports.$barerauth = exports.$basicauth = void 0;
const types_1 = require("./types");
const commons_1 = require("./commons");
const tserrors_1 = require("./tserrors");
const utils_1 = require("./utils");
const strings_1 = require("./strings");
const axios_1 = __importDefault(require("axios"));
const data_1 = require("./data");
const tsdata_1 = require("./tsdata");
const array_1 = require("./array");
const tscharset_1 = require("./tscharset");
const tsobject_1 = require("./tsobject");
function $basicauth(login, pwd, encoding) {
    return 'Basic ' + (0, data_1.$encodeBase64)(`${login}:${pwd}`, (0, tscharset_1.$charset)(encoding));
}
exports.$basicauth = $basicauth;
function $barerauth(base64StringOrData) {
    const tok = (0, commons_1.$isstring)(base64StringOrData) ? base64StringOrData : (0, data_1.$encodeBase64)(base64StringOrData);
    return `Bearer ${tok}`;
}
exports.$barerauth = $barerauth;
function $query(baseURL, query) {
    let params = new URLSearchParams();
    for (let [key, value] of Object.entries(query)) {
        key = (0, strings_1.$ftrim)(key);
        if (key.length && (0, commons_1.$ok)(value)) {
            if ((0, tsobject_1.$iscollection)(value)) {
                value = value.getItems();
            }
            if ((0, commons_1.$isarray)(value) || value instanceof Set) {
                let uniques = new Set();
                for (let v of (value || Set)) {
                    if ((0, commons_1.$ok)(v)) {
                        v = (0, commons_1.$string)(v);
                        if (!uniques.has(v)) {
                            uniques.add(v);
                            params.append(key, v);
                        }
                    }
                }
            }
            else {
                params.append(key, (0, commons_1.$string)(value));
            }
        }
    }
    const s = params.toString();
    return (0, commons_1.$length)(s) ? baseURL + '?' + s : baseURL;
}
exports.$query = $query;
var Resp;
(function (Resp) {
    Resp[Resp["Continue"] = 100] = "Continue";
    Resp[Resp["SwitchingProtocol"] = 101] = "SwitchingProtocol";
    Resp[Resp["Processing"] = 103] = "Processing";
    Resp[Resp["OK"] = 200] = "OK";
    Resp[Resp["Created"] = 201] = "Created";
    Resp[Resp["Accepted"] = 202] = "Accepted";
    Resp[Resp["NonAuthoritativeInformation"] = 203] = "NonAuthoritativeInformation";
    Resp[Resp["NoContent"] = 204] = "NoContent";
    Resp[Resp["ResetContent"] = 205] = "ResetContent";
    Resp[Resp["PartialContent"] = 206] = "PartialContent";
    Resp[Resp["MultiStatus"] = 207] = "MultiStatus";
    Resp[Resp["AlreadyReported"] = 208] = "AlreadyReported";
    Resp[Resp["ContentDifferent"] = 210] = "ContentDifferent";
    Resp[Resp["IMUsed"] = 226] = "IMUsed";
    Resp[Resp["MultipleChoices"] = 300] = "MultipleChoices";
    Resp[Resp["Moved"] = 301] = "Moved";
    Resp[Resp["Found"] = 302] = "Found";
    Resp[Resp["SeeOther"] = 303] = "SeeOther";
    Resp[Resp["NotModified"] = 304] = "NotModified";
    Resp[Resp["UseProxy"] = 305] = "UseProxy";
    Resp[Resp["SwitchProxy"] = 306] = "SwitchProxy";
    Resp[Resp["TemporaryRedirect"] = 307] = "TemporaryRedirect";
    Resp[Resp["PermanentRedirect"] = 308] = "PermanentRedirect";
    Resp[Resp["TooManyRedirects"] = 310] = "TooManyRedirects";
    Resp[Resp["BadRequest"] = 400] = "BadRequest";
    Resp[Resp["Unauthorized"] = 401] = "Unauthorized";
    Resp[Resp["PaymentRequired"] = 402] = "PaymentRequired";
    Resp[Resp["Forbidden"] = 403] = "Forbidden";
    Resp[Resp["NotFound"] = 404] = "NotFound";
    Resp[Resp["NotAllowed"] = 405] = "NotAllowed";
    Resp[Resp["NotAcceptable"] = 406] = "NotAcceptable";
    Resp[Resp["ProxyAuthenticationRequired"] = 407] = "ProxyAuthenticationRequired";
    Resp[Resp["TimeOut"] = 408] = "TimeOut";
    Resp[Resp["Conflict"] = 409] = "Conflict";
    Resp[Resp["Gone"] = 410] = "Gone";
    Resp[Resp["LengthRequired"] = 411] = "LengthRequired";
    Resp[Resp["PreconditionFailed"] = 412] = "PreconditionFailed";
    Resp[Resp["TooLarge"] = 413] = "TooLarge";
    Resp[Resp["URITooLong"] = 414] = "URITooLong";
    Resp[Resp["UnsupportedMedia"] = 415] = "UnsupportedMedia";
    Resp[Resp["RequestedRangeUnsatisfiable"] = 416] = "RequestedRangeUnsatisfiable";
    Resp[Resp["ExpectationFailed"] = 417] = "ExpectationFailed";
    Resp[Resp["TeaPot"] = 418] = "TeaPot";
    Resp[Resp["Misdirected"] = 421] = "Misdirected";
    Resp[Resp["Unprocessable"] = 422] = "Unprocessable";
    Resp[Resp["Locked"] = 423] = "Locked";
    Resp[Resp["MethodFailure"] = 424] = "MethodFailure";
    Resp[Resp["TooEarly"] = 425] = "TooEarly";
    Resp[Resp["UpgradeRequired"] = 426] = "UpgradeRequired";
    Resp[Resp["PreconditionRequired"] = 428] = "PreconditionRequired";
    Resp[Resp["TooManyRequests"] = 429] = "TooManyRequests";
    Resp[Resp["LegallyUnavailable"] = 451] = "LegallyUnavailable";
    Resp[Resp["Unrecoverable"] = 456] = "Unrecoverable";
    Resp[Resp["InternalError"] = 500] = "InternalError";
    Resp[Resp["NotImplemented"] = 501] = "NotImplemented";
    Resp[Resp["BadGateway"] = 502] = "BadGateway";
    Resp[Resp["Unavailable"] = 503] = "Unavailable";
    Resp[Resp["GatewayTimeOut"] = 504] = "GatewayTimeOut";
    Resp[Resp["NotSupported"] = 505] = "NotSupported";
    Resp[Resp["VariantAlsoNegotiates"] = 506] = "VariantAlsoNegotiates";
    Resp[Resp["InsufficientStorage"] = 507] = "InsufficientStorage";
    Resp[Resp["LoopDetected"] = 508] = "LoopDetected";
    Resp[Resp["BandwidthLimitExceeded"] = 509] = "BandwidthLimitExceeded";
    Resp[Resp["NotExtended"] = 510] = "NotExtended";
    Resp[Resp["NetworkAuthenticationRequired"] = 511] = "NetworkAuthenticationRequired";
})(Resp = exports.Resp || (exports.Resp = {}));
var Verb;
(function (Verb) {
    Verb["Get"] = "GET";
    Verb["Post"] = "POST";
    Verb["Put"] = "PUT";
    Verb["Delete"] = "DELETE";
    Verb["Patch"] = "PATCH";
})(Verb = exports.Verb || (exports.Verb = {}));
var RespType;
(function (RespType) {
    RespType["Json"] = "json";
    RespType["Buffer"] = "arraybuffer";
    RespType["String"] = "text";
    RespType["Stream"] = "stream";
})(RespType = exports.RespType || (exports.RespType = {}));
exports.NO_BODY = undefined;
exports.NO_HEADERS = {};
;
class TSRequest {
    constructor(baseURL = '', opts = {}) {
        this.token = '';
        this.basicAuth = '';
        this.defaultTimeOut = 1000;
        this.commonHeaders = {};
        this._baseURL = TSRequest.DefaultURL;
        this.baseURL = baseURL;
        this.commonHeaders = (0, commons_1.$ok)(opts === null || opts === void 0 ? void 0 : opts.headers) ? _standardHeaders(opts.headers) : {};
        if (opts.auth instanceof ArrayBuffer || opts.auth instanceof Uint8Array || opts.auth instanceof tsdata_1.TSData || (0, commons_1.$isstring)(opts.auth)) {
            this.setToken(opts.auth);
        }
        else if ((0, commons_1.$isarray)(opts.auth)) {
            for (let v of opts.auth) {
                if (!(0, commons_1.$isunsigned)(v, types_1.UINT8_MAX)) {
                    tserrors_1.TSError.throw('TSRequest.constructor(): malformed authentification token', { baseURL: baseURL, options: opts });
                }
            }
            this.setToken(opts.auth);
        }
        else if ((0, commons_1.$ok)(opts.auth)) {
            this.setAuth(opts.auth);
        }
        if ((0, commons_1.$ok)(opts.timeout) && opts.timeout < 0) {
            tserrors_1.TSError.throw('TSRequest.constructor(): if set, timeout option should be positive', { baseURL: baseURL, options: opts });
        }
        const commonTimeout = (0, commons_1.$tounsigned)(opts.timeout);
        if (commonTimeout > 0) {
            this.defaultTimeOut = commonTimeout;
        }
        this._managesCredential = !!opts.managesCredentials;
        this._baseURL = baseURL.length > 0 ? baseURL : TSRequest.DefaultURL;
        this._resetChannel();
    }
    get baseURL() { return this._baseURL; }
    set baseURL(s) {
        if (!s.length) {
            s = TSRequest.DefaultURL;
        }
        if (s !== this._baseURL) {
            this._baseURL = s;
            this._resetChannel();
        }
    }
    get managesCredential() { return this._managesCredential; }
    set managesCredential(flag) {
        if ((flag && !this._managesCredential) || (!flag && this._managesCredential)) {
            this._managesCredential = flag;
            this._resetChannel();
        }
    }
    setAuth(auth) {
        if ((0, commons_1.$ok)(auth) && (0, commons_1.$length)(auth.login)) {
            this.basicAuth = $basicauth(auth.login, auth.password, auth.encoding);
        }
        else {
            this.basicAuth = '';
        }
    }
    setToken(token) {
        if ((0, commons_1.$length)(token)) {
            token = $barerauth(token);
        }
        else {
            this.token = '';
        }
    }
    request(relativeURL, method, responseType, statuses = [200], body, suplHeaders, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            const resp = yield this.req(relativeURL, method, responseType, body, suplHeaders, timeout);
            return [statuses.includes(resp.status) ? resp.response : null, resp.status];
        });
    }
    req(relativeURL, method = Verb.Get, responseType = RespType.Json, body = null, suplHeaders = {}, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = {
                url: relativeURL,
                method: method,
                responseType: responseType,
                headers: Object.assign(Object.assign({}, this.commonHeaders), _standardHeaders(suplHeaders)),
                validateStatus: () => true
            };
            if ((0, commons_1.$length)(this.token)) {
                config.headers['Authorization'] = this.token;
            }
            else if ((0, commons_1.$length)(this.basicAuth)) {
                config.headers['Authorization'] = this.basicAuth;
            }
            if ((0, commons_1.$ok)(body)) {
                if (body instanceof tsdata_1.TSData) {
                    config.data = body.mutableBuffer;
                }
                else if (body instanceof Uint8Array) {
                    config.data = (0, data_1.$arrayBufferFromBytes)(body);
                }
                else {
                    config.data = body;
                }
            }
            ;
            if ((0, commons_1.$ok)(timeout) && timeout < 0) {
                tserrors_1.TSError.throw('TSRequest.req(): if set, timeout parameter should be positive or 0', {
                    relativeURL: relativeURL,
                    method: method,
                    responseType: responseType,
                    body: body,
                    suplHeaders: suplHeaders,
                    timeout: timeout
                });
            }
            timeout = (0, commons_1.$tounsigned)(timeout);
            if (!timeout) {
                timeout = this.defaultTimeOut;
            }
            let ret = null;
            let status = 0;
            let headers = {};
            const timeoutError = tserrors_1.TSUniqueError.timeoutError();
            try {
                const response = yield (0, utils_1.$timeout)(this._channel(config), timeout, timeoutError);
                ret = responseType === RespType.Buffer ? Buffer.from(response.data) : response.data;
                status = response.status;
                headers = response.headers;
            }
            catch (e) {
                if (e === timeoutError || e.code === 'ECONNABORTED' || e.code === 'ETIMEDOUT' || e.code === 'ERR_PARSE_TIMEOUT') {
                    ret = null;
                    status = Resp.TimeOut;
                }
                else if (e.code === 'ECONNREFUSED') {
                    ret = null;
                    status = Resp.Misdirected;
                }
                else if ((0, commons_1.$isnumber)(e.statusCode)) {
                    ret = null;
                    status = e.statusCode;
                }
                else if ((0, commons_1.$isnumber)(e.status)) {
                    ret = null;
                    status = e.status;
                }
                else {
                    throw e;
                }
            }
            return { status: status, response: ret, headers: headers };
        });
    }
    _resetChannel() {
        this._channel = axios_1.default.create({ baseURL: this._baseURL, withCredentials: !!this._managesCredential });
    }
}
exports.TSRequest = TSRequest;
TSRequest.DefaultURL = 'http://localhost/';
function _standardHeaders(headers) {
    const entries = (0, commons_1.$ok)(headers) ? Object.entries(headers) : [];
    const ret = {};
    for (let [key, value] of entries) {
        ret[key.capitalize()] = (0, commons_1.$isarray)(value) ? (0, array_1.$map)(value, i => `${i}`) : `${value}`;
    }
    return ret;
}
URLSearchParams.prototype.query = function query() {
    const ret = {};
    let total = 0;
    for (const [key, value] of this) {
        if (key.length > 0) {
            ret[key] = value;
            total++;
        }
    }
    return total > 0 ? ret : null;
};
//# sourceMappingURL=tsrequest.js.map