/// <reference types="node" />
/// <reference types="node" />
import * as http from "http";
import { Nullable, TSDictionary, uint16 } from "./types";
import { TSPreflightController, TSEndpointsDefinition, TSServerStartStatus, TSWebSiteDefinition } from "./tsserver_types";
export type TSServerLogger = (server: TSServer, req: http.IncomingMessage | undefined, type: TSServerLogType, messages: string) => Promise<void>;
export declare enum TSServerLogType {
    Developer = "Dev",
    Log = "Info",
    Warning = "Warning",
    Error = "Error"
}
export interface TSServerOptions {
    logger?: TSServerLogger;
    host?: string;
    port?: uint16;
    rootPath?: string;
    webSites?: TSDictionary<TSWebSiteDefinition | string>;
    logInfo?: boolean;
    developer?: boolean;
    key?: Nullable<Buffer>;
    certificate?: Nullable<Buffer>;
    maxHeaderSize?: Nullable<number>;
    tlsSecTimeout?: Nullable<number>;
    handshakeMsTimeout?: Nullable<number>;
    forceCloseMsTimeout?: Nullable<number>;
    preflightController?: Nullable<TSPreflightController> | 'default' | 'permisive';
    clearCacheInterval?: Nullable<number>;
}
export declare class TSServer {
    private static __server;
    private static readonly DEFAULT_CLOSE_TIMEOUT;
    private static __methodColors;
    private static _internalClearTimer;
    private static _lastClearTimerInterval;
    readonly host: string;
    readonly port: number;
    readonly rootPath: string;
    readonly isHTTPs: boolean;
    private _serverOptions;
    private _httpServer;
    private _endPoints;
    private _sites;
    private _logger;
    private _logInfo;
    private _connections;
    private _terminating;
    private _forceCloseTimeout;
    private _developerMode;
    private _preflightControler;
    private _allowedMethodsSet;
    private _preflightResponseCache;
    static start(endPoints: Nullable<TSDictionary<TSEndpointsDefinition>>, opts?: Nullable<TSServerOptions>): Promise<TSServerStartStatus | Error>;
    static isRunning(): Promise<boolean>;
    static clearCaches(): Promise<void>;
    static stopClearCachesInterval(): void;
    static setClearCachesInterval(seconds?: Nullable<number>): void;
    static stop(): Promise<Error | undefined>;
    private constructor();
    private _start;
    private _onConnection;
    private _onRequest;
    private _requestDidEnd;
    private _clearCaches;
    private _stop;
    private _internalStopServer;
}
