"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSServer = exports.TSServerLogType = void 0;
const http = __importStar(require("http"));
const https = __importStar(require("https"));
const commons_1 = require("./commons");
const strings_1 = require("./strings");
const fs_1 = require("./fs");
const tserrors_1 = require("./tserrors");
const tsrequest_1 = require("./tsrequest");
const types_1 = require("./types");
const utils_1 = require("./utils");
const mapset_1 = require("./mapset");
const tsurl_1 = require("./tsurl");
const tsserver_types_1 = require("./tsserver_types");
const tsserver_endpoints_1 = require("./tsserver_endpoints");
const tsserver_websites_1 = require("./tsserver_websites");
var TSServerLogType;
(function (TSServerLogType) {
    TSServerLogType["Developer"] = "Dev";
    TSServerLogType["Log"] = "Info";
    TSServerLogType["Warning"] = "Warning";
    TSServerLogType["Error"] = "Error";
})(TSServerLogType = exports.TSServerLogType || (exports.TSServerLogType = {}));
;
class TSServer {
    static start(endPoints, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            if (TSServer.__server) {
                return tsserver_types_1.TSServerStartStatus.AlreadyRunning;
            }
            if ((0, commons_1.$objectcount)(endPoints) === 0 && (0, commons_1.$objectcount)(opts === null || opts === void 0 ? void 0 : opts.webSites) === 0) {
                return new tserrors_1.TSError('TSServer must at least serve one static website or one end point', {
                    endPoints: endPoints,
                    options: opts
                });
            }
            try {
                TSServer.__server = new TSServer((0, commons_1.$value)(endPoints, {}), (0, commons_1.$value)(opts, {}));
            }
            catch (e) {
                return e;
            }
            yield TSServer.__server._start();
            return TSServer.__server.isHTTPs ? tsserver_types_1.TSServerStartStatus.HTTPS : tsserver_types_1.TSServerStartStatus.HTTP;
        });
    }
    static isRunning() {
        return __awaiter(this, void 0, void 0, function* () { return (0, commons_1.$ok)(TSServer.__server); });
    }
    static clearCaches() {
        return __awaiter(this, void 0, void 0, function* () {
            if (TSServer.__server) {
                yield TSServer.__server._clearCaches();
            }
            else {
                TSServer.stopClearCachesInterval();
            }
        });
    }
    static stopClearCachesInterval() {
        if ((0, commons_1.$ok)(TSServer._internalClearTimer)) {
            clearInterval(TSServer._internalClearTimer);
            TSServer._internalClearTimer = undefined;
        }
    }
    static setClearCachesInterval(seconds) {
        this.stopClearCachesInterval();
        this._lastClearTimerInterval = Math.max(Math.min((0, commons_1.$isnumber)(seconds) ? seconds : TSServer._lastClearTimerInterval, 86400), 60);
        TSServer._internalClearTimer = setInterval(__clearCaches, this._lastClearTimerInterval * 1000);
    }
    static stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (TSServer.__server) {
                const e = yield TSServer.__server._stop();
                if (!(0, commons_1.$ok)(e)) {
                    TSServer.stopClearCachesInterval();
                    delete TSServer.__server;
                    return undefined;
                }
                return e;
            }
            return undefined;
        });
    }
    constructor(endPoints, opts = {}) {
        this.rootPath = '';
        this.isHTTPs = false;
        this._serverOptions = {};
        this._connections = new Map;
        this._terminating = false;
        this._allowedMethodsSet = new Set(Object.values(tsrequest_1.Verb));
        this._preflightResponseCache = new Map();
        this._logger = (0, commons_1.$ok)(opts.logger) ? opts.logger : _internalLogger;
        this._developerMode = !!opts.developer;
        this._logInfo = !!opts.logInfo || this._developerMode;
        this.isHTTPs = (0, commons_1.$length)(opts.key) > 0 && (0, commons_1.$length)(opts.certificate) > 0;
        if (this.isHTTPs) {
            this._serverOptions.cert = opts.certificate;
            this._serverOptions.key = opts.key;
            let t = (0, commons_1.$unsigned)(opts.tlsSecTimeout);
            if (t > 0) {
                this._serverOptions.sessionTimeout = t;
            }
            t = (0, commons_1.$unsigned)(opts.handshakeMsTimeout);
            if (t > 0) {
                this._serverOptions.handshakeTimeout = t;
            }
        }
        const n = (0, commons_1.$unsigned)(opts.maxHeaderSize);
        if (n > 0) {
            this._serverOptions.maxHeaderSize = n;
        }
        const ft = (0, commons_1.$unsigned)(opts.forceCloseMsTimeout);
        this._forceCloseTimeout = ft === 0 ? TSServer.DEFAULT_CLOSE_TIMEOUT : ft;
        const rp = (0, strings_1.$ftrim)(opts.rootPath);
        if (rp.length > 1 && (0, fs_1.$isabsolutepath)(rp)) {
            this.rootPath = rp;
        }
        this._sites = [];
        if ((0, commons_1.$ok)(opts.webSites)) {
            const keys = (0, commons_1.$keys)(opts.webSites);
            if ((0, utils_1.$inbrowser)() && keys.length) {
                tserrors_1.TSError.throw('TSServer cannot handle static websides inside a browser', { endPoints: endPoints, options: opts });
            }
            if ((0, commons_1.$count)(keys)) {
                keys.forEach(u => {
                    this._sites.push(new tsserver_websites_1.TSStaticWebsite(u, opts.webSites[u]));
                });
            }
        }
        this._endPoints = [];
        (0, commons_1.$keys)(endPoints).forEach(path => {
            this._endPoints.push(new tsserver_endpoints_1.TSServerEndPoint(path, endPoints[path]));
        });
        if ((0, commons_1.$ok)(opts.port)) {
            if (!(0, commons_1.$isunsigned)(opts.port, types_1.UINT16_MAX)) {
                tserrors_1.TSError.throw(`TSServer.constructor(): Bad ${this.isHTTPs ? 'HTTPS' : 'HTTP'} server port ${opts.port}.`, {
                    endPoints: endPoints,
                    options: opts
                });
            }
            this.port = opts.port;
        }
        else {
            this.port = 3000;
        }
        this.host = (0, strings_1.$ftrim)(opts.host);
        if (!this.host.length) {
            this.host = `${this.isHTTPs ? 'https' : 'http'}://localhost/`;
        }
        if (!(0, commons_1.$ok)(opts.preflightController) || opts.preflightController === 'default') {
            this._preflightControler = _defaultPreflightController;
        }
        else if (opts.preflightController === 'permisive') {
            this._preflightControler = _permisivePreflightController;
        }
        else {
            this._preflightControler = opts.preflightController;
        }
        TSServer.setClearCachesInterval(opts === null || opts === void 0 ? void 0 : opts.clearCacheInterval);
    }
    _start() {
        return __awaiter(this, void 0, void 0, function* () {
            const managementCallBack = (req, res) => __awaiter(this, void 0, void 0, function* () {
                var _a, _b;
                try {
                    const sm = (0, commons_1.$value)((_a = req.method) === null || _a === void 0 ? void 0 : _a.toUpperCase(), tsrequest_1.Verb.Get);
                    const potentialUrl = tsurl_1.TSURL.compose(this.host, (0, commons_1.$string)(req.url));
                    if (!(0, commons_1.$ok)(potentialUrl)) {
                        tserrors_1.TSError.throw('Bad request url endpoint', tsrequest_1.Resp.NotFound, {
                            host: this.host,
                            path: (0, commons_1.$string)(req.url),
                            serverError: tsserver_types_1.TSServerErrorCodes.BadEndPointPath
                        });
                    }
                    const url = potentialUrl;
                    const originKey = `${url.pathname}[${(0, commons_1.$value)(req.headers["origin"], '*')}]`;
                    if (sm === 'OPTIONS') {
                        const preflightResponse = yield this._preflightControler(url, req.headers, res);
                        if ((0, commons_1.$ok)(preflightResponse)) {
                            const npfr = _normalizePreflightResponse(preflightResponse, this._allowedMethodsSet);
                            res.setHeader("Access-Control-Allow-Origin", npfr.allowedOrigin);
                            res.setHeader("Access-Control-Allow-Methods", npfr.allowedMethods);
                            res.setHeader("Access-Control-Allow-Headers", npfr.allowedHeaders);
                            res.setHeader("Access-Control-Max-Age", (0, commons_1.$string)(npfr.timeout));
                            res.writeHead(tsrequest_1.Resp.NoContent);
                            res.end();
                            npfr.timeout = npfr.timeout + (0, utils_1.$mark)();
                            this._preflightResponseCache.set(originKey, npfr);
                            if (this._logInfo) {
                                yield this._logger(this, req, TSServerLogType.Log, `did accept preflight request for resource '${url.pathname}'.`);
                            }
                        }
                        else {
                            res.writeHead(tsrequest_1.Resp.NotAllowed);
                            res.end();
                            yield this._logger(this, req, TSServerLogType.Warning, `did refuse preflight request for resource '${url.pathname}'.`);
                        }
                        return;
                    }
                    const method = this._allowedMethodsSet.has(sm) ? sm : undefined;
                    if (!(0, commons_1.$ok)(method)) {
                        tserrors_1.TSError.throw(`Request method '${req.method}' is not allowed.`, tsrequest_1.Resp.NotAllowed, {
                            method: req.method,
                            path: (0, commons_1.$length)(url.pathname) ? url.pathname : '/',
                            serverError: tsserver_types_1.TSServerErrorCodes.UnknownMethod
                        });
                    }
                    if (!(0, commons_1.$length)(url.pathname) || url.pathname === '/') {
                        if (this.rootPath.length) {
                            url.pathname = this.rootPath;
                        }
                        else {
                            tserrors_1.TSError.throw('Root path is not Accessible', tsrequest_1.Resp.Forbidden, {
                                method: req.method,
                                path: '/',
                                serverError: tsserver_types_1.TSServerErrorCodes.InnaccessibleRoot
                            });
                        }
                    }
                    let sep = undefined;
                    let parameters = {};
                    this._endPoints.forEach(ep => {
                        const params = ep.parametersFromPath(url.pathname);
                        if ((0, commons_1.$ok)(params) && (!(0, commons_1.$defined)(sep) || sep.depth < ep.depth)) {
                            sep = ep;
                            parameters = params;
                        }
                    });
                    if ((0, commons_1.$ok)(sep)) {
                        let preflightResponse = this._preflightResponseCache.get(originKey);
                        if ((0, commons_1.$ok)(preflightResponse) && (!(0, commons_1.$ok)(preflightResponse === null || preflightResponse === void 0 ? void 0 : preflightResponse.timeout) || preflightResponse.timeout >= (0, utils_1.$mark)())) {
                            this._preflightResponseCache.delete(originKey);
                            preflightResponse = undefined;
                        }
                        if ((0, commons_1.$ok)(preflightResponse === null || preflightResponse === void 0 ? void 0 : preflightResponse.allowedOrigin)) {
                            if ((0, commons_1.$ok)(preflightResponse.timeout) && preflightResponse.timeout < (0, utils_1.$mark)()) {
                                res.setHeader('Access-Control-Allow-Origin', preflightResponse.allowedOrigin);
                            }
                        }
                        yield sep.execute({
                            url: url,
                            method: method,
                            parameters: parameters,
                            message: req,
                            developerMode: this._developerMode,
                            query: {},
                            body: undefined
                        }, res);
                        if (this._logInfo) {
                            yield this._logger(this, req, TSServerLogType.Log, `did handle ${TSServer.__methodColors[method]}${method}&w resource '${url.pathname}'.`);
                        }
                        return;
                    }
                    else if (method === tsrequest_1.Verb.Get) {
                        for (let s of this._sites) {
                            const [b, type] = s.getStaticResource(url.pathname);
                            if ((0, commons_1.$ok)(b)) {
                                if (this._logInfo) {
                                    yield this._logger(this, req, TSServerLogType.Log, `did handle static resource '${url.pathname}'.`);
                                }
                                res.setHeader('Content-Type', type);
                                res.writeHead(tsrequest_1.Resp.OK);
                                res.end(b);
                                return;
                            }
                        }
                    }
                    tserrors_1.TSError.throw(`endpoint ${method} '${url.pathname}' was not found.`, tsrequest_1.Resp.NotFound, {
                        method: method,
                        path: (0, commons_1.$length)(url.pathname) ? url.pathname : '/',
                        serverError: tsserver_types_1.TSServerErrorCodes.ResourceNotFound
                    });
                }
                catch (e) {
                    let ret = {};
                    ret.status = (0, commons_1.$isunsigned)(e === null || e === void 0 ? void 0 : e.status) && Object.values(tsrequest_1.Resp).includes(e.status) ? e.status : tsrequest_1.Resp.InternalError;
                    ret.error = e.message;
                    if (!(0, commons_1.$length)(ret.error)) {
                        ret.error = 'Unknown internal Error';
                    }
                    ;
                    if ((0, commons_1.$ok)(e.info)) {
                        ret.info = e.info;
                    }
                    if ((0, commons_1.$isnumber)(e.errorCode)) {
                        ret.errorCode = e.errorCode;
                    }
                    yield this._logger(this, req, TSServerLogType.Warning, `${ret.status} - ${ret.error}`);
                    const errors = (_b = ret.info) === null || _b === void 0 ? void 0 : _b.errors;
                    if ((0, commons_1.$isarray)(errors)) {
                        const n = (0, commons_1.$count)(errors);
                        for (let i = 0; i < n; i++) {
                            yield this._logger(this, req, TSServerLogType.Warning, `      + error[${i + 1}]: ${(0, commons_1.$string)(errors[i])}`);
                        }
                    }
                    res.writeHead(ret.status, { 'Content-Type': 'application/json' });
                    res.end(JSON.stringify(ret));
                }
            });
            this._httpServer = this.isHTTPs ?
                https.createServer(this._serverOptions, managementCallBack) :
                http.createServer(this._serverOptions, managementCallBack);
            this._httpServer.on('connection', this._onConnection.bind(this));
            this._httpServer.on('request', this._onRequest.bind(this));
            this._httpServer.listen(this.port, () => __awaiter(this, void 0, void 0, function* () {
                if (this._logInfo) {
                    yield this._logger(this, undefined, TSServerLogType.Log, `running on port ${this.port} '${this.host}' ...`);
                }
            }));
        });
    }
    _onConnection(socket) {
        this._connections.set(socket, { refcount: 0, closeHeaderSent: false });
        socket.once('close', () => this._connections.delete(socket));
    }
    _onRequest(request, response) {
        const socket = request.socket;
        const info = this._connections.get(socket);
        if (info) {
            info.refcount++;
            if (this._terminating && !response.headersSent) {
                response.setHeader('Connection', 'close');
                info.closeHeaderSent = true;
            }
        }
        response.on('finish', () => { this._requestDidEnd(request); });
    }
    _requestDidEnd(request) {
        const socket = request.socket;
        const info = this._connections.get(socket);
        if (info) {
            info.refcount--;
            if (this._terminating && info.refcount === 0 && info.closeHeaderSent) {
                socket.end();
            }
        }
    }
    _clearCaches() {
        return __awaiter(this, void 0, void 0, function* () {
            const timestamp = (0, utils_1.$mark)();
            (0, mapset_1.$conditionalClearMap)(this._preflightResponseCache, (_, v) => !(0, commons_1.$ok)(v.timeout) || v.timeout > timestamp);
            for (let s of this._sites) {
                yield s.clearCaches();
            }
        });
    }
    _stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._logInfo) {
                yield this._logger(this, undefined, TSServerLogType.Log, `server is exiting...`);
            }
            if ((0, commons_1.$ok)(this._httpServer)) {
                const ret = yield this._internalStopServer();
                if ((0, commons_1.$ok)(ret)) {
                    yield this._logger(this, undefined, TSServerLogType.Error, `cannot stop for reason ${ret.name}:\n${ret.message}`);
                }
                return ret;
            }
            ;
            return undefined;
        });
    }
    _internalStopServer() {
        return __awaiter(this, void 0, void 0, function* () {
            if ((0, commons_1.$ok)(this._httpServer)) {
                this._terminating = true;
                const ret = new Promise((resolve, reject) => {
                    this._httpServer.close((error) => {
                        if ((0, commons_1.$ok)(error)) {
                            return reject(error);
                        }
                        resolve(undefined);
                    });
                });
                this._connections.forEach((info, socket) => {
                    if (info.refcount === 0) {
                        socket.end();
                    }
                });
                setTimeout(() => {
                    this._connections.forEach((_, socket) => {
                        if ((0, commons_1.$ismethod)(socket, 'destroy')) {
                            socket.destroy();
                        }
                        else {
                            socket.end();
                        }
                    });
                    this._connections.clear();
                }, this._forceCloseTimeout);
                return ret;
            }
            return undefined;
        });
    }
}
exports.TSServer = TSServer;
TSServer.__server = undefined;
TSServer.DEFAULT_CLOSE_TIMEOUT = 15000;
TSServer.__methodColors = { GET: '&j', POST: '&u', PUT: '&v', DELETE: '&r', PATCH: '&o' };
TSServer._internalClearTimer = undefined;
TSServer._lastClearTimerInterval = 900;
function __clearCaches() {
    return __awaiter(this, void 0, void 0, function* () { TSServer.clearCaches(); });
}
let __logIndex = 0;
const _internalLogger = (server, _, type, message) => __awaiter(void 0, void 0, void 0, function* () {
    if (!__logIndex) {
        (0, utils_1.$logterm)('');
    }
    (0, utils_1.$logterm)(`&0&xfoundation-ts[&w${server.isHTTPs ? 'https' : 'http'} server&x:${server.port}]-${__TSServerlogHeaders[type]}&0 &w${message} &d(${++__logIndex})&0`);
});
const __TSServerlogHeaders = {
    'Dev': "&X&w DEVELOP ",
    'Info': "&C&w LOGGING ",
    'Warning': "&O&w WARNING ",
    'Error': "&R&w  ERROR  "
};
function _normalizePreflightResponse(r, verbSet) {
    let origin = (0, strings_1.$ftrim)(r.allowedOrigin);
    if (!origin.length) {
        origin = '*';
    }
    let timeout = (0, commons_1.$unsigned)(r.timeout);
    if (!timeout || timeout > 86400) {
        timeout = 86400;
    }
    if (timeout < 10) {
        timeout = 10;
    }
    let methodSet = (0, commons_1.$strings)(r.allowedMethods).filteredSet(m => {
        m = (0, strings_1.$ftrim)(m).toUpperCase();
        return verbSet.has(m) ? m : undefined;
    });
    let methods = methodSet.size && !methodSet.has('*') ? Array.from(methodSet) : Object.values(tsrequest_1.Verb);
    methods.push('OPTIONS');
    let headersSet = (0, commons_1.$strings)(r.allowedHeaders).filteredSet(h => {
        h = (0, strings_1.$ftrim)(h).toLowerCase();
        return h.length ? h : undefined;
    });
    let headers;
    if (headersSet.has('*')) {
        headers = '*';
    }
    else {
        methodSet.add('content-type');
        methodSet.add('accept');
        headers = Array.from(headersSet);
    }
    return { allowedOrigin: origin, allowedMethods: methods, allowedHeaders: headers, timeout: timeout };
}
function _permisivePreflightController(url, headers, res) {
    return __awaiter(this, void 0, void 0, function* () {
        return { allowedOrigin: undefined, allowedHeaders: undefined, allowedMethods: undefined };
    });
}
function _defaultPreflightController(url, headers, res) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const acceptedHeaders = (0, commons_1.$value)((_a = headers["access-control-allow-headers"]) === null || _a === void 0 ? void 0 : _a.split(',').map(s => (0, strings_1.$ftrim)(s).toLowerCase()), []);
        if (!acceptedHeaders.includes('content-type')) {
            acceptedHeaders.push('content-type');
        }
        const acceptedMethods = (_b = headers["access-control-request-method"]) === null || _b === void 0 ? void 0 : _b.split(',').map(s => (0, strings_1.$ftrim)(s).toUpperCase());
        return { allowedOrigin: headers["origin"], allowedHeaders: acceptedHeaders, allowedMethods: acceptedMethods };
    });
}
//# sourceMappingURL=tsserver.js.map