"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSServerEndPoint = void 0;
const commons_1 = require("./commons");
const strings_1 = require("./strings");
const tserrors_1 = require("./tserrors");
const tsparser_1 = require("./tsparser");
const tsrequest_1 = require("./tsrequest");
const tsserver_types_1 = require("./tsserver_types");
const utils_1 = require("./utils");
const tscharset_1 = require("./tscharset");
class TSServerEndPoint {
    constructor(path, definition) {
        path = (0, strings_1.$ftrim)(path);
        const len = path.length;
        if ((0, commons_1.$isfunction)(definition)) {
            definition = { GET: { controller: definition } };
        }
        else if ((0, commons_1.$isproperty)(definition, 'controller')) {
            definition = { GET: definition };
        }
        const def = definition;
        const errorOptions = { path: path, definition: def };
        if (len < 2) {
            tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): end points path '${path}' is too short`, errorOptions);
        }
        const methods = (0, commons_1.$keys)(def);
        if (methods.length === 0) {
            tserrors_1.TSError.throw(`TSServerEndPoint.constructor() : end points path '${path}' has no method defined.`, errorOptions);
        }
        this._managers = new Map();
        methods.forEach(m => {
            if (!Object.values(tsrequest_1.Verb).includes(m)) {
                tserrors_1.TSError.throw(`TSServerEndPoint.constructor() : end points path '${path}' did define invalid '${m}' request method.`, errorOptions);
            }
            this._managers.set(m, new TSServerEndPointManager(m, ((0, commons_1.$isfunction)(def[m]) ? { controller: def[m] } : def[m]), errorOptions));
        });
        this.uri = '/';
        this._tokens = [];
        this.depth = 1;
        let State;
        (function (State) {
            State[State["Start"] = 0] = "Start";
            State[State["Standard"] = 1] = "Standard";
            State[State["Bracket"] = 2] = "Bracket";
            State[State["Token"] = 3] = "Token";
            State[State["TokenType"] = 4] = "TokenType";
        })(State || (State = {}));
        ;
        let state = State.Start;
        let constructUri = true;
        let regString = '';
        let currentToken = '';
        let currentType = '';
        for (let i = 0; i < len; i++) {
            const c = path.charAt(i);
            switch (state) {
                case State.Start:
                    if (c !== '/') {
                        tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): end points path '${path}' is not absolute.`, Object.assign({ position: i }, errorOptions));
                    }
                    state = State.Standard;
                    break;
                case State.Standard:
                    switch (c) {
                        case '{':
                            state = State.Bracket;
                            constructUri = false;
                            break;
                        case '}':
                            tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): Misplaced '}' character in path '${path}'.`, Object.assign({ position: i }, errorOptions));
                        case '/':
                            if (constructUri) {
                                this.uri += c;
                            }
                            else {
                                regString += '\\/';
                            }
                            this.depth++;
                            break;
                        case '-':
                        case '$':
                        case '+':
                        case '*':
                        case '?':
                        case '.':
                        case '(':
                        case ')':
                            if (constructUri) {
                                this.uri += c;
                            }
                            else {
                                regString += '\\' + c;
                            }
                            break;
                        case '^':
                        case '\\':
                        case '[':
                        case ']':
                        case '`':
                        case '|':
                            tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): found forbidden character '\\u${c.charCodeAt(0).toHex4}' in path '${path}'.`, Object.assign({ position: i, character: c }, errorOptions));
                        default:
                            if (constructUri) {
                                this.uri += c.toLowerCase();
                            }
                            else {
                                regString += c;
                            }
                            break;
                    }
                    break;
                case State.Bracket:
                    if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
                        currentToken = c;
                        state = State.Token;
                        break;
                    }
                    tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): found forbidden first character '\\u${c.charCodeAt(0).toHex4}' in parametric token in path '${path}'.`, Object.assign({ position: i, character: c }, errorOptions));
                case State.Token:
                    if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') || c === '-' || c === '_' || c === '.') {
                        currentToken += c;
                        break;
                    }
                    else if (c === ':') {
                        state = State.TokenType;
                        currentType = '';
                        break;
                    }
                    else if (c === '}') {
                        this._tokens.push({ name: currentToken, parser: tsparser_1.TSParser.define('string') });
                        regString += '([a-zA-Z0-9][.a-zA-Z0-9_\-]*)';
                        currentToken = '';
                        currentType = '';
                        state = State.Standard;
                        break;
                    }
                    tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): found forbidden character '\\u${c.charCodeAt(0).toHex4}' in parametric token in path '${path}'.`, Object.assign({ position: i, character: c }, errorOptions));
                case State.TokenType:
                    if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
                        currentType += c.toLowerCase();
                        break;
                    }
                    else if (c === '}') {
                        if ((0, tsparser_1.isLeafParserType)(currentType)) {
                            this._tokens.push({ name: currentToken, parser: tsparser_1.TSParser.define(currentType) });
                            regString += '([a-zA-Z0-9][.a-zA-Z0-9_\-]*)';
                            currentToken = '';
                            currentType = '';
                            state = State.Standard;
                            break;
                        }
                        tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): found unknown type '${currentType}' of token '${currentToken}' in path '${path}'.`, Object.assign({ position: i, character: c }, errorOptions));
                    }
                    tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): found forbidden first character '\\u${c.charCodeAt(0).toHex4}' in parametric type of token '${currentToken}' in path '${path}'.`, Object.assign({ position: i, character: c }, errorOptions));
            }
        }
        if (state !== State.Standard) {
            tserrors_1.TSError.throw(`TSServerEndPoint.constructor(): malformed parametric path '${path}'.`, errorOptions);
        }
        this._pathRegex = regString.length === 0 ? null : new RegExp('^' + regString + '$', 'i');
    }
    get isStatic() { return !(0, commons_1.$ok)(this._pathRegex); }
    parametersFromPath(path) {
        if (path.startsWith(this.uri)) {
            const parametricPart = path.slice(this.uri.length);
            const ret = {};
            if (!parametricPart.length && !(0, commons_1.$ok)(this._pathRegex)) {
                return ret;
            }
            else if (parametricPart.length && (0, commons_1.$ok)(this._pathRegex)) {
                const m = parametricPart.match(this._pathRegex);
                if ((0, commons_1.$ok)(m)) {
                    const len = this._tokens.length;
                    if (m.length === len + 1) {
                        for (let i = 0; i < len; i++) {
                            const token = this._tokens[i];
                            const value = m[i + 1];
                            if (!token.parser.validate(value)) {
                                tserrors_1.TSError.throw(`Bad parameter '${token.name}' of type '${token.parser.nodeType}' in url path '${path}'`, tsrequest_1.Resp.BadRequest, {
                                    token: token.name,
                                    type: token.parser.nodeType,
                                    path: path,
                                    serverError: tsserver_types_1.TSServerErrorCodes.BadParameter
                                });
                            }
                            ret[token.name] = token.parser.rawInterpret(value);
                        }
                    }
                    return ret;
                }
            }
        }
        return null;
    }
    execute(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const manager = this._managers.get(req.method);
            if (!(0, commons_1.$ok)(manager)) {
                tserrors_1.TSError.throw(`Method '${req.method}' not implemented on url '${req.url.pathname}'`, tsrequest_1.Resp.NotImplemented, {
                    method: req.method,
                    path: (0, commons_1.$length)(req.url.pathname) ? req.url.pathname : '/',
                    serverError: tsserver_types_1.TSServerErrorCodes.MethodNotImplemented
                });
            }
            _developerLog(req, 'will execute manager', manager === null || manager === void 0 ? void 0 : manager.developerDesc(req.url.pathname));
            yield manager.execute(req, res);
        });
    }
}
exports.TSServerEndPoint = TSServerEndPoint;
class TSServerEndPointManager {
    constructor(m, ep, errorOptions) {
        this._queryParser = _defineParser(ep.query, 'query', errorOptions);
        this._bodyParser = _defineParser(ep.body, 'body', errorOptions);
        this._responseParser = _defineParser(ep.response, 'response', errorOptions);
        if (!(0, commons_1.$isfunction)(ep.controller)) {
            tserrors_1.TSError.throw(`TSServerEndPointManager.constructor() : end points path '${errorOptions.path}' has no valid controller function.`, errorOptions);
        }
        this._controller = ep.controller;
        this._method = m;
    }
    developerDesc(msg) { return `${this._method}:${(0, commons_1.$string)(msg)} [queryParser:${_yesNo(this._queryParser)}, bodyParser:${_yesNo(this._bodyParser)}, responseParser:${_yesNo(this._responseParser)}]`; }
    execute(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            if ((0, commons_1.$ok)(this._queryParser)) {
                const params = req.url.searchParams.query();
                const options = { errors: [], context: 'url' };
                _developerLog(req, 'request query', params);
                if (!this._queryParser.validate(params, options)) {
                    _developerLog(req, 'impossible to validate query');
                    tserrors_1.TSError.throw(`Bad query for ${this._method} request on url '${req.url.pathname}'`, tsrequest_1.Resp.BadRequest, {
                        method: this._method,
                        path: (0, commons_1.$length)(req.url.pathname) ? req.url.pathname : '/',
                        errors: options.errors,
                        serverError: tsserver_types_1.TSServerErrorCodes.BadQueryStructure
                    });
                }
                req.query = this._queryParser.rawInterpret(params);
                _developerLog(req, 'final interpreted query', req.query);
            }
            if ((0, commons_1.$ok)(this._bodyParser)) {
                let body = yield (0, utils_1.$readStreamBuffer)(req.message);
                const options = { errors: [] };
                _developerLog(req, 'request body', body);
                if (!(0, commons_1.$ok)(body) && this._bodyParser.mandatory) {
                    _developerLog(req, 'missing body content');
                    tserrors_1.TSError.throw(`No body content for ${this._method} request on url '${req.url.pathname}'`, tsrequest_1.Resp.BadRequest, {
                        method: this._method,
                        path: (0, commons_1.$length)(req.url.pathname) ? req.url.pathname : '/',
                        serverError: tsserver_types_1.TSServerErrorCodes.MissingBody
                    });
                }
                ;
                const mtype = (0, strings_1.$ftrim)(req.message.headers['content-type']).toLowerCase();
                _developerLog(req, 'MIME type', mtype);
                if (!mtype.length) {
                    _developerLog(req, 'bad MIME type');
                    tserrors_1.TSError.throw(`No mime type specifies for body of ${this._method} request on url '${req.url.pathname}'`, tsrequest_1.Resp.BadRequest, {
                        method: this._method,
                        path: (0, commons_1.$length)(req.url.pathname) ? req.url.pathname : '/',
                        serverError: tsserver_types_1.TSServerErrorCodes.MissingMimeType
                    });
                }
                if (mtype.startsWith('text/') || TSServerEndPointManager.__textMimes.includes(mtype)) {
                    _developerLog(req, 'text type', true);
                    body = tscharset_1.TSCharset.utf8Charset().stringFromBytes(body);
                    if (mtype === 'application/json') {
                        _developerLog(req, 'json type', true);
                        options.context = 'json';
                        try {
                            body = JSON.parse(body);
                        }
                        catch (e) {
                            _developerLog(req, 'impossible to decode JSON body');
                            tserrors_1.TSError.throw(`Unable to parse JSON body of ${this._method} request on url '${req.url.pathname}'`, tsrequest_1.Resp.BadRequest, {
                                method: this._method,
                                path: (0, commons_1.$length)(req.url.pathname) ? req.url.pathname : '/',
                                parsingError: e,
                                serverError: tsserver_types_1.TSServerErrorCodes.BadJSONBody
                            });
                        }
                        _developerLog(req, 'json decoded body', body);
                    }
                }
                if (!this._bodyParser.validate(body, options)) {
                    _developerLog(req, 'impossible to validate body');
                    tserrors_1.TSError.throw(`Bad body content for ${this._method} request on url '${req.url.pathname}'`, tsrequest_1.Resp.BadRequest, {
                        method: this._method,
                        path: (0, commons_1.$length)(req.url.pathname) ? req.url.pathname : '/',
                        errors: options.errors,
                        serverError: tsserver_types_1.TSServerErrorCodes.BadBodyStructure
                    });
                }
                req.body = this._bodyParser.rawInterpret(body);
                _developerLog(req, 'final intepreted body', req.body);
            }
            _developerLog(req, 'calling internal controller');
            yield this._controller(req, new tsserver_types_1.TSServerResponse(res, this._responseParser));
            _developerLog(req, 'resumed from internal controller');
        });
    }
}
TSServerEndPointManager.__textMimes = [
    'application/json',
    'application/calendar+xml',
    'application/gml+xml',
    'application/javascript',
    'application/xhtml+xml',
    'application/x-latex'
];
function _defineParser(node, instanceVar, errorOptions) {
    if ((0, commons_1.$ok)(node)) {
        const errors = [];
        const parser = tsparser_1.TSParser.define(node, errors);
        if (!(0, commons_1.$ok)(parser)) {
            tserrors_1.TSError.throw(`TSServerEndPointManager.constructor() : end points path '${errorOptions.path}' has invalid ${instanceVar} definition.`, Object.assign({ parserErrors: errors }, errorOptions));
        }
        return parser;
    }
    return undefined;
}
function _yesNo(v) { return (0, commons_1.$ok)(v) ? true : false; }
function _developerLog(req, item, value = undefined) {
    if (req.developerMode) {
        const is = (0, commons_1.$isstring)(value);
        const ok = (0, commons_1.$ok)(value);
        (0, utils_1.$logterm)(`&0&xfoundation-ts[&pDEBUG&x]-${item}:&0${is ? ' "' : (ok ? '\n' : ' ')}&y${(0, utils_1.$inspect)(value)}${is ? '&x"' : ''}&0`);
    }
}
//# sourceMappingURL=tsserver_endpoints.js.map