/// <reference types="node" />
import * as http from "http";
import { Resp, Verb } from "./tsrequest";
import { Nullable, TSDataLike, TSDictionary } from "./types";
import { TSLeafNode, TSNode, TSParser } from "./tsparser";
import { TSURL } from "./tsurl";
export interface TSServerRequest {
    message: http.IncomingMessage;
    method: Verb;
    url: TSURL;
    parameters: TSDictionary;
    query: TSDictionary;
    body: any;
    developerMode: boolean;
}
export declare class TSServerResponse {
    response: http.ServerResponse;
    responseParser: Nullable<TSParser>;
    constructor(response: http.ServerResponse, responseParser: Nullable<TSParser>);
    setHeader(name: string, value: number | string | ReadonlyArray<string>): this;
    returnData(data: Nullable<TSDataLike>): void;
    returnData(data: Nullable<TSDataLike>, statusOrType: Resp | string): void;
    returnData(data: Nullable<TSDataLike>, type: string, status: Resp): void;
    returnString(s: Nullable<string>): void;
    returnString(s: Nullable<string>, statusOrType: Resp | string): void;
    returnString(s: Nullable<string>, type: string, status: Resp): void;
    returnObject(v: any): void;
    returnObject(v: any, statusOrType: Resp | string): void;
    returnObject(v: any, type: string, status: Resp): void;
    returnEmpty(status?: Resp): void;
    returnError(jsonErrorDescriptionOrStatus: Nullable<object> | Resp): void;
    returnError(jsonErrorDescription: Nullable<object>, status: Resp): void;
}
export type TSEndPointController = (req: TSServerRequest, resp: TSServerResponse) => Promise<void>;
export interface TSEndPoint {
    controller: TSEndPointController;
    query?: {
        [key: string]: TSLeafNode;
    };
    body?: TSNode;
    response?: TSNode;
}
export type TSEndPointsDefinitionDictionary = {
    [key in Verb]?: TSEndPoint | TSEndPointController;
};
export type TSEndpointsDefinition = TSEndPointsDefinitionDictionary | TSEndPoint | TSEndPointController;
export interface TSPreflightResponse {
    allowedOrigin: Nullable<string>;
    allowedMethods: Nullable<string | string[]>;
    allowedHeaders: Nullable<string | string[]>;
    timeout?: Nullable<number>;
}
export type TSPreflightController = (url: TSURL, headers: http.IncomingHttpHeaders, res: http.ServerResponse) => Promise<Nullable<TSPreflightResponse>>;
export declare enum TSServerStartStatus {
    AlreadyRunning = 0,
    HTTP = 1,
    HTTPS = 2
}
export type TSResourceMimeChecker = (extension: Nullable<string>) => Nullable<string>;
export interface TSWebSiteDefinition {
    folder: string;
    mimeChecker?: TSResourceMimeChecker;
    maxCacheSize?: number;
    maxCachedFileSize?: number;
    maxCachedFiles?: number;
    maxBlacklistedFiles?: number;
}
export declare enum TSServerErrorCodes {
    MissingMimeType = 10000,
    UnknownMethod = 10001,
    ResourceNotFound = 10002,
    MethodNotImplemented = 10003,
    InnaccessibleRoot = 10004,
    BadParameter = 10005,
    BadQueryStructure = 10006,
    BadBodyStructure = 10007,
    MissingBody = 10008,
    BadResponseStructure = 10009,
    ForbiddenDataResponse = 10010,
    BadJSONBody = 10011,
    BadEndPointPath = 10012
}
