"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSServerErrorCodes = exports.TSServerStartStatus = exports.TSServerResponse = void 0;
const tsrequest_1 = require("./tsrequest");
const commons_1 = require("./commons");
const data_1 = require("./data");
const tserrors_1 = require("./tserrors");
class TSServerResponse {
    constructor(response, responseParser) {
        this.response = response;
        this.responseParser = responseParser;
    }
    setHeader(name, value) { this.response.setHeader(name, value); return this; }
    returnData() { _returnData(this, arguments); }
    returnString() { _returnString(this, arguments); }
    returnObject() { _returnObject(this, arguments); }
    returnEmpty(status = tsrequest_1.Resp.NoContent) { this.response.writeHead(status); this.response.end(); }
    returnError() { _returnError(this, arguments); }
}
exports.TSServerResponse = TSServerResponse;
var TSServerStartStatus;
(function (TSServerStartStatus) {
    TSServerStartStatus[TSServerStartStatus["AlreadyRunning"] = 0] = "AlreadyRunning";
    TSServerStartStatus[TSServerStartStatus["HTTP"] = 1] = "HTTP";
    TSServerStartStatus[TSServerStartStatus["HTTPS"] = 2] = "HTTPS";
})(TSServerStartStatus = exports.TSServerStartStatus || (exports.TSServerStartStatus = {}));
var TSServerErrorCodes;
(function (TSServerErrorCodes) {
    TSServerErrorCodes[TSServerErrorCodes["MissingMimeType"] = 10000] = "MissingMimeType";
    TSServerErrorCodes[TSServerErrorCodes["UnknownMethod"] = 10001] = "UnknownMethod";
    TSServerErrorCodes[TSServerErrorCodes["ResourceNotFound"] = 10002] = "ResourceNotFound";
    TSServerErrorCodes[TSServerErrorCodes["MethodNotImplemented"] = 10003] = "MethodNotImplemented";
    TSServerErrorCodes[TSServerErrorCodes["InnaccessibleRoot"] = 10004] = "InnaccessibleRoot";
    TSServerErrorCodes[TSServerErrorCodes["BadParameter"] = 10005] = "BadParameter";
    TSServerErrorCodes[TSServerErrorCodes["BadQueryStructure"] = 10006] = "BadQueryStructure";
    TSServerErrorCodes[TSServerErrorCodes["BadBodyStructure"] = 10007] = "BadBodyStructure";
    TSServerErrorCodes[TSServerErrorCodes["MissingBody"] = 10008] = "MissingBody";
    TSServerErrorCodes[TSServerErrorCodes["BadResponseStructure"] = 10009] = "BadResponseStructure";
    TSServerErrorCodes[TSServerErrorCodes["ForbiddenDataResponse"] = 10010] = "ForbiddenDataResponse";
    TSServerErrorCodes[TSServerErrorCodes["BadJSONBody"] = 10011] = "BadJSONBody";
    TSServerErrorCodes[TSServerErrorCodes["BadEndPointPath"] = 10012] = "BadEndPointPath";
})(TSServerErrorCodes = exports.TSServerErrorCodes || (exports.TSServerErrorCodes = {}));
function _returnString(r, args) {
    _statusAndType(r, 'returnString', 'text/plain', args);
    r.response.end((0, commons_1.$value)((0, commons_1.$string)(args[0]), ''));
}
function _returnData(r, args) {
    _statusAndType(r, 'returnData', 'application/octet-stream', args);
    r.response.end(_compatibleData(args[0]));
}
function _returnObject(r, args) {
    _statusAndType(r, 'returnObject', 'application/json', args);
    let v = args[0];
    if ((0, commons_1.$isdataobject)(v)) {
        tserrors_1.TSError.throw('TSServerResponse.returnObject() : Impossible to return data as object response', {
            serverError: TSServerErrorCodes.ForbiddenDataResponse
        });
    }
    if ((0, commons_1.$ok)(r.responseParser)) {
        const opts = { errors: [], context: 'json' };
        if (!r.responseParser.validate(v, opts)) {
            tserrors_1.TSError.throw('TSServerResponse.returnObject(): Invalid structured response', {
                errors: opts.errors,
                serverError: TSServerErrorCodes.BadResponseStructure
            });
        }
        v = r.responseParser.rawEncode(v, opts);
    }
    r.response.end(JSON.stringify(v, undefined, 2));
}
function _returnError(r, args) {
    if (args.length > 2) {
        tserrors_1.TSError.throw('TSServerResponse.returnError() : Bad arguments');
    }
    let v = undefined;
    let status = tsrequest_1.Resp.InternalError;
    switch (args.length) {
        case 0:
            r.returnEmpty(status);
            return;
        case 1:
            if (typeof args[0] === 'number') {
                r.returnEmpty(args[0]);
                return;
            }
            v = args[0];
            break;
        case 2:
            v = args[0];
            status = args[1];
            break;
        default:
            tserrors_1.TSError.throw('TSServerResponse.returnError() : Bad arguments');
    }
    r.response.writeHead(status, { 'Content-Type': 'application/json' });
    r.response.end(JSON.stringify(v, undefined, 2));
}
function _statusAndType(r, fn, defaultType, args) {
    let status = tsrequest_1.Resp.OK;
    let type = defaultType;
    switch (args.length) {
        case 1: break;
        case 2:
            if (typeof args[0] === 'string') {
                type = args[1];
            }
            else {
                status = args[1];
            }
            break;
        case 3:
            type = args[1];
            status = args[2];
            break;
        default:
            tserrors_1.TSError.throw(`TSServerResponse.${fn}() : Bad arguments`);
    }
    r.response.writeHead(status, { 'Content-Type': type });
}
function _compatibleData(data) {
    if (!(0, commons_1.$ok)(data)) {
        return Buffer.from('');
    }
    if (data instanceof Uint8Array || data instanceof ArrayBuffer) {
        return data;
    }
    return (0, data_1.$bufferFromDataLike)(data);
}
//# sourceMappingURL=tsserver_types.js.map