"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSExpectAgent = exports.TSUnaryTest = exports.TSTestDescriptor = exports.TSTestGroup = exports.TSGenericTest = exports.TSTest = exports.TSTester = void 0;
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const tsrange_1 = require("./tsrange");
const tsrangeset_1 = require("./tsrangeset");
const tsinterval_1 = require("./tsinterval");
const types_1 = require("./types");
const utils_1 = require("./utils");
const tslist_1 = require("./tslist");
const strings_1 = require("./strings");
const noopTest = (_) => __awaiter(void 0, void 0, void 0, function* () { });
const PADN = 5;
class TSTester {
    static group(s, f, opts) {
        return new TSTestGroup(s, f, Object.assign(Object.assign({}, TSTester.globalOptions), opts));
    }
    constructor(s) {
        this.groups = [];
        this._names = new Set();
        this.desc = s;
    }
    addGroup(s, f, name, opts) { this.addGroups([TSTester.group(s, f, Object.assign(Object.assign({}, TSTester.globalOptions), opts))], name); }
    addGroups(grps, name) {
        const hasName = (0, commons_1.$length)(name) > 0;
        if (hasName) {
            this._names.add(name);
        }
        if (grps instanceof TSTestGroup) {
            grps = [grps];
        }
        for (let g of grps) {
            this.groups.push(g);
            if (hasName) {
                g.name = name;
            }
            else if ((0, commons_1.$length)(g.name) > 0) {
                this._names.add(g.name);
            }
        }
    }
    log(format, ...args) {
        (0, utils_1.$logterm)(format, args);
    }
    get names() { return Array.from(this._names); }
    containsName(s) { return (0, commons_1.$length)(s) ? this._names.has(s) : false; }
    dumpGroupsList(clearScreen = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const n = (0, commons_1.$count)(this.groups);
            if (n > 0) {
                this.log(`${clearScreen ? '&Z' : ''}&eTSTester will now list &Y&k ${this.desc} &0&e :`);
                let i = 1;
                for (let g of this.groups) {
                    yield g.prepare();
                    (0, utils_1.$writeterm)(`&e${i.fpad4()} `);
                    g.dumpGroupInfo();
                    (0, utils_1.$logterm)('');
                    i++;
                }
            }
            else {
                this.log(`&aTSTester has no test groups defined for &O&w ${this.desc} &0&e !`);
            }
        });
    }
    run(opts = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (opts.listTests) {
                yield this.dumpGroupsList(opts.clearScreen);
                return;
            }
            const start = (0, utils_1.$mark)();
            this.log(`${opts.clearScreen ? '&Z' : ''}&eTSTester will now start &Y&k ${this.desc} &0&e :`);
            let expectations = 0;
            let expectationsFailed = 0;
            let focusNames = (0, commons_1.$count)(opts.focusNames) ? opts.focusNames.filter(fn => this.containsName(fn)) : [];
            for (let g of this.groups) {
                if ((0, commons_1.$length)(g.name) && focusNames.includes(g.name)) {
                    g.focused = true;
                }
                yield g.prepare();
            }
            let groups = this.groups.filter(g => g.focused);
            const n = (0, commons_1.$count)(groups);
            if (n > 0) {
                this.log(`&0&O&r Test will be restricted to ${n} groups out of all ${this.groups.length} &0`);
            }
            else {
                groups = this.groups;
            }
            let passed = 0, failed = 0, silent = 0;
            let stopit = false;
            for (let g of groups) {
                let groupExpected, groupFailed;
                [groupExpected, groupFailed, stopit] = yield g.run();
                if (stopit) {
                    break;
                }
                expectations += groupExpected;
                expectationsFailed += groupFailed;
                if (groupFailed === 0) {
                    if (g.silent) {
                        silent++;
                    }
                    else {
                        passed++;
                    }
                }
                else {
                    failed++;
                }
            }
            if (stopit) {
                this.log('&R&w  TESTS DID STOP ON FIRST TEST FAILED  &0');
                if ((0, commons_1.$isfunction)(opts.stopItCallback)) {
                    opts.stopItCallback(this);
                }
            }
            else {
                this.log(`&y${expectations.toString().padStart(PADN)}&0&e test${expectations === 1 ? 'was' : 's were'} executed in &y${(0, utils_1.$ellapsed)(start)}&0`);
                if (passed > 0) {
                    this.log(`&g${passed.toString().padStart(PADN)}&0&j group${passed === 1 ? ' ' : 's'} of tests did &G&w  PASS  &0`);
                }
                if (silent > 0) {
                    this.log(`&g${silent.toString().padStart(PADN)}&0&j&? group${passed === 1 ? ' ' : 's'} of tests did &J&?&w  PASS IN SILENT MODE  &0`);
                }
                if (failed > 0) {
                    this.log(`&r${failed.toString().padStart(PADN)}&0&o group${failed === 1 ? ' ' : 's'} of tests did &R&w  FAIL  &0`);
                }
                if ((passed > 0 || silent > 0) && !failed) {
                    this.log('&G&w  ALL TESTS PASSED  &0');
                }
                else if (expectationsFailed > 0) {
                    this.log(`&R&w${expectationsFailed.toString().padStart(PADN)} TEST${expectationsFailed > 1 ? 'S' : ''} FAILED  &0`);
                }
            }
        });
    }
}
exports.TSTester = TSTester;
exports.TSTest = TSTester;
TSTester.globalOptions = {};
class TSGenericTest {
    constructor(s, f, opts) {
        this.name = undefined;
        this.focused = false;
        this.silent = false;
        this.stopOnFirstFail = false;
        this.desc = s.doubleEscape('&').normalizeSpaces();
        this.fn = f;
        if (opts === null || opts === void 0 ? void 0 : opts.focus) {
            this.focused = true;
        }
        if (opts === null || opts === void 0 ? void 0 : opts.silent) {
            this.silent = true;
        }
        if (opts === null || opts === void 0 ? void 0 : opts.stopOnFirstFail) {
            this.stopOnFirstFail = true;
        }
        if ((0, commons_1.$length)(opts === null || opts === void 0 ? void 0 : opts.name) > 0) {
            this.name = opts.name;
        }
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () { return [0, 0, false]; });
    }
    log(format, ...args) { (0, utils_1.$logterm)(format, args); }
}
exports.TSGenericTest = TSGenericTest;
class TSTestGroup extends TSGenericTest {
    constructor(s, f, opts) {
        super(s, f, opts);
        this._unaries = [];
        if (opts === null || opts === void 0 ? void 0 : opts.focusGroup) {
            this.focused = true;
        }
    }
    unary(s, f, opts = {}) {
        if (this.silent) {
            opts.silent = true;
        }
        if (this.stopOnFirstFail) {
            opts.stopOnFirstFail = true;
        }
        this._unaries.push(new TSUnaryTest(this, s, f, opts));
    }
    description(str) { if ((0, commons_1.$length)(str) && !this.silent) {
        this._unaries.push(new TSTestDescriptor(str));
    } }
    prepare() {
        return __awaiter(this, void 0, void 0, function* () { yield this.fn(this); });
    }
    log(format, ...args) { super.log('  ' + format, args); }
    dumpGroupInfo() {
        const n = this._unaries.length;
        if (n > 0) {
            const silent = this.silent ? 'silent ' : '';
            const focused = this.focused ? 'focused ' : '';
            const name = (0, commons_1.$length)(this.name) ? `&J&w ${this.name} &0` : '&U&k <Unamed> &0';
            let tests = n === 1 ? `1 test` : `${n} tests`;
            (0, utils_1.$writeterm)(`&u${silent}${focused}group ${this.desc} ${name}&B&w ${tests} &0`);
        }
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const start = (0, utils_1.$mark)();
            let expectations = 0;
            let expectationFailed = 0;
            let stopit = false;
            if (!this.silent) {
                this.log(`&u- Running ${this.silent ? "silent " : ""}group tests &U&k ${this.desc} &0`);
            }
            let unaries = this._unaries.filter(u => u.focused);
            if (unaries.length === 0) {
                unaries = this._unaries;
            }
            for (let u of unaries) {
                let unaryExpected, unaryFailed;
                [unaryExpected, unaryFailed, stopit] = yield u.run();
                expectationFailed += unaryFailed;
                expectations += unaryExpected;
                if (stopit) {
                    break;
                }
            }
            if (!stopit) {
                if (this.silent && expectationFailed > 0) {
                    this.log(`&u- Did run group tests &U&k ${this.desc} &0`);
                }
                if (!this.silent || expectationFailed > 0) {
                    (0, utils_1.$logterm)(`${expectationFailed == 0 ? '&j' : '\n&o'}    ${this.desc} tests ${expectationFailed == 0 ? '&G&w  PASSED  ' : '&R&w  FAILED  '}&0&y in ${(0, utils_1.$ellapsed)(start)}\n`);
                }
            }
            return [expectations, expectationFailed, stopit];
        });
    }
}
exports.TSTestGroup = TSTestGroup;
class TSTestDescriptor extends TSGenericTest {
    constructor(s) { super((0, utils_1.$term)(s), noopTest); }
    log(format, ...args) { super.log('    ' + format, args); }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            (0, utils_1.$logterm)(`&x     ➤ logging &y${this.desc} &0`);
            return [0, 0, false];
        });
    }
}
exports.TSTestDescriptor = TSTestDescriptor;
class TSUnaryTest extends TSGenericTest {
    constructor(g, s, f, opts) {
        super(s, f, opts);
        this._failed = 0;
        this._passed = 0;
        this._expected = 0;
        this._registrations = {};
        this.group = g;
        if (opts === null || opts === void 0 ? void 0 : opts.focusGroup) {
            g.focused = true;
        }
    }
    description(str) { this.group.description(str); }
    log(format, ...args) { super.log('    ' + format, args); }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this._passed = 0;
            this._failed = 0;
            if (!this.silent) {
                (0, utils_1.$writeterm)(`&x     ➤ &ltesting &B&w ${this.desc} &0`);
            }
            yield this.fn(this);
            if (!this.silent) {
                if (this._failed > 0) {
                    (0, utils_1.$writeterm)('\n     ');
                    this.printRegistrations();
                }
                if (this._passed > 0) {
                    (0, utils_1.$writeterm)(`&J&k ${this._passed} OK &0`);
                }
                if (this._failed > 0) {
                    (0, utils_1.$writeterm)(`&R&w ${this._failed} KO &0`);
                }
                (0, utils_1.$logterm)('');
            }
            else if (this._failed > 0) {
                (0, utils_1.$logterm)(`&x     ➤ &ldid execute test &B&w ${this.desc} &0&R&w ${this._failed} KO &0`);
            }
            return [this._expected, this._failed, this.stopOnFirstFail && this._failed > 0];
        });
    }
    register(name, value) { if ((0, commons_1.$defined)(value) && (0, commons_1.$length)(name)) {
        this._registrations[name] = value;
    } }
    printRegistrations() {
        const keys = (0, commons_1.$keys)(this._registrations);
        const n = keys.length;
        if (n === 1) {
            (0, utils_1.$logterm)(`\n&Y&b  REGISTERED ITEM &r&!${keys[0]}  &0&c\n${(0, utils_1.$inspect)(this._registrations[keys[0]])}&0\n`);
        }
        else if (n > 1) {
            (0, utils_1.$logterm)('\n&Y&b  REGISTERED ITEMS  &0');
            keys.forEach(k => { (0, utils_1.$logterm)(`&x  + &o&_&/&!${k}&0&x: &c${(0, utils_1.$inspect)(this._registrations[k])}&0`); });
            (0, utils_1.$logterm)('');
        }
    }
    expect(v, msg) {
        this._expected++;
        const len = (0, commons_1.$length)(msg);
        if (!len) {
            msg = this._expected.toString().padStart(4);
        }
        else if (len > 6) {
            msg = (0, strings_1.$left)(msg, 6);
        }
        return new TSExpectAgent(this, v, msg.toLowerCase());
    }
    expect0(v) { return this.expect(v, 'expct0'); }
    expect1(v) { return this.expect(v, 'expct1'); }
    expect2(v) { return this.expect(v, 'expct2'); }
    expect3(v) { return this.expect(v, 'expct3'); }
    expect4(v) { return this.expect(v, 'expct4'); }
    expect5(v) { return this.expect(v, 'expct5'); }
    expect6(v) { return this.expect(v, 'expct6'); }
    expect7(v) { return this.expect(v, 'expct7'); }
    expect8(v) { return this.expect(v, 'expct8'); }
    expect9(v) { return this.expect(v, 'expct9'); }
    expectA(v) { return this.expect(v, 'expctA'); }
    expectB(v) { return this.expect(v, 'expctB'); }
    expectC(v) { return this.expect(v, 'expctC'); }
    expectD(v) { return this.expect(v, 'expctD'); }
    expectE(v) { return this.expect(v, 'expctE'); }
    expectF(v) { return this.expect(v, 'expctF'); }
    expectG(v) { return this.expect(v, 'expctG'); }
    expectH(v) { return this.expect(v, 'expctH'); }
    expectI(v) { return this.expect(v, 'expctI'); }
    expectJ(v) { return this.expect(v, 'expctJ'); }
    expectK(v) { return this.expect(v, 'expctK'); }
    expectL(v) { return this.expect(v, 'expctL'); }
    expectM(v) { return this.expect(v, 'expctM'); }
    expectN(v) { return this.expect(v, 'expctN'); }
    expectO(v) { return this.expect(v, 'expctO'); }
    expectP(v) { return this.expect(v, 'expctP'); }
    expectQ(v) { return this.expect(v, 'expctQ'); }
    expectR(v) { return this.expect(v, 'expctR'); }
    expectS(v) { return this.expect(v, 'expctS'); }
    expectT(v) { return this.expect(v, 'expctT'); }
    expectU(v) { return this.expect(v, 'expctU'); }
    expectV(v) { return this.expect(v, 'expctV'); }
    expectW(v) { return this.expect(v, 'expctW'); }
    expectX(v) { return this.expect(v, 'expctX'); }
    expectY(v) { return this.expect(v, 'expctY'); }
    expectZ(v) { return this.expect(v, 'expctZ'); }
    fail() { this._failed++; return false; }
    pass() { this._passed++; return true; }
}
exports.TSUnaryTest = TSUnaryTest;
class TSExpectAgent {
    constructor(step, value, msg = '') {
        this.eq = this.toBe;
        this.neq = this.notToBe;
        this.is = this.toBe;
        this.isnot = this.notToBe;
        this.OK = this.toBeOK;
        this.KO = this.toBeNotOK;
        this.def = this.toBeDefined;
        this.undef = this.toBeUndefined;
        this.notnull = this.toBeNotNull;
        this.null = this.toBeNull;
        this.empty = this.toBeEmpty;
        this.filled = this.toBeNotEmpty;
        this.true = this.toBeTruthy;
        this.false = this.toBeFalsy;
        this._step = step;
        this._value = value;
        this._message = msg;
    }
    toBe(aValue) { var _a; return !(0, compare_1.$equal)(aValue, this._value) ? this._elogfail(aValue) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    notToBe(aValue) { var _a; return (0, compare_1.$equal)(aValue, this._value) ? this._nelogfail(aValue) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeTruthy() { var _a; return !this._value ? this._elogfail(true) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeFalsy() { var _a; return !!this._value ? this._elogfail(false) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeUndefined() { var _a; return (0, commons_1.$defined)(this._value) ? this._elogfail(undefined) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeDefined() { var _a; return !(0, commons_1.$defined)(this._value) ? this._nelogfail(undefined) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeNull() { var _a; return this._value !== null ? this._elogfail(null) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeNotNull() { var _a; return this._value === null ? this._nelogfail(null) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeOK() { var _a; return !(0, commons_1.$ok)(this._value) ? this._elogfail('<a non null defined value>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeNotOK() { var _a; return (0, commons_1.$ok)(this._value) ? this._elogfail('<a null or undefined value>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeNaN() { var _a; return !isNaN(this._value) ? this._elogfail(NaN) : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeANumber() { var _a; return !(0, commons_1.$isnumber)(this._value) ? this._elogfail('<a valid number>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAnInt() { var _a; return !(0, commons_1.$isint)(this._value) ? this._elogfail('<an integer>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAnUnsigned() { var _a; return !(0, commons_1.$isunsigned)(this._value) ? this._elogfail('<an unsigned>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAString() { var _a; return !(0, commons_1.$isstring)(this._value) ? this._elogfail('<a string>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeBool() { var _a; return !(0, commons_1.$isbool)(this._value) ? this._elogfail('<a boolean>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAnEmail() { var _a; return !(0, commons_1.$isemail)(this._value) ? this._elogfail('<an email>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAnUrl() { var _a; return !(0, commons_1.$isurl)(this._value) ? this._elogfail('<an url>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAnUUID() { var _a; return !(0, commons_1.$isuuid)(this._value) ? this._elogfail('<an UUID>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAnObject() { var _a; return !(0, commons_1.$isobject)(this._value) ? this._elogfail('<an object>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeArray() { var _a; return !(0, commons_1.$isarray)(this._value) ? this._elogfail('<an array>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeADate() { var _a; return !(0, commons_1.$isdate)(this._value) ? this._elogfail('<a date>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeAFunction() { var _a; return !(0, commons_1.$isfunction)(this._value) ? this._elogfail('<a function>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    toBeEmpty() {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        if (this._value instanceof Set) {
            return this._value.size > 0 ? this._elogfail('<an empty Set>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
        }
        if (this._value instanceof Map) {
            return this._value.size > 0 ? this._elogfail('<an empty Map>') : (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
        }
        if ((0, commons_1.$isarray)(this._value)) {
            return (0, commons_1.$count)(this._value) > 0 ? this._elogfail('<an empty Array>') : (_c = this._step) === null || _c === void 0 ? void 0 : _c.pass();
        }
        if ((0, commons_1.$isstring)(this._value)) {
            return (0, commons_1.$length)(this._value) > 0 ? this._elogfail('<an empty String>') : (_d = this._step) === null || _d === void 0 ? void 0 : _d.pass();
        }
        if ((0, commons_1.$isdataobject)(this._value)) {
            return (0, commons_1.$length)(this._value) > 0 ? this._elogfail('<an empty buffer>') : (_e = this._step) === null || _e === void 0 ? void 0 : _e.pass();
        }
        if (this._value instanceof tslist_1.TSList) {
            return this._value.count > 0 ? this._elogfail('<an empty List>') : (_f = this._step) === null || _f === void 0 ? void 0 : _f.pass();
        }
        if (this._value instanceof tsrange_1.TSRange || this._value instanceof tsrangeset_1.TSRangeSet || this._value instanceof tsinterval_1.TSInterval) {
            return !this._value.isEmpty ? this._elogfail('<an empty interval>') : (_g = this._step) === null || _g === void 0 ? void 0 : _g.pass();
        }
        return (0, commons_1.$ok)(this._value) ? this._elogfail('<not a container>') : (_h = this._step) === null || _h === void 0 ? void 0 : _h.pass();
    }
    toBeNotEmpty() {
        var _a, _b, _c, _d, _e, _f, _g;
        if (this._value instanceof Set) {
            return this._value.size === 0 ? this._elogfail('<a significant Set>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass();
        }
        if (this._value instanceof Map) {
            return this._value.size === 0 ? this._elogfail('<a significant Map>') : (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
        }
        if ((0, commons_1.$isarray)(this._value)) {
            return (0, commons_1.$count)(this._value) === 0 ? this._elogfail('<a significant Array>') : (_c = this._step) === null || _c === void 0 ? void 0 : _c.pass();
        }
        if ((0, commons_1.$isstring)(this._value)) {
            return (0, commons_1.$length)(this._value) === 0 ? this._elogfail('<a significant String>') : (_d = this._step) === null || _d === void 0 ? void 0 : _d.pass();
        }
        if ((0, commons_1.$isdataobject)(this._value)) {
            return (0, commons_1.$length)(this._value) === 0 ? this._elogfail('<a significant buffer>') : (_e = this._step) === null || _e === void 0 ? void 0 : _e.pass();
        }
        if (this._value instanceof tslist_1.TSList) {
            return this._value.count === 0 ? this._elogfail('<a significant List>') : (_f = this._step) === null || _f === void 0 ? void 0 : _f.pass();
        }
        if (this._value instanceof tsrange_1.TSRange || this._value instanceof tsrangeset_1.TSRangeSet || this._value instanceof tsinterval_1.TSInterval) {
            return this._value.isEmpty ? this._elogfail('<a significant interval>') : (_g = this._step) === null || _g === void 0 ? void 0 : _g.pass();
        }
        return this._elogfail((0, commons_1.$ok)(this._value) ? '<not a container>' : 'null or undefined value');
    }
    toBeUnordered(aValue) { return !(0, commons_1.$ok)(this._value) || !(0, commons_1.$ok)(aValue) || !(0, compare_1.$unorderedEqual)(this._value, aValue) ? this._elogfail(aValue) : this._step.pass(); }
    gt(aValue) { var _a; return (0, compare_1.$compare)(aValue, this._value) !== types_1.Ascending ? this._compfail(aValue, '>') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    gte(aValue) { var _a; return (0, compare_1.$compare)(aValue, this._value) === types_1.Descending ? this._compfail(aValue, '≥') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    lt(aValue) { var _a; return (0, compare_1.$compare)(aValue, this._value) !== types_1.Descending ? this._compfail(aValue, '<') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    lte(aValue) { var _a; return (0, compare_1.$compare)(aValue, this._value) === types_1.Ascending ? this._compfail(aValue, '≤') : (_a = this._step) === null || _a === void 0 ? void 0 : _a.pass(); }
    _compfail(aValue, op) {
        var _a;
        const start = this._writeMessage();
        (0, utils_1.$logterm)(`&adid expect value:&O&w${(0, utils_1.$inspect)(this._value).replace(/&/g, '&&')}&0`);
        (0, utils_1.$logterm)(`${start}&eto be ${op} to value:&E&b${(0, utils_1.$inspect)(aValue)}&0`);
        return (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    _elogfail(aValue) {
        var _a;
        const start = this._writeMessage();
        (0, utils_1.$logterm)(`&edid expect value:&E&b${(0, utils_1.$inspect)(aValue).replace(/&/g, '&&')}&0`);
        (0, utils_1.$logterm)(`${start}&adid get as value:&O&w${(0, utils_1.$inspect)(this._value).replace(/&/g, '&&')}&0`);
        return (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    _nelogfail(aValue) {
        var _a;
        this._writeMessage();
        (0, utils_1.$logterm)(`&adid not expect  :&O&w${(0, utils_1.$inspect)(aValue).replace(/&/g, '&&')}&0`);
        return (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    _writeMessage() {
        const l = this._message.length;
        (0, utils_1.$writeterm)(`\n&y${this._message}&0 `);
        return l > 0 ? "".padStart(l + 1, " ") : "";
    }
}
exports.TSExpectAgent = TSExpectAgent;
//# sourceMappingURL=tstester.js.map