import { Comparison, Nullable, url } from "./types";
import { TSClone, TSObject } from "./tsobject";
export interface TSURLParseOptions {
    acceptedProtocols?: Nullable<string[]>;
    refusesParameters?: Nullable<boolean>;
    throwsError?: Nullable<boolean>;
}
export declare class TSURL implements TSObject, TSClone<TSURL> {
    static HostMaxLength: number;
    static AutoEscapeChars: string[];
    static StandardURLProtocols: Set<string>;
    static AutoEscapeCharSet: Set<string>;
    static EndhostCharSet: Set<string>;
    private _auth;
    private _hash;
    private _hostname;
    private _pathname;
    private _port;
    private _protocol;
    private _search;
    private _w3cProtocol;
    private _ipv6;
    private _acceptedProtocols;
    private _searchParams;
    private _href;
    static from(source: URL, options?: Nullable<TSURLParseOptions>): TSURL | null;
    static compose(origin: Nullable<string>, path: string, options?: Nullable<TSURLParseOptions>): TSURL | null;
    static url(source: Nullable<string>, options?: Nullable<TSURLParseOptions>): TSURL | null;
    private constructor();
    get href(): url;
    get w3c(): boolean;
    get origin(): string;
    get host(): string;
    get searchParams(): URLSearchParams;
    get auth(): string;
    get hash(): string;
    get search(): string;
    get pathname(): string;
    set pathname(s: string);
    get hostname(): string;
    get port(): string;
    set port(p: string);
    get protocol(): string;
    set protocol(s: string);
    isEqual(other: any): boolean;
    compare(other: any): Comparison;
    toString(): string;
    toJSON: () => string;
    toArray(): TSURL[];
    clone(): TSURL;
    private _calculated_href;
}
