"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.$stack = exports.$logheader = exports.$hexadump = exports.$writeterm = exports.$logterm = exports.$termclean = exports.$term = exports.$insp = exports.$inspect = exports.$readStreamBuffer = exports.$timeout = exports.$sleep = exports.$ellapsed = exports.$mark = exports.$exit = exports.$inbrowser = exports.$noop = void 0;
const util_1 = require("util");
const commons_1 = require("./commons");
const strings_1 = require("./strings");
const string_tables_1 = require("./string_tables");
const data_1 = require("./data");
const number_1 = require("./number");
const tserrors_1 = require("./tserrors");
const $noop = () => { };
exports.$noop = $noop;
function $inbrowser() {
    if (!(0, commons_1.$defined)($inbrowser.flag)) {
        let inb = false;
        try {
            inb = (0, commons_1.$defined)(navigator) || !(0, commons_1.$isproperty)(process, 'stdout') || !(0, commons_1.$ismethod)(process === null || process === void 0 ? void 0 : process.stdout, 'write');
        }
        catch (_a) {
            inb = false;
        }
        $inbrowser.flag = inb;
    }
    return $inbrowser.flag;
}
exports.$inbrowser = $inbrowser;
function $exit(errorCode) {
    if (!(0, commons_1.$isint)(errorCode)) {
        errorCode = 0;
    }
    if (!$inbrowser() && process && typeof (process === null || process === void 0 ? void 0 : process.exit) === 'function') {
        process.exit(errorCode);
    }
    tserrors_1.TSError.throw(`Premature exit with code ${errorCode}`, errorCode);
}
exports.$exit = $exit;
function $mark() {
    var _a;
    if (!$inbrowser() && process && typeof (process === null || process === void 0 ? void 0 : process.hrtime) === 'function') {
        const [seconds, nanoSeconds] = process.hrtime();
        return seconds + nanoSeconds / 1000000000.0;
    }
    else if (typeof ((_a = window === null || window === void 0 ? void 0 : window.performance) === null || _a === void 0 ? void 0 : _a.now) === 'function') {
        return window.performance.now() / 1000.0;
    }
    else {
        return Date.now() / 1000.0;
    }
}
exports.$mark = $mark;
function $ellapsed(previousMark) {
    return (0, number_1.$unit)($mark() - previousMark, {
        unitName: "second",
        unit: "s",
        decimalPlaces: 3,
        minimalUnit: -2,
        ignoreZeroDecimals: true,
        ignoreMinimalUnitDecimals: true
    });
}
exports.$ellapsed = $ellapsed;
function $sleep(ms) {
    return __awaiter(this, void 0, void 0, function* () { new Promise(res => setTimeout(res, ms)); });
}
exports.$sleep = $sleep;
function $timeout(promise, time, exception) {
    let timer;
    return Promise.race([
        promise,
        new Promise((_, rejection) => timer = setTimeout(rejection, time, exception))
    ]).finally(() => clearTimeout(timer));
}
exports.$timeout = $timeout;
function $readStreamBuffer(stream) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!(0, commons_1.$ok)(stream)) {
            return null;
        }
        return new Promise((resolve, reject) => {
            let chunks = [];
            stream.on("data", (chunk) => chunks.push(chunk));
            stream.on("end", () => resolve(Buffer.concat(chunks)));
            stream.on("error", _ => reject(null));
        });
    });
}
exports.$readStreamBuffer = $readStreamBuffer;
function $inspect(v, level) { return $inbrowser() ? _tsinspect(v, level) : _nodeInspect(v, level); }
exports.$inspect = $inspect;
function $insp(v, level) { return _tsinspect(v, level); }
exports.$insp = $insp;
function $term(s, escapeChar = '&') {
    if ($inbrowser()) {
        return $termclean(s, escapeChar);
    }
    let fmtlen = (0, commons_1.$length)(s);
    let ret = "";
    if (fmtlen) {
        let escape = false;
        if ((0, commons_1.$length)(escapeChar) !== 1 || escapeChar == '\x1b') {
            escapeChar = '&';
        }
        for (let i = 0; i < fmtlen; i++) {
            const c = s.charAt(i);
            if (escape) {
                escape = false;
                switch (c) {
                    case escapeChar:
                        ret += escapeChar;
                        break;
                    case '0':
                        ret += "\x1b[0m";
                        break;
                    case '1':
                        ret += "\x1b[1m";
                        break;
                    case '!':
                        ret += "\x1b[1m";
                        break;
                    case '>':
                        ret += "\x1b[2m";
                        break;
                    case '?':
                        ret += "\x1b[2m";
                        break;
                    case '/':
                        ret += "\x1b[3m";
                        break;
                    case '_':
                        ret += "\x1b[4m";
                        break;
                    case '%':
                        ret += "\x1b[5m";
                        break;
                    case '<':
                        ret += "\x1b[7m";
                        break;
                    case '-':
                        ret += "\x1b[9m";
                        break;
                    case 'h':
                        ret += "\x1b[0G";
                        break;
                    case 'H':
                        ret += "\x1b[H";
                        break;
                    case 'z':
                        ret += '\x1b[2K\x1b[0G';
                        break;
                    case 'Z':
                        ret += '\x1b[2J\x1b[H';
                        break;
                    case 'a':
                        ret += "\x1b[38;5;216m";
                        break;
                    case 'A':
                        ret += "\x1b[48;5;216m";
                        break;
                    case 'b':
                        ret += "\x1b[34m";
                        break;
                    case 'B':
                        ret += "\x1b[44m";
                        break;
                    case 'c':
                        ret += "\x1b[36m";
                        break;
                    case 'C':
                        ret += "\x1b[46m";
                        break;
                    case 'd':
                        ret += "\x1b[38;5;238m";
                        break;
                    case 'D':
                        ret += "\x1b[48;5;238m";
                        break;
                    case 'e':
                        ret += "\x1b[38;5;229m";
                        break;
                    case 'E':
                        ret += "\x1b[48;5;229m";
                        break;
                    case 'g':
                        ret += "\x1b[32m";
                        break;
                    case 'G':
                        ret += "\x1b[42m";
                        break;
                    case 'j':
                        ret += "\x1b[38;5;121m";
                        break;
                    case 'J':
                        ret += "\x1b[48;5;121m";
                        break;
                    case 'k':
                        ret += "\x1b[30m";
                        break;
                    case 'K':
                        ret += "\x1b[40m";
                        break;
                    case 'l':
                        ret += "\x1b[38;5;252m";
                        break;
                    case 'L':
                        ret += "\x1b[48;5;252m";
                        break;
                    case 'm':
                        ret += "\x1b[35m";
                        break;
                    case 'M':
                        ret += "\x1b[45m";
                        break;
                    case 'o':
                        ret += "\x1b[38;5;208m";
                        break;
                    case 'O':
                        ret += "\x1b[48;5;208m";
                        break;
                    case 'p':
                        ret += "\x1b[38;5;212m";
                        break;
                    case 'P':
                        ret += "\x1b[48;5;212m";
                        break;
                    case 'r':
                        ret += "\x1b[31m";
                        break;
                    case 'R':
                        ret += "\x1b[41m";
                        break;
                    case 'u':
                        ret += "\x1b[38;5;117m";
                        break;
                    case 'U':
                        ret += "\x1b[48;5;117m";
                        break;
                    case 'v':
                        ret += "\x1b[38;5;99m";
                        break;
                    case 'V':
                        ret += "\x1b[48;5;99m";
                        break;
                    case 'w':
                        ret += "\x1b[37m";
                        break;
                    case 'W':
                        ret += "\x1b[47m";
                        break;
                    case 'x':
                        ret += "\x1b[38;5;244m";
                        break;
                    case 'X':
                        ret += "\x1b[48;5;244m";
                        break;
                    case 'y':
                        ret += "\x1b[33m";
                        break;
                    case 'Y':
                        ret += "\x1b[43m";
                        break;
                    default:
                        ret += escapeChar;
                        ret += c;
                        break;
                }
            }
            else if (c === escapeChar) {
                escape = true;
            }
            else {
                ret += c;
            }
        }
        if (escape) {
            ret += escapeChar;
        }
    }
    return ret;
}
exports.$term = $term;
function $termclean(s, escapeChar = '&') {
    let len = (0, commons_1.$length)(s);
    let ret = "";
    if (len) {
        let State;
        (function (State) {
            State[State["Standard"] = 0] = "Standard";
            State[State["EscapeChar"] = 1] = "EscapeChar";
            State[State["EscapeEscape"] = 2] = "EscapeEscape";
        })(State || (State = {}));
        ;
        let state = State.Standard;
        let i = 0;
        let escapeSequenceStart = 0;
        if ((0, commons_1.$length)(escapeChar) !== 1 || escapeChar == '\x1b') {
            escapeChar = '&';
        }
        while (i < len) {
            const c = s.charAt(i);
            switch (state) {
                case State.Standard:
                    if (c === escapeChar) {
                        state = State.EscapeChar;
                    }
                    else if (c === '\x1b') {
                        state = State.EscapeEscape;
                        escapeSequenceStart = i;
                    }
                    else {
                        ret += c;
                    }
                    break;
                case State.EscapeEscape:
                    if ('mGHJK'.includes(c)) {
                        state = State.Standard;
                    }
                    else if (!"[0123456789;".includes(c)) {
                        state = State.Standard;
                        ret += '\x1b';
                        i = escapeSequenceStart;
                    }
                    break;
                case State.EscapeChar:
                    if (c === escapeChar) {
                        ret += escapeChar;
                    }
                    else if (!"01>/_%<-?!aAbBcCdDeEgGhHjJkKlLmMoOpPrRuUvVwWxXyYzZ".includes(c)) {
                        ret += escapeChar;
                        i--;
                    }
                    state = State.Standard;
            }
            i++;
        }
    }
    return ret;
}
exports.$termclean = $termclean;
function $logterm(format, ...args) {
    return _logwriteterm(true, format, args);
}
exports.$logterm = $logterm;
function $writeterm(format, ...args) {
    return _logwriteterm(false, format, args);
}
exports.$writeterm = $writeterm;
function $hexadump(source) {
    let data = (0, commons_1.$ok)(source) ? (0, data_1.$bytesFromDataLike)(source) : [];
    let sourceLen = data.length;
    let len = sourceLen % 16 > 0 ? sourceLen + 16 - sourceLen % 16 : sourceLen;
    $logterm("&0&x         00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F chars&0");
    $logterm("&0&x---------------------------------------------------------------------------&0");
    function _H(x) { return (x < sourceLen ? data[x] : 0).toHex2(); }
    function _B(x) { return x < sourceLen ? (data[x] === 38 ? String.fromCharCode(0xff08) : (data[x] >= 32 && data[x] < 128 ? String.fromCharCode(data[x]) : '&B⋅&G')) : ''; }
    for (let p = 0; p < len; p += 16) {
        $logterm(`&0&j${p.toHex8()} ${_H(p)} ${_H(p + 1)} ${_H(p + 2)} ${_H(p + 3)} ${_H(p + 4)} ${_H(p + 5)} ${_H(p + 6)} ${_H(p + 7)} ${_H(p + 8)} ${_H(p + 9)} ${_H(p + 10)} ${_H(p + 11)} ${_H(p + 12)} ${_H(p + 13)} ${_H(p + 14)} ${_H(p + 15)} &w&G&w ${_B(p)}${_B(p + 1)}${_B(p + 2)}${_B(p + 3)}${_B(p + 4)}${_B(p + 5)}${_B(p + 6)}${_B(p + 7)}${_B(p + 8)}${_B(p + 9)}${_B(p + 10)}${_B(p + 11)}${_B(p + 12)}${_B(p + 13)}${_B(p + 14)}${_B(p + 15)} &0`);
    }
    $logterm("&0&x---------------------------------------------------------------------------&0");
}
exports.$hexadump = $hexadump;
function $logheader(s, width = 0, style = '&w', starStyle = '&x') {
    if (!width) {
        width = s.length;
    }
    $logterm("\n&0" + starStyle + "".padEnd(width + 4, '*'));
    $logterm(starStyle + '* ' + style + s.padEnd(width + 1, ' ') + '&0' + starStyle + '*');
    $logterm(starStyle + "".padEnd(width + 4, '*') + "&0\n");
}
exports.$logheader = $logheader;
function $stack() {
    const previousPrepareStackTrace = Error.prepareStackTrace;
    Error.prepareStackTrace = (_, stack) => stack;
    const stack = new Error().stack;
    Error.prepareStackTrace = previousPrepareStackTrace;
    return stack;
}
exports.$stack = $stack;
function _nodeInspect(v, level = 10) { return (0, util_1.inspect)(v, false, level); }
class _tsInspectNode {
    constructor(level) {
        this.left = '{';
        this.right = '}';
        this.sepa = ': ';
        this.lineWidth = 70;
        this.level = level;
    }
    toString() {
        const prefix = _tsInspectNode.__prefixString.slice(0, 2 * this.level);
        let s = (0, commons_1.$defined)(this.id) ? `<ref *${this.id}> ` : '';
        if ((0, commons_1.$length)(this.type)) {
            s += this.type + ' ';
        }
        s += this.left;
        let continued = false;
        for (let e of this.value) {
            if (continued) {
                s += ',';
            }
            s += `\n${prefix}  ${e.key}${this.sepa}` + ((0, commons_1.$isstring)(e.value) ? e.value : e.value.toString());
            continued = true;
        }
        s += '\n' + prefix + this.right;
        const s2 = (0, strings_1.$normspaces)(s);
        return s2.length < this.lineWidth ? s2 : s;
    }
}
_tsInspectNode.__prefixString = "                                                                                                                                      ";
class _tsArrayNode extends _tsInspectNode {
    constructor(level) {
        super(level);
        this.right = ']';
        this.left = '[';
    }
    toString() {
        const plen = 2 * this.level;
        const prefix = _tsInspectNode.__prefixString.slice(0, plen);
        let s = (0, commons_1.$defined)(this.id) ? `<ref *${this.id}> ` : '';
        if ((0, commons_1.$length)(this.type)) {
            s += this.type + ' ';
        }
        s += this.left;
        let continued = false;
        for (let o of this.value) {
            if (continued) {
                s += ',';
            }
            s += '\n' + prefix + '  ' + ((0, commons_1.$isstring)(o) ? o : o.toString());
            continued = true;
        }
        s += '\n' + prefix + this.right;
        const s2 = (0, strings_1.$normspaces)(s);
        return s2.length < this.lineWidth ? s2 : s;
    }
}
class _tsSetNode extends _tsArrayNode {
    constructor(level) {
        super(level);
        this.right = '}';
        this.left = '{';
    }
}
function _tsinspect(v, level = 10) {
    let inspectContext = {
        nextId: 1,
        max: Math.min((0, commons_1.$unsigned)(level), 63) + 1,
        map: new Map(),
    };
    function _recursiveInspect(v, context, level) {
        const t = typeof v;
        switch (t) {
            case 'string': {
                const js = JSON.stringify(v);
                return "'" + js.slice(1, js.length - 1) + "'";
            }
            case 'undefined': return 'undefined';
            case 'symbol':
            case 'boolean':
            case 'bigint':
            case 'number': return `${v}`;
            case 'function': return `[Function: ${v.name}]`;
            case 'object': {
                if (v === null) {
                    return 'null';
                }
                else if ((0, commons_1.$isfunction)(v.leafInspect)) {
                    return v.leafInspect();
                }
                let circularReference = context.map.get(v);
                if ((0, commons_1.$defined)(circularReference)) {
                    if (!(0, commons_1.$defined)(circularReference.id)) {
                        circularReference.id = context.nextId;
                        context.nextId = context.nextId + 1;
                    }
                    return `[Circular *${circularReference.id}]`;
                }
                const isSet = v instanceof Set;
                if (isSet || (0, commons_1.$isarray)(v)) {
                    if (level === context.max) {
                        return isSet ? '[Set]' : '[Array]';
                    }
                    if (isSet) {
                        v = Array.from(v);
                        circularReference = new _tsSetNode(level);
                        circularReference.type = `Set(${v.length})`;
                    }
                    else {
                        circularReference = new _tsArrayNode(level);
                    }
                    context.map.set(v, circularReference);
                    circularReference.value = v.map(e => _recursiveInspect(e, context, level + 1));
                }
                else {
                    if (level === context.max) {
                        return '[Object]';
                    }
                    circularReference = new _tsInspectNode(level);
                    const isMap = v instanceof Map;
                    if (isMap) {
                        circularReference.sepa = ' => ';
                        circularReference.type = `Map(${v.size})`;
                    }
                    else {
                        const n = v.constructor.name;
                        if (n !== 'Object') {
                            circularReference.type = n;
                        }
                    }
                    context.map.set(v, circularReference);
                    const entries = (0, commons_1.$isfunction)(v.entries) ? v.entries() : Object.entries(v);
                    let allEntries = [];
                    for (let [k, o] of entries) {
                        let value = _recursiveInspect(o, context, level + 1);
                        allEntries.push({ key: (isMap ? `'${k}'` : k), value: value });
                    }
                    circularReference.value = allEntries;
                }
                context.map.delete(v);
                return circularReference;
            }
        }
    }
    return _recursiveInspect(v, inspectContext, 0).toString();
}
const FoundationHTMLEncodingExtended = string_tables_1.FoundationHTMLEncoding;
FoundationHTMLEncodingExtended[10] = "<br/>";
FoundationHTMLEncodingExtended[11] = "<br/>";
FoundationHTMLEncodingExtended[12] = "<br/>";
FoundationHTMLEncodingExtended[13] = "<br/>";
function _logwriteterm(log, format, args) {
    format = (0, commons_1.$length)(format) ? $term(format) : '';
    if ($inbrowser()) {
        const c = (0, commons_1.$isfunction)(document === null || document === void 0 ? void 0 : document.getElementById) ? document.getElementById("ftsconsole") : undefined;
        if ((0, commons_1.$ok)(c)) {
            format = (0, strings_1.$HTML)(format, FoundationHTMLEncodingExtended);
            for (let a of args) {
                format += (0, strings_1.$HTML)(a.toString(), FoundationHTMLEncodingExtended);
            }
            let content = c.innerHTML;
            content += format;
            if (log) {
                content += '<br/>';
            }
            c.innerHTML = content;
        }
        else {
            if ((0, commons_1.$count)(args)) {
                format += args.join('');
            }
            console.log(format);
        }
    }
    else {
        if ((0, commons_1.$count)(args)) {
            format += args.join('');
        }
        if (log) {
            console.log(format);
        }
        else {
            process.stdout.write(format);
        }
    }
}
//# sourceMappingURL=utils.js.map